Package: BgeeDB
Type: Package
Title: Annotation and gene expression data retrieval from Bgee
        database. TopAnat, an anatomical entities Enrichment Analysis
        tool for UBERON ontology
Version: 2.38.1
Date: 2024-08-20
Author: Andrea Komljenovic [aut, cre], Julien Roux [aut, cre]
Maintainer: Julien Wollbrett <julien.wollbrett@unil.ch>, Julien Roux <julien.roux@unibas.ch>, Andrea Komljenovic  <andreakomljenovic@gmail.com>, Frederic Bastian <bgee@sib.swiss>
Description: A package for the annotation and gene expression
    data download from Bgee database, and TopAnat analysis: GO-like enrichment of anatomical terms,
    mapped to genes by expression patterns.
Encoding: UTF-8
Depends: R (>= 3.6.0), topGO, tidyr
Imports: R.utils, data.table, curl, RCurl, digest, methods, stats,
        utils, dplyr, RSQLite, graph, Biobase, zellkonverter, anndata,
        HDF5Array, bread
License: GPL-3 + file LICENSE
Roxygen: list(markdown = TRUE)
URL: https://github.com/BgeeDB/BgeeDB_R
BugReports: https://github.com/BgeeDB/BgeeDB_R/issues
VignetteBuilder: knitr
biocViews: Software, DataImport, Sequencing, GeneExpression,
        Microarray, GO, GeneSetEnrichment
Suggests: knitr, BiocStyle, testthat, rmarkdown, markdown
LazyLoad: yes
RoxygenNote: 7.3.2
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/BgeeDB
git_branch: RELEASE_3_23
git_last_commit: a460555
git_last_commit_date: 2026-04-29
Repository: Bioconductor 3.23
Date/Publication: 2026-04-29
Packaged: 2026-04-29 20:40:56 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(Bgee)
export(deleteLocalData)
export(deleteOldData)
export(formatData)
export(getAnnotation)
export(getCellProcessedData)
export(getData)
export(getIntegratedCalls)
export(getSampleProcessedData)
export(listBgeeRelease)
export(listBgeeSpecies)
export(loadTopAnatData)
export(makeTable)
export(topAnat)
exportClasses(Bgee)
import(Biobase)
import(HDF5Array)
import(RCurl)
import(RSQLite)
import(anndata)
import(bread)
import(curl)
import(data.table)
import(digest)
import(graph)
import(methods)
import(stats)
import(topGO)
import(utils)
import(zellkonverter)
importFrom(R.utils,gunzip)
importFrom(dplyr,"%>%")
importFrom(tidyr,spread_)

#' @title Retrieve Bgee processed expression values at cell level for single cell data in Bgee.
#'
#' @description This function loads the processed gene count matrix of single cell experiments in Bgee.
#' These data are available for all droplet based single-cell RNA-seq and full length single-cell RNA-seq.
#' These files are stored in the H5AD format.
#'
#' @param myBgeeObject A Reference Class Bgee object, notably specifying the targeted species 
#' and data type.
#'
#' @param experimentId Filter allowing to specify one ArrayExpress or GEO accession, e.g., 
#' GSE43721. Can not be null
#' 
#' @param package Name of the R package used to load the H5AD cell data. It can either be \"anndata\"
#' or \"zellkonverter\". If zellkonverter is used then its default reader (R) is used.
#' 
#' @author Julien Wollbrett
#'
#' @examples{
#' \dontrun{
#'   bgee <- Bgee$new(species = "Gallus_gallus", dataType = "sc_droplet_based")
#'   cellProcessedData <- getCellProcessedData(bgee, experimentId = "ERP132576")
#' }
#' }
#'
#' @import zellkonverter anndata HDF5Array
#' @export
#' 
getCellProcessedData <- function(myBgeeObject, experimentId, package = "zellkonverter") {
  destFile <- downloadCellProcessedFile(myBgeeObject = myBgeeObject, experimentId = experimentId)
  error_message <- NULL
  experimentH5ad <- NULL
  if (package == "zellkonverter") {
    experimentH5ad <- readH5AD(file = destFile, reader = "R")
  } else if (package == "anndata") {
    tryCatch( { experimentH5ad <- read_h5ad(filename = destFile) },
      error = function(e) {error_message <<- e})
    experimentH5ad <- read_h5ad(filename = destFile)
  }
  if (! is.null(error_message)) {
    stop("an error occured : ", error_message)
  }
  return(experimentH5ad)
}

#' @title Download Bgee cell processed data
#'
#' @description This function allows to donwload Bgee H5AD files
#' 
#' @return the path to the downloaded cell processed data file
#' 
#' @noMd
#' @noRd
downloadCellProcessedFile <- function(myBgeeObject, experimentId) {
  if (length(experimentId) != 1) {
    stop("One experimentId has to be provided.")
  }
  if(length(package) != 1 | ! package %in% c("anndata", "zellkonverter")) {
    stop("the R package used to load H5AD cell data should either be anndata or zellkonverter.")
  }
  if (length(myBgeeObject$dataType) != 1 | ! myBgeeObject$dataType %in% c("sc_droplet_based", "sc_full_length")) {
    stop("Cell raw data are only available for single cell data. Please choose \"sc_droplet_based\" or \"sc_full_length\"",
         " as a datatpe of the Bgee object to be able to download such data")
  }
  experimentAnnotation <- getAnnotation(myBgeeObject)$experiment.annotation
  if (!experimentId %in% experimentAnnotation$Experiment.ID) {
    stop("the experiment ID:", experimentId, " provided does not exist in Bgee for the species ",
         myBgeeObject$species, ". Please first look at the available annotation using the ",
         "`getAnnotation(myBgeeObject)` function.")
  }
  destFile <- file.path(myBgeeObject$pathToData, paste0(experimentId, "_", myBgeeObject$dataType, ".h5ad"))
  downloadURL <- gsub("EXPIDPATTERN", experimentId, myBgeeObject$experimentH5adUrl)
  
  if (!file.exists(destFile)) {
    bgee_download_file(url = downloadURL, destfile = destFile, mode = 'wb')
  }
  return(destFile)
}

#' @title Retrieve Bgee calls from one species.
#'
#' @description This function loads the integrated expression calls from one species. These calls are 
#' equivalent to the one provided in the gene page of the Bgee website. The calls have been
#' generated using all datatypes present in Bgee (EST, In Situ, Affymetrix, bulk RNA-Seq and 
#' single-cell RNA-Seq).
#'
#' @param myBgeeObject A Reference Class Bgee object, notably specifying the targeted species 
#' and data type.
#'
#' @param conditionParameters Specify which condition parameter you are interested
#' in. It can be `anatEntity` or `allCondParams`. The `anatEntity` option retrieve calls generated 
#' using only anatomical entity and cell-type as a condition parameter. The `allCondParams` option retrieve calls generated taking
#' into consideration all condition parameters present in Bgee (anat, entity, cell-type, developmental stage,
#' sex and strain). By default the `anatEntity` option is selected.
#' 
#' @param advancedColumns Boolean allowing to specify if advancend columns are required. The default value
#' is `FALSE` meaning that only gene , condition parameters, call quality, FDR, expression rank and
#' expression score are retrieved. If `TRUE` then a lot more information used to generate the calls are
#' retrieved like the number of self and descendant observation for each datatype, or the rank and score
#' per datatype. 
#' 
#' @param geneIds List of genes for which expression calls have to be retrieved. It has to be ensembl
#' IDs if the Bgee genome source is Ensembl (e.g ENSG00000244734) or RefSeq IDs if the Bgee genome source
#' is RefSeq (e.g 734881) but not gene names (e.g HBB, Apoc1, etc.).
#'
#' @param anatEntityIds List of anatomical entity IDs for which expression calls have to be retrieved.
#' It has to be IDs (e.g UBERON:0000955) and not names (e.g brain)
#'
#' @return Return a data.table containing all calls from the specified species
#'
#' @author Julien Wollbrett
#'
#' @examples{
#'   bgee <- Bgee$new(species = "Danio_rerio")
#'   callsHer1 <- getIntegratedCalls(bgee, geneIds="ENSDARG00000014722")
#' }
#'
#' @import data.table bread
#' @importFrom R.utils gunzip
#' @export
#' 
getIntegratedCalls <- function(myBgeeObject, conditionParameters="anatEntity", advancedColumns=FALSE,
    geneIds=NULL, anatEntityIds=NULL) {
  if (! conditionParameters %in% c("anatEntity", "allCondParams")) {
    stop("the value of the conditionParameters argument should be either anatEntity or callCondParams.")
  }
  if(compareVersion(a = gsub("_", ".", myBgeeObject$release), b = "14.0") < 0) {
    stop("Integrated expression calls are available starting from Bgee 14.0.")
  }
  if(compareVersion(a = gsub("_", ".", myBgeeObject$release), b = "15.0") < 0 & conditionParameters != "anatEntity") {
    stop("calls files containing all condition parameters are only available starting from Bgee 15.0.\nPlease choose ",
         "conditionParameters=\"anatEntity\" if you really want to retrieve integrated expression calls from Bgee ",
         gsub("_", ".", myBgeeObject$release))
  }
  message("Start to download calls generated using all Bgee datatypes for ",
    gsub(pattern = "_", replacement = " ", x = myBgeeObject$speciesName))
  # Start to generate the proper download URL
  callsUrl <- myBgeeObject$callsUrl
  # Replace SPECIES_NAME pattern by actual species name
  callsUrl <- gsub(pattern = "SPECIES_NAME", replacement = myBgeeObject$speciesName, x = callsUrl)
  # Now replace COLUMN_TYPE pattern by the type of columns to retrieve
  callsUrl <- gsub(pattern = "COLUMN_TYPE", replacement = ifelse(advancedColumns, "expr_advanced", "expr_simple"),
    x = callsUrl)
  # And finally replace _COND_PARAMS pattern by the condition parameters requested
  callsUrl <- gsub(pattern = "_COND_PARAMS", replacement = ifelse(conditionParameters == "anatEntity",
    "", "_all_conditions"), x = callsUrl)
  
  # download the file and uncompress it
  destFile <- file.path(myBgeeObject$pathToData, basename(callsUrl))
  uncompressedDestFile <- gsub(pattern = ".gz$", replacement = "", x = destFile)
  if (!file.exists(uncompressedDestFile)) {
    bgee_download_file(url = callsUrl, destfile = destFile, mode = 'wb')
    output <- gunzip(destFile)
  }
  
  # combine genes to search in order to retrieve all rows with one of the gene Id
  outputCalls <- NULL
  timeNeeded <- NULL
  if (is.null(geneIds) & is.null(anatEntityIds)) {
    timeNeeded <- system.time(outputCalls <- fread(file = uncompressedDestFile, header = TRUE,
      quote = FALSE))
  } else {
    if (is.null(anatEntityIds) | !is.null(geneIds) & length(geneIds) < length(anatEntityIds)) {
      geneSearchPattern <- paste(geneIds, collapse =  "|")
      timeNeeded <- system.time(outputCalls <- breadWithErrorManagement(file = uncompressedDestFile,
        patterns = geneSearchPattern, filtered_columns = "Gene ID"))
      if (!is.null(anatEntityIds)) {
        outputCalls <- outputCalls[outputCalls$`Anatomical entity ID` %in% anatEntityIds,]
      }
    } else if (is.null(geneIds) | !is.null(anatEntityIds) & length(anatEntityIds) < length(geneIds)) {
      anatEntitySearchPattern <- paste(anatEntityIds, collapse = "|")
      timeNeeded <- system.time(outputCalls <- bfilter(file = uncompressedDestFile,
        patterns = anatEntitySearchPattern, filtered_columns = "Anatomical entity ID"))
      if (!is.null(geneIds)) {
        outputCalls <- outputCalls[outputCalls$`Gene ID` %in% geneIds,]
      }
    }
  }
  message("Expression calls have been retrieved in ", count_time(timeNeeded["elapsed"]))
  return(outputCalls)
  
}

breadWithErrorManagement <- function (file, patterns, filtered_columns) {
  outputCalls <- NULL
  tryCatch( {
    outputCalls <- bread(file = file,
      patterns = patterns, filtered_columns = filtered_columns)
    },
    error = function(e) {
      stop("can not run the bread function on your system. If you are ",
      "on Windows please first install 'RTools', 'Git' or 'Cygwin' to allow Unix commands ",
      "like \'grep\', \'sed\', \'wc\', \'awk\' or \'cut\'. If you do not want to install ",
      "such tools you can also run the getIntegratedCalls() function without geneIds and ",
      "anatEntityIds and filter afterwards.")
    })
}

# internal function managing download of files. Different approach is used depending on the OS.
# In Windows wininet is used by default but does not allow download longer than 30 seconds.
# In order to remove this timeout problem the package check if libcurl is available. If available
# curl package is used. Otherwise wininet is used and can potentially create timeout.
# in Mac and Linux libcurl is used.
#' @import curl
#' @noMd
#' @noRd

bgee_download_file <- function(url, destfile, quiet = TRUE, mode = NULL) {
  if(.Platform$OS.type == "windows") {
    if(capabilities("libcurl")) {
      if(is.null(mode)) {
        success <- curl_download(url = url, destfile = destfile, quiet = quiet)
      } else{
        success <- curl_download(url = url, destfile = destfile, quiet = quiet, mode = mode)
      }
      if(gsub("\\", "/", success, fixed = TRUE) == destfile | success == destfile) {
        return(0)
      } else {
        return(success)
      }
    } else {
      if(is.null(mode)) {
        warning("Windows OS without compatibility with libcurl. Could result to problems to download files.")
        warning("Please read the installation section of the vignette to solve any potential error.")
        return(download.file(url = url, destfile = destfile, quiet = quiet))
      } else{
        return(download.file(url = url, destfile = destfile, quiet = quiet, mode = mode))
      }
    }
  } else{
    if(is.null(mode)) {
      return(download.file(url = url, destfile = destfile, quiet = quiet))
    } else{
      return(download.file(url = url, destfile = destfile, quiet = quiet, mode = mode))
    }
  }
}

count_time <- function(seconds) {
  if (seconds < 60) {
    return(paste(seconds, "second(s)"))
  } else {
    minutes <- floor(seconds / 60)
    remaining_seconds <- seconds %% 60
    
    if (remaining_seconds == 0) {
      return(paste(minutes, "minute(s)"))
    } else {
      return(paste(minutes, "minute(s) and", remaining_seconds, "second(s)"))
    }
  }
}

#' @title Format RNA-seq or Affymetrix data downloaded from Bgee.
#'
#' @description This function formats the data downloaded with the getSampleRawData() function into an object of the Bioconductor "expressionSet" Class.
#'
#' @param myBgeeObject A Reference Class Bgee object, notably specifying the targeted species and data type.
#'
#' @param data A list of data frames including data from multiple experiments, or a data frame including data from a single experiment.
#'
#' @param stats A character indicating what expression values should be used in the formatted data expressionSet object matrix.
#'  \itemize{
#'    \item{"rpkm" for bulk RNA-seq (Bgee release 13.2 and before)}
#'    \item{"fpkm" for bulk RNA-seq (Bgee release 14.0 to 14.2)}
#'    \item{"counts" for RNA-seq}
#'    \item{"tpm" for bulk RNA-seq or full length single cell RNA-seq (Bgee release 14 and above)}
#'    \item{"cpm" for droplet based single cell RNA-seq (Bgee release 15.2 and above)}
#'    \item{"intensities" for Affymetrix microarrays}
#'  }
#'
#' @param callType A character indicating whether intensities should be displayed only for present (i.e., expressed) genes, present high quality genes, or all genes (default).
#'  \itemize{
#'    \item{"present"}
#'    \item{"present high quality"}
#'    \item{"all"}
#'  }
#'
#' @return If data was a list of data frames from multiple experiments, returns a list of ExpressionSet objects. If data was a data frame from a single experiment, returns an ExpressionSet object.
#'
#' @author Andrea Komljenovic and Julien Roux.
#'
#' @examples{
#'   bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
#'   dataMouseGSE43721 <- getData(bgee, experimentId = "GSE43721")
#'   dataMouseGSE43721.tpm <- formatData(bgee,
#'                                        dataMouseGSE43721,
#'                                        callType = "present",
#'                                        stats = "tpm")
#' }
#'
#' @importFrom dplyr %>%
#' @importFrom tidyr spread_
#' @export

formatData = function(myBgeeObject, data, stats = NULL, callType = "all"){

  ## check that fields of Bgee object are not empty
  if (length(myBgeeObject$quantitativeData) == 0 | length(myBgeeObject$dataType) == 0){
    stop("ERROR: there seems to be a problem with the input Bgee class object, some fields are empty. Please check that the object is valid.")
  }

  ## Check that download of data is possible for targeted species and data type
  if (myBgeeObject$quantitativeData != TRUE){
    stop("ERROR: formatting quantitative data is not possible for the species and data type of the input Bgee class object.")
  }

  if (length(stats) == 0){
    if (myBgeeObject$dataType == "affymetrix"){
      cat("\nWARNING: stats parameter set to \"intensities\" for the next steps.\n")
      stats <- "intensities"
    } else if (myBgeeObject$dataType %in% c("rna_seq", "sc_full_length")) {
      stop("Please specify the stats parameters. Should be set to \"counts\", \"tpm\" (For Bgee 14.0 and above), \"rpkm\" (for Bgee 13.2) or \"fpkm\" (between Bgee 14.0 and 14.2)")
    } else if (myBgeeObject$dataType %in% c("sc_droplet_based")) {
      stop("Please specify the stats parameters. Should be set to \"counts\" or \"cpm\"")
    }
  # manage stats error for all datatypes and Bgee releases
  } else if (myBgeeObject$dataType == "affymetrix" & stats != "intensities"){
    cat("\nWARNING: For Affymetrix microarray data, stats parameter can only be set to \"intensities\". This will be used for the next steps.\n")
    stats <- "intensities"
  } else if (myBgeeObject$dataType == "rna_seq" & compareVersion(gsub("_", ".", myBgeeObject$release), "13.2") <= 0 & !(stats %in% c('rpkm', 'counts'))){
    stop("Choose whether data formatting should create a matrix of RPKMs or read counts, with stats option set as \"rpkm\" or \"counts\"")
  } else if (myBgeeObject$dataType == "rna_seq" & compareVersion(gsub("_", ".", myBgeeObject$release), "14.0") >= 0 
      & compareVersion(gsub("_", ".", myBgeeObject$release), "14.2") <= 0 & !(stats %in% c('fpkm', 'counts', 'tpm'))){
    stop("Choose whether data formatting should create a matrix of FPKMs, TPMs or read counts, with stats option set as \"fpkm\", \"tpm\" or \"counts\"")
  } else if (myBgeeObject$dataType == "rna_seq" & compareVersion(gsub("_", ".", myBgeeObject$release), "14.2") > 0 & !(stats %in% c('counts', 'tpm'))){
    stop("Choose whether data formatting should create a matrix of TPMs or read counts, with stats option set as \"tpm\" or \"counts\"")
  } else if (myBgeeObject$dataType == "sc_full_length" & !(stats %in% c('counts', 'tpm'))){
    stop("Choose whether data formatting should create a matrix of TPMs or read counts, with stats option set as \"tpm\" or \"counts\"")
  } else if (myBgeeObject$dataType == "sc_droplet_based" & !(stats %in% c('counts', 'cpm'))){
    stop("Choose whether data formatting should create a matrix of CPMs or read counts, with stats option set as \"cpm\" or \"counts\"")
  }

  if(!(callType %in% c('present','present high quality','all'))){
    stop("Choose between displaying intensities for present genes, present high quality genes or all genes, e.g., 'present', 'present high quality', 'all' ")
  }

  if(length(data) == 1) data[[1]] else data

  cat("\nExtracting expression data matrix...\n")
  if(stats  == "rpkm"){
    columns <- c("Library.ID", "Gene.ID", "RPKM")
    expr <- .extract.data(data, columns, callType)
  } else if(stats  == "fpkm"){
    columns <- c("Library.ID", "Gene.ID", "FPKM")
    expr <- .extract.data(data, columns, callType)
  } else if(stats  == "tpm"){
    columns <- c("Library.ID", "Gene.ID", "TPM")
    expr <- .extract.data(data, columns, callType)
  } else if (stats == "counts"){
    columns <- c("Library.ID", "Gene.ID", "Read.count")
    expr <- .extract.data(data, columns, callType)
  } else if(stats  == "cpm"){
    columns <- c("Library.ID", "Gene.ID", "CPM")
    expr <- .extract.data(data, columns, callType)
  } else {
    columns <- c("Chip.ID", "Probeset.ID", "Log.of.normalized.signal.intensity", "Gene.ID")
    expr <- .extract.data(data, columns, callType)
  }
  ## one data matrix
  if(is.data.frame(expr$assayData)){

    ## sort objects to have samples in the same order
    expr$assayData <- expr$assayData[, order(names(expr$assayData))]
    expr$pheno <- expr$pheno[order(sampleNames(expr$pheno))]

    ## create ExpressionSet object
    eset <- new('ExpressionSet',
                exprs=as.matrix(expr$assayData),
                phenoData=expr$pheno,
                featureData=expr$features)
  } else if(is.list(expr$assayData)){
    ## multiple data matrices
    eset <- mapply(function(x,y,z){
      ## sort objects to have samples in the same order
      x <- x[, order(names(x))]
      y <- y[order(sampleNames(y))]

      ## create ExpressionSet object
      new('ExpressionSet',
          exprs=as.matrix(x),
          phenoData=y,
          featureData=z)
    },
    expr$assayData, expr$pheno, expr$features)
  }
  return(eset)
}

## Take in unformatted data downloaded from Bgee and outputs a list of expression matrices, phenotype annotations, feature annotations, and calls.
.extract.data <- function(data, columns, callType){
  ## if multiple data frames (multiple experiments or chips)
  if(class(data) == "list"){
    calls <- lapply(data, function(x) .calling(x, callType, columns[3]))
    expr <- lapply(calls,
                   function(x) {
                     ## subset the data to keep relevant columns
                     xt <- x[, columns[1:3]]
                     ## from sample and feature columns, create a matrix with features as rows and samples as columns
                     xtt <- xt %>% spread_(columns[1], columns[3])
                     rownames(xtt) <- xtt[, columns[2]]
                     ## Remove feature column to keep only data
                     xtt[,-1, drop = FALSE]
                   }
    )
    cat("\nExtracting features information...\n")
    features <- mapply(.extract.data.feature, calls, expr, rep(list(columns), times=length(calls)))
    cat("\nExtracting samples information...\n")
    phenos <- mapply(.extract.data.pheno, calls, rep(list(columns[1]), times=length(calls)))
  } else {
    ## if only a single dataframe
    calls <- .calling(data, callType, columns[3])
    ## subset the data to keep relevant columns
    xt <- calls[, columns[1:3]]
    xtt <- xt %>% spread_(columns[1], columns[3])
    rownames(xtt) <- xtt[, columns[2]]
    ## Remove feature column to keep only data
    expr <- xtt[,-1, drop = FALSE]

    cat("\nExtracting features information...\n")
    features <- .extract.data.feature( calls, expr, columns)
    cat("\nExtracting samples information...\n")
    phenos <- .extract.data.pheno( calls, columns[1])
  }
  return(list(assayData = expr, pheno = phenos, features = features, calls = calls))
}

# Extract feature data (probesets or genes)
.extract.data.feature <- function(calls, expr, columns){
  ## RNA-seq
  if(length(columns) == 3){
    fdata <- calls[match(rownames(expr), calls[, columns[2]]), columns[2], drop = FALSE]
  }
  ## Affymetrix, 4 columns
  else if(length(columns) == 4){
    fdata <- calls[match(rownames(expr), calls[, columns[2]]), columns[c(2,4)], drop = FALSE]
  }
  rownames(fdata) <- fdata[, columns[2]]
  fdata <- as(fdata, "AnnotatedDataFrame")
  return(fdata)
}

# Extract annotation of samples
.extract.data.pheno <- function(calls, column){
  phdata <- calls[, c(column, "Anatomical.entity.ID", "Anatomical.entity.name", "Stage.ID", "Stage.name")]
  phdata <- phdata[!duplicated(phdata[, column]), ]
  rownames(phdata) <- phdata[, column]
  phdata <- as.data.frame(phdata)
  metadata <- data.frame(labelDescription=colnames(phdata),
                         row.names=colnames(phdata))
  phenodata <- new("AnnotatedDataFrame",
                   data=phdata,
                   varMetadata=metadata
  )
  return(phenodata)
}

.calling <- function(x, callType, column){
  ## check data type
  if(callType == "present"){
    cat("  Keeping only present genes.\n")
    x[(x$Detection.flag == "absent"), column] <- NA
  } else if (callType == "present high quality"){
    cat("  Keeping only present high quality genes.\n")
    x[which(x$Detection.flag == "absent" | x$Detection.quality == "poor quality"), column] <- NA
  }
  return(x)
}

context("expect_output")


test_that("Formatting gene expression files", {
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  data_bgee_mouse_exp <- getData(bgee, experimentId  = "GSE43721")
  gene.expression <- formatData(bgee, data_bgee_mouse_exp, callType = "present", stats = "tpm")
  expect_true(class(gene.expression) == "ExpressionSet" )
  gene.expression <- formatData(bgee, data_bgee_mouse_exp, callType = "present", stats = "counts")
  expect_true(class(gene.expression) == "ExpressionSet" )
  # not available since bgee_v15_1
  expect_error(formatData(bgee, data_bgee_mouse_exp, callType = "present", stats = "fpkm"),
               "Choose whether data formatting should create a matrix of TPMs or read counts, with stats option set as \"tpm\" or \"counts\"")
  # not available since bgee_v14
  expect_error(formatData(bgee, data_bgee_mouse_exp, callType = "present", stats = "rpkm"), 
               "Choose whether data formatting should create a matrix of TPMs or read counts, with stats option set as \"tpm\" or \"counts\"")
  #test formatting for files generated before bgee_v14
  bgee_v13 <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq", release = "13_2")
  data_bgee_mouse_exp_v13 <- getData(bgee_v13, experimentId  = "GSE43721")
  expect_error(formatData(bgee_v13, data_bgee_mouse_exp_v13, callType = "present", stats = "tpm"), 
               "Choose whether data formatting should create a matrix of RPKMs or read counts, with stats option set as \"rpkm\" or \"counts\"")
  expect_error(formatData(bgee_v13, data_bgee_mouse_exp_v13, callType = "present", stats = "fpkm"), 
               "Choose whether data formatting should create a matrix of RPKMs or read counts, with stats option set as \"rpkm\" or \"counts\"")  
  gene.expression <- formatData(bgee_v13, data_bgee_mouse_exp_v13, callType = "present", stats = "counts")
  expect_true(class(gene.expression) == "ExpressionSet" )
  gene.expression <- formatData(bgee_v13, data_bgee_mouse_exp_v13, callType = "present", stats = "rpkm")
  expect_true(class(gene.expression) == "ExpressionSet" )

  
})

#' @title Bgee Reference Class
#'
#' @description This is used to specify information at the beginning of a BgeeDB working session, for example, the targeted species and data type. An object of this class is then passed as argument to other functions of the package to provide these informations. See examples in vignette.
#'
#' @details Bgee (\url{https://www.bgee.org}) integrates different expression data types (RNA-seq, Affymetrix microarray, ESTs, and in-situ hybridizations) from multiple animal species. Expression patterns are based exclusively on curated "normal", healthy, expression data (e.g., no gene knock-out, no treatment, no disease), to provide a reference atlas of normal gene expression.
#'
#' @field dataType A vector of characters indicating data type(s) to be used. To be chosen among:
#' \itemize{
#'   \item{"rna_seq: target only bulk RNA-seq data"}
#'   \item{"sc_full_length: target only full length single-cell RNA-seq data"}
#'   \item{"sc_droplet_based: target only droplet based single-cell RNA-seq data"}
#'   \item{"affymetrix: target only Affymtrix microarray data"}
#'   \item{"est: target only ESTs data"}
#'   \item{"in_situ: target only In Situ data"}
#' }
#' By default all data type are included: \code{c("rna_seq","sc_full_length","sc_droplet_based","affymetrix","est","in_situ")}. For download of quantitative expression data, a single data type should be chosen among "rna_seq", "sc_full_length", sc_droplet_based" or "affymetrix".
#'
#' @field pathToData Path to the directory where the data files are stored. By default the working directory is used. If many analyses are launched in parallel, please consider re-using the cached data files instead of redownlaoding them for each analysis.
#'
#' @field release Bgee release number to download data from, in the form "Release.subrelease" or "Release_subrelease", e.g., "15.2" or 15_2". Work for release >=13.2. By default, the latest relase of Bgee is used.
#'
#' @field sendStats A field specifying whether monitoring of users is performed for our internal usage statistics. This is useful to improve the settings of our servers and to get reliable usage statistics (e.g., when asking for funding for Bgee). No identification of the users is attempted, nor possible. Default to TRUE. This option can be set to FALSE, notably if all data files are in cache and that users want to be able to work offline.
#'
#' @field quantitativeData A field specifying if a single type of quantitative expression data ("rna_seq", "sc_full_length", "sc_droplet_based" or "affymetrix") was specified and if it is available for targeted species, helping the package to know if it should proceed with the execution of getAnnotation() and getData() functions.
#'
#' @examples{
#'  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
#'  bgee <- Bgee$new(species = "Mus_musculus")
#' }
#'
#' @import methods Biobase RCurl data.table
#' @export Bgee
#' @exportClass Bgee

Bgee <- setRefClass(
  "Bgee",

  fields = list(
    species = "character",
    speciesName = "character",
    speciesId = "numeric",
    dataType = "character",
    pathToData = "character",
    pathToCalls = "character",
    release = "character",
    annotationUrl = "character",
    experimentUrl = "character",
    allExperimentsUrl = "character",
    experimentH5adUrl = "character",
    callsUrl = "character",
    topAnatUrl = "character",
    sendStats = "logical",
    quantitativeData = "logical",
    apiKey = "character",
    sqlite_extension = "character"
  ),
  
  methods = list(
    initialize = function(..., sqlite_extension="_bgee.db") {
      callSuper(..., sqlite_extension="_bgee.db")

      ## check data type
      if (length(dataType) == 0) {
        cat("\nNOTE: You did not specify any data type. The argument dataType will be set to c(\"rna_seq\",\"affymetrix\",\"est\",\"in_situ\",\"sc_full_length\", \"sc_droplet_based\") for the next steps.\n")
        dataType <<- c("rna_seq","affymetrix","est","in_situ","sc_full_length", "sc_droplet_based")
      } else if ( !sum(dataType %in% c("rna_seq","affymetrix","est","in_situ","sc_full_length", "sc_droplet_based")) %in% 1:6 ){
        stop("ERROR: you need to specify at least one valid data type to be used among \"rna_seq\", \"affymetrix\", \"sc_full_length\", \"sc_droplet_based\", \"est\" and \"in_situ\".")
      }
      if ( length(dataType) != sum(dataType %in% c("rna_seq","affymetrix","est","in_situ", "sc_full_length", "sc_droplet_based")) ){
        cat("\nWARNING: you apparently specified a data type that is not among \"rna_seq\", \"affymetrix\", \"sc_full_length\", \"sc_droplet_based\", \"est\" and \"in_situ\". This will be removed. Please check for typos.\n")
        dataType <<- dataType[dataType %in% c("rna_seq","affymetrix","sc_full_length","est","in_situ", "sc_droplet_based")]
      }


      ## check path of folder to store cached files
      pathToData <<- path.expand(pathToData)
      if (length(pathToData) == 0) {
        pathToData <<- getwd()
      } else if (length(pathToData) == 1) {
        if (!file.exists(pathToData)) {
          stop("ERROR: please specify a valid and existing path to store data files.")
        }
      } else {
        stop("ERROR: Invalid path for data files.")
      }


      ## Get release information
      cat("\nQuerying Bgee to get release information...\n")
      allReleases <- try(.getBgeeRelease(removeFile = FALSE), silent=TRUE)
      if (class(allReleases) == "data.frame"){
        file.rename(from=file.path(getwd(), 'release.tsv'),
                    to=file.path(pathToData, "release.tsv"))
      } else if (class(allReleases) == "try-error"){
        if (file.exists(file.path(pathToData, "release.tsv"))){
          cat(paste0("\nWARNING: BgeeDB could not access Bgee releases information from the internet, but a release information file was found in the download directory ",
                     pathToData, ". This release file will be used, but be warned that it may not be up to date!\n"))
          allReleases <- read.table(file.path(pathToData, "release.tsv"), header=TRUE, sep="\t")
        } else {
          stop("ERROR: BgeeDB could not access Bgee releases information. Is your internet connection working?")
        }
      }

      if (length(release) == 0) {
        release <<- gsub("\\.", "_", format(allReleases$release, drop0Trailing = F)[1])
      } else if (length(release) == 1) {
        # In case the release number is written with a dot
        release <<- as.character(gsub("\\.", "_", release))
        if(grepl("_",release)){
          # test if required release exists
          # TODO same code than in the listSpeciesRelease
          if (sum(as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))) != 1) {
            stop("ERROR: The specified release number is invalid, or is not available for BgeeDB.")
          }
        } else {
          stop("Bgee release number must follow the format \"Release.subrelease\" or \"Release_subrelease\", e.g., \"15.2\" or \"15_2\". See function listBgeeRelease() to see available releases.")
        }
      } else {
        stop("ERROR: The specified release number is invalid.")
      }

      ## Specify URL to use to download expression calls. Can be done for any species
      callsUrl <<- as.character(allReleases$Calls.URL[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
      ## Specify URL to be used for topAnat. Can be done for any species and data type
      topAnatUrl <<-  as.character(allReleases$TopAnat.URL[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
      if ( !grepl("/$", topAnatUrl) ){
        topAnatUrl <<- paste0(topAnatUrl, "/")
      }


      ## Get species information
      if (file.exists(paste0(pathToData, "/species_Bgee_", release, ".tsv"))){
        cat(paste0("\nNOTE: the file describing Bgee species information for release ", release, " was found in the download directory ",
                   pathToData, ". Data will not be redownloaded.\n"))
        allSpecies <- read.table(paste0(pathToData, "/species_Bgee_", release, ".tsv"),
                                 header=TRUE,
                                 sep="\t",
                                 blank.lines.skip=TRUE,
                                 as.is=TRUE)
      } else {
        allSpecies <- listBgeeSpecies(release = release,
                                      allReleases = allReleases,
                                      removeFile = FALSE)
      }

      ## check species argument (compulsory, no default value)
      if (length(species) == 0) {
        stop("ERROR: You did not specify any species.")
      } else if (length(species) > 1) {
        stop("ERROR: only one species is allowed.")
      } else if (grepl("^\\d+$", species)) {
        ## if species was specified as a taxonomic ID
        if (sum(allSpecies$ID == species) != 1) {
          stop(paste0("ERROR: The specified species taxonomic Id is invalid, or not available in Bgee release ",
                      release, "."))
        } else {
          speciesId <<- as.numeric(species)
          speciesName <<- gsub(" ", "_", paste(allSpecies[allSpecies$ID == species, 2:3], collapse = "_"))
        }
      } else {
        speciesSplitted <- unlist(regmatches(species, regexpr("_", species), invert = TRUE))
        # update species name in case it is composed of more than one word (e.g lupus familiaris)
        speciesSplitted[2] <- gsub("_", " ", speciesSplitted[2])
        if (sum(allSpecies$GENUS == speciesSplitted[1] &
                allSpecies$SPECIES_NAME == speciesSplitted[2]) != 1) {
          stop(paste0("ERROR: The specified species name is invalid, or not available in Bgee release ",
                      release, "."))
        } else {
          speciesName <<- gsub(" ", "_", species)
          speciesId <<- as.numeric(allSpecies$ID[allSpecies$GENUS == speciesSplitted[1] &
                                                   allSpecies$SPECIES_NAME == speciesSplitted[2]])
        }
      }


      ## Set quantitativeData field to FALSE by default
      quantitativeData <<- FALSE

      ## For quantitative data download, there should be only 1 data type specified: rna_seq, sc_full_length, sc_droplet_based or affymetrix
      if (length(dataType) == 1){
        ## If only RNA-seq data, check availability in species
        if (dataType == "rna_seq") {
          quantitativeData <<- allSpecies$RNA_SEQ[allSpecies$ID == speciesId]
        }
        
        else if (dataType == "affymetrix") {
          quantitativeData <<- allSpecies$AFFYMETRIX[allSpecies$ID == speciesId]
        }
        ## If only full length single cell data, check availability in species
        else if (dataType == "sc_full_length") {
          if(compareVersion(a = gsub("_", ".", release), b = "15.0") >= 0) {
            if(compareVersion(a = gsub("_", ".", release), b = "15.2") >= 0) {
              quantitativeData <<- allSpecies$SC_FULL_LENGTH[allSpecies$ID == speciesId]
            } else {
              quantitativeData <<- allSpecies$SC_RNA_SEQ[allSpecies$ID == speciesId]
            }
          } else {
            stop("single-cell full-length data are only available for Bgee 15 and later.")
          }
        }
        else if (dataType == "sc_droplet_based") {
          if(compareVersion(a = gsub("_", ".", release), b = "15.2") >= 0) {
            quantitativeData <<- allSpecies$SC_DROPLET_BASED[allSpecies$ID == speciesId]
          } else {
            stop("single-cell droplet_based data are only available for Bgee 15.2 and later.")
          }
        }

        ## if quantitative data download can be done, fill the URLs fields
        if (quantitativeData == TRUE){
          if (dataType == "rna_seq") {
            ## annotation file
            annotationUrl <<- as.character(
              allReleases$RNA.Seq.annotation.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## Data from specific experiment
            experimentUrl <<- as.character(
              allReleases$RNA.Seq.experiment.value.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## Data from all experiments
            allExperimentsUrl <<- as.character(
              allReleases$RNA.Seq.all.value.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
          } else if (dataType == "affymetrix") {
            ## annotation file
            annotationUrl <<- as.character(
              allReleases$Affymetrix.annotation.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## Data from specific experiment
            experimentUrl <<- as.character(
              allReleases$Affymetrix.experiment.value.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## Data from all experiments
            allExperimentsUrl <<- as.character(
              allReleases$Affymetrix.all.value.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
          } else if (dataType == "sc_full_length") {
            ## annotation file
            annotationUrl <<- as.character(
              allReleases$Full.Length.single.cell.RNA.Seq.annotation.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## Data from specific experiment
            experimentUrl <<- as.character(
              allReleases$Full.Length.single.cell.RNA.Seq.experiment.value.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## Data from all experiments
            allExperimentsUrl <<- as.character(
              allReleases$Full.Length.single.cell.RNA.Seq.all.value.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## h5ad from specific experiment
            experimentH5adUrl <<- as.character(
              allReleases$Full.Length.single.cell.RNA.Seq.H5AD.experiment.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
          } else if (dataType == "sc_droplet_based") {
            ## annotation file
            annotationUrl <<- as.character(
              allReleases$Droplet.Based.single.cell.RNA.Seq.annotation.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## Data from specific experiment
            experimentUrl <<- as.character(
              allReleases$Droplet.Based.single.cell.RNA.Seq.experiment.value.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            ## Data from all experiments
            allExperimentsUrl <<- as.character(
              allReleases$Droplet.Based.single.cell.RNA.Seq.all.value.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
            experimentH5adUrl <<- as.character(
              allReleases$Droplet.Based.single.cell.RNA.Seq.H5AD.experiment.URL.pattern[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
          }

          ## Regex substitution to get the correct URLs
          annotationUrl <<- gsub("SPECIESNAMEPATTERN", speciesName, annotationUrl)
          allExperimentsUrl <<- gsub("SPECIESNAMEPATTERN", speciesName, allExperimentsUrl)
          experimentUrl <<- gsub("SPECIESNAMEPATTERN", speciesName, experimentUrl)
          experimentH5adUrl <<- gsub("SPECIESNAMEPATTERN", speciesName, experimentH5adUrl)
          ## Note: one more substitution is needed here for the experiment id. This is done in the function loading the data.
        }
      }


      ## create sub-folder with species name to store downloaded files
      pathToData <<- paste0(pathToData, "/", speciesName, "_Bgee_", release)
      if (!file.exists(pathToData)) {
        dir.create(pathToData)
      }


      ## sendStats field
      if (length(sendStats) == 0) {
        sendStats <<- TRUE
      } else if (length(sendStats) != 1 | (length(sendStats) == 1 & !is.logical(sendStats))) {
        cat("\nNOTE: You did not specify a valid value for the \"sendStats\" field (should be TRUE or FALSE). The field will be set to TRUE for the next steps.\n")
        sendStats <<- TRUE
      }


      ## Create a concatenated string made of several variables that should be unique to the user, using Sys.info(), Sys.getenv() and R.version variables.
      myUserInfo <- c(Sys.info()[c("sysname", "release", "version", "machine", "login")], Sys.getenv()[c("R_HOME", "R_LIBS_USER")], R.version)
      ## Use library digest to create a SHA512 hash, that will be used as API key
      apiKey <<- digest(myUserInfo, algo = "sha512")
      cat(paste0("\nAPI key built: ", apiKey, "\n"))

      ## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
      ## The API key allows to monitor usage of our package and to limit the number of simultaneous queries
      ## to the server from the same user. It is a secure hash that does not allow the identification of
      ## the user.
      ## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


      ## Message to users. Can for example be used to encourage users to update their version of the package.
      messageToUsers <- as.character(allReleases$messageToUsers[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))])
      if(!(is.na(messageToUsers) || is.null(messageToUsers) || identical(messageToUsers, ''))){
        cat(paste0("\nIMPORTANT INFORMATION: ", messageToUsers, "\n"))
      }

    }
  )
)

#' @title List Bgee releases available to use with BgeeDB package
#'
#' @description Returns information on available Bgee releases, the access URL for FTP and webservice, and the date of release
#'
#' @param release A character specifying a targeted release number. In the form "Release.subrelease" or "Release_subrelease", e.g., "13.2" or 13_2". If not specified, all available releases are shown.
#'
#' @return A data frame with information on Bgee releases.
#'
#' @examples{
#'  listBgeeRelease()
#' }
#'
#' @author Julien Roux, Julien Wollbrett
#' @export

# Function displaying the user a data frame describing all releases available for Bgee
listBgeeRelease <- function(release=NULL){
  cat("Downloading release information from Bgee...\n")
  allReleases <- .getBgeeRelease()
  if (length(release)==1){
    if (sum(allReleases$release == gsub("_", ".", release))==1){
      cat(paste0("Only displaying information from targeted release ",
                 gsub("_", ".", release), "\n"))
      allReleases <- allReleases[allReleases$release == gsub("_", ".", release), ]
    } else {
      stop("ERROR: The specified release number is invalid or is not available for BgeeDB.")
    }
  }
  ## Only return the columns of interest to the user
  return(allReleases[, c("release","release.date", "FTP.URL", "TopAnat.URL")])
}

# Function returning a data frame describing all releases available for Bgee
.getBgeeRelease <- function(removeFile=TRUE){
  ## query FTP to get file describing all releases
  releaseUrl <- 'https://www.bgee.org/ftp/release_v2.tsv'
  success <- try(bgee_download_file(url = releaseUrl, quiet = TRUE,
                           destfile = file.path(getwd(), 'release.tsv.tmp')))
  if (success == 0 & file.exists(file.path(getwd(), 'release.tsv.tmp'))){
    file.rename(from=file.path(getwd(), 'release.tsv.tmp'),
                to=file.path(getwd(), 'release.tsv'))
    allReleases <- read.table("release.tsv", header=TRUE, sep="\t")
    if (removeFile == TRUE){
      file.remove(file.path(getwd(), 'release.tsv'))
    }
  } else {
    file.remove(file.path(getwd(), 'release.tsv.tmp'))
    stop("ERROR: File describing releases could not be downloaded from FTP.")
  }
  ## Keep release available with the current version of the package
  allAvailableReleases <- allReleases[  sapply(as.character(allReleases$minimumVersionBgeeDB), compareVersion, as.character(packageVersion("BgeeDB"))) <= 0,]
  return(allAvailableReleases)
}

## -----------------------------------------------------------------------------
#if (!requireNamespace("BiocManager", quietly=TRUE))
    #install.packages("BiocManager")
#BiocManager::install("BgeeDB")

## ----message = FALSE, warning = FALSE-----------------------------------------
library(BgeeDB)

## -----------------------------------------------------------------------------
listBgeeSpecies()

## -----------------------------------------------------------------------------
listBgeeSpecies(release = "13.2", order = 2)

## -----------------------------------------------------------------------------
bgee <- Bgee$new(species = "Danio_rerio", dataType = "rna_seq")

## -----------------------------------------------------------------------------
annotation_bgee_zebrafish <- getAnnotation(bgee)
# list the first experiments and libraries
lapply(annotation_bgee_zebrafish, head)

## -----------------------------------------------------------------------------
# download all RNA-seq experiments from zebrafish
data_bgee_zebrafish <- getSampleProcessedData(bgee)
# number of experiments downloaded
length(data_bgee_zebrafish)
# check the downloaded data
lapply(data_bgee_zebrafish, head)
# isolate the first experiment
data_bgee_experiment1 <- data_bgee_zebrafish[[1]]

## -----------------------------------------------------------------------------
# download data for GSE68666
data_bgee_zebrafish_gse68666 <- getSampleProcessedData(bgee, experimentId = "GSE68666")

## ----eval=FALSE---------------------------------------------------------------
# # Examples of data downloading using different filtering combination
# # retrieve zebrafish RNA-Seq data for heart (UBERON:0000955) or brain (UBERON:0000948)
# data_bgee_zebrafish_filters <- getSampleProcessedData(bgee, anatEntityId = c("UBERON:0000955","UBERON:0000948"))
# # retrieve zebrafish RNA-Seq data for embryo (UBERON:0000922) part of the experiment GSE68666
# data_bgee_zebrafish_filters <- getSampleProcessedData(bgee, experimentId = "GSE68666", anatEntityId = "UBERON:0000922")
# # retrieve zebrafish RNA-Seq data for head kidney (UBERON:0007132) or swim bladder (UBERON:0006860) from post-juvenile adult stage (UBERON:0000113)
# data_bgee_zebrafish_filters <- getSampleProcessedData(bgee, stageId = "UBERON:0000113", anatEntityId = c("UBERON:0007132","UBERON:0006860"))
# # retrieve zebrafish RNA-Seq data for brain (UBERON:0000948) and all substructures of brain from post-juvenile adult stage (UBERON:0000113)
# data_bgee_zebrafish_filters <- getSampleProcessedData(bgee, stageId = "UBERON:0000113", anatEntityId = "UBERON:0000948", withDescendantAnatEntities = TRUE)

## -----------------------------------------------------------------------------
# use only present calls and fill expression matrix with TPM values
gene.expression.zebrafish.tpm <- formatData(bgee, data_bgee_zebrafish_gse68666, callType = "present", stats = "tpm")
gene.expression.zebrafish.tpm 

## ----eval=FALSE---------------------------------------------------------------
# #create a bgee object to download droplet based data from Gallus gallus
# bgee <- Bgee$new(species = "Gallus_gallus", dataType = "sc_droplet_based")
# # download cell data for one RNA-seq experiment
# cell_data_bgee_gallus_gallus <- getCellProcessedData(bgee, experimentId = "ERP132576")

## -----------------------------------------------------------------------------
# Creating new Bgee class object
bgee <- Bgee$new(species = "Danio_rerio", release = "15.2")

## -----------------------------------------------------------------------------
# Loading calls of expression
myTopAnatData <- loadTopAnatData(bgee)
# Look at the data
## str(myTopAnatData)

## ----eval=FALSE---------------------------------------------------------------
# ## Loading silver and gold expression calls from affymetrix data made on embryonic samples only
# ## This is just given as an example, but is not run in this vignette because only few data are returned
# bgee <- Bgee$new(species = "Danio_rerio", dataType="affymetrix", release = "15.2")
# myTopAnatData <- loadTopAnatData(bgee, stage="UBERON:0000068", confidence="silver")

## ----eval=FALSE---------------------------------------------------------------
# # if (!requireNamespace("BiocManager", quietly=TRUE))
#     # install.packages("BiocManager")
# # BiocManager::install("biomaRt")
# library(biomaRt)
# ensembl <- useMart("ENSEMBL_MART_ENSEMBL", dataset="drerio_gene_ensembl", host="mar2016.archive.ensembl.org")
# 
# # get the mapping of Ensembl genes to phenotypes. It will corresponds to the background genes
# universe <- getBM(filters=c("phenotype_source"), value=c("ZFIN"), attributes=c("ensembl_gene_id","phenotype_description"), mart=ensembl)
# 
# # select phenotypes related to pectoral fin
# phenotypes <- grep("pectoral fin", unique(universe$phenotype_description), value=T)
# 
# # Foreground genes are those with an annotated phenotype related to "pectoral fin"
# myGenes <- unique(universe$ensembl_gene_id[universe$phenotype_description %in% phenotypes])
# 
# # Prepare the gene list vector
# geneList <- factor(as.integer(unique(universe$ensembl_gene_id) %in% myGenes))
# names(geneList) <- unique(universe$ensembl_gene_id)
# summary(geneList)
# 
# # Prepare the topGO object
# myTopAnatObject <-  topAnat(myTopAnatData, geneList)

## -----------------------------------------------------------------------------
data(geneList)
myTopAnatObject <-  topAnat(myTopAnatData, geneList)

## -----------------------------------------------------------------------------
results <- runTest(myTopAnatObject, algorithm = 'weight', statistic = 'fisher')

## -----------------------------------------------------------------------------
# Display results sigificant at a 1% FDR threshold
tableOver <- makeTable(myTopAnatData, myTopAnatObject, results, cutoff = 0.01)
head(tableOver)

## -----------------------------------------------------------------------------
# In order to retrieve significant genes mapped to the term " paired limb/fin bud"
term <- "UBERON:0004357"
termStat(myTopAnatObject, term) 

# 172 genes mapped to this term for Bgee 15.2
genesInTerm(myTopAnatObject, term)
# 37 significant genes mapped to this term for Bgee 15.2
annotated <- genesInTerm(myTopAnatObject, term)[["UBERON:0004357"]]
annotated[annotated %in% sigGenes(myTopAnatObject)]

## ----eval = FALSE-------------------------------------------------------------
# bgee <- Bgee$new(species="Mus_musculus", release = "14.1")
# # delete all old .rds files of species Mus musculus
# deleteOldData(bgee)

## ----eval = FALSE-------------------------------------------------------------
# bgee <- Bgee$new(species="Mus_musculus", release = "14.1")
# # delete local SQLite database of species Mus musculus from Bgee 14.1
# deleteLocalData(bgee)

#' @title Retrieve data from Bgee to perform GO-like enrichment of anatomical terms, mapped to genes by expression patterns.
#'
#' @description This function loads a mapping from genes to anatomical structures based on calls of expression in anatomical structures. It also loads the structure of the anatomical ontology.
#'
#' @details The expression calls come from Bgee (\url{http://bgee.org}), that integrates different expression data types (RNA-seq, Affymetrix microarray, ESTs, or in-situ hybridizations) from multiple animal species. Expression patterns are based exclusively on curated "normal", healthy, expression data (e.g., no gene knock-out, no treatment, no disease), to provide a reference atlas of normal gene expression. Anatomical structures are identified using IDs from the Uberon ontology (browsable at \url{http://www.ontobee.org/ontology/UBERON}). The mapping from genes to anatomical structures includes only the evidence of expression in these specific structures, and not the expression in their substructures (i.e., expression data are not propagated). The retrieval of propagated expression data might be implemented in the future, but meanwhile, it can be obtained using specialized packages such as topGO, see the \code{topAnat.R} function.
#'
#' @param myBgeeObject An output object from Bgee$new().
#'
#' @param callType A character of indicating the type of expression calls to be used for enrichment. Only calls for significant detection of expression are implemented so far ("presence"). Differential expression calls, based on differential expression analysis, might be implemented in the future.
#'
#' @param stage A character indicating the targeted developmental stages for the analysis. Developmental stages can be chosen from the developmental stage ontology used in Bgee (available at \url{https://github.com/obophenotype/developmental-stage-ontologies}). If a stage is specified, the expression pattern mapped to this stage and all children developmental stages (substages) will be retrieved. Default is NULL, meaning that expression patterns of genes are retrieved regardless of the developmental stage displaying expression; this is equivalent to specifying stage="UBERON:0000104" (life cycle, the root of the stage ontology).
#' For information, the most useful stages (going no deeper than level 3 of the ontology) include:
#' \itemize{
#'   \item{UBERON:0000068 (embryo stage)}
#'   \itemize{
#'     \item{UBERON:0000106 (zygote stage)}
#'     \item{UBERON:0000107 (cleavage stage)}
#'     \item{UBERON:0000108 (blastula stage)}
#'     \item{UBERON:0000109 (gastrula stage)}
#'     \item{UBERON:0000110 (neurula stage)}
#'     \item{UBERON:0000111 (organogenesis stage)}
#'     \item{UBERON:0007220 (late embryonic stage)}
#'     \item{UBERON:0004707 (pharyngula stage)}
#'   }
#'   \item{UBERON:0000092 (post-embryonic stage)}
#'   \itemize{
#'     \item{UBERON:0000069 (larval stage)}
#'     \item{UBERON:0000070 (pupal stage)}
#'     \item{UBERON:0000066 (fully formed stage)}
#'   }
#' }
#'
#' @param confidence A character indicating if only high quality present calls should be
#' retrieved. For Bgee releases prior to 14, options are "all" (default) or "high_quality".
#' For Bgee release 14 and above, options are "silver" (default) and "gold".
#'
#' @param timeout local timeout used when the function is run. It allows to modify the
#' timeout used by default in R to download files. If not provided the timeout will be
#' fixed at 1800 secondes. When the functions exits (either naturally or as the result
#' of an error) the download timeout go back to the original one used in the R session.
#'
#' @return A list of 4 elements:
#' \itemize{
#'   \item{A \code{gene2anatomy} list, mapping genes to anatomical structures based on expression calls.}
#'   \item{A \code{organ.names} data frame, with the name corresponding to UBERON IDs.}
#'   \item{A \code{organ.relationships} list, giving the relationships between anatomical structures in the UBERON ontology (based on parent-child "is_a" and "part_of" relationships).}
#'   \item{The Bgee class object thta was used to retrieve the data.}
#' }
#'
#' @author Julien Roux, Julien Wollbrett
#'
#' @examples{
#'   bgee <- Bgee$new(species = "Danio_rerio", dataType = "rna_seq")
#'   myTopAnatData <- loadTopAnatData(bgee)
#' }
#'
#' @import utils digest
#' @export
loadTopAnatData <- function(myBgeeObject, callType="presence", confidence=NULL, stage=NULL, timeout = 1800){
  OLD_WEBSERVICE_VERSION = '13.2'

  ## check that fields of Bgee object are not empty
  if (length(myBgeeObject$speciesId) == 0 | length(myBgeeObject$topAnatUrl) == 0 | length(myBgeeObject$dataType) == 0 | length(myBgeeObject$pathToData) == 0 | length(myBgeeObject$sendStats) == 0){
    stop("ERROR: there seems to be a problem with the input Bgee class object, some fields are empty. Please check that the object is valid.")
  }
  if ( callType != "presence" ){
    stop("ERROR: no other call types than \"presence\" expression calls can be retrieved for now.")
  }
  if ( compareVersion(gsub("_", ".", myBgeeObject$release), OLD_WEBSERVICE_VERSION) > 0 ){
    if ( is.null(confidence) ){
      confidence = "silver"
    }
    if ( (confidence != "gold") && (confidence != "silver") ){
      stop(paste0("ERROR: the data confidence parameter specified is not among the allowed values. For Bgee ", myBgeeObject$release, " allowed values are \"silver\" or \"gold\".\nBy default \"silver\" quality is selected."))
    }
  } else {
    if(is.null(confidence)){
      confidence = "all"
    }
    if ( (confidence != "all") && (confidence != "high_quality") ){
      stop(paste0("ERROR: the data confidence parameter specified is not among the allowed values. For Bgee ", myBgeeObject$release, " allowed values are \"all\" or \"high_quality\".\nBy default \"all\" quality is selected."))
    }
  }

  ## Set the timeout option to timeout value to let some time to the server to send data (default is 1800 sec.)
  op <- options(timeout = timeout)
  ## on exit change back options to initial values
  on.exit(options(op))

  ## First query: organ relationships
  organRelationshipsFileName <- paste0("topAnat_AnatEntitiesRelationships_", myBgeeObject$speciesId, ".tsv")
  ## Check if file is already in cache
  if (file.exists(file.path(myBgeeObject$pathToData, organRelationshipsFileName))){
    cat(paste0("\nNOTE: an organ relationships file was found in the download directory ", myBgeeObject$pathToData,
        ". Data will not be redownloaded.\n"))
  } else {
    cat("\nBuilding URLs to retrieve organ relationships from Bgee.........\n")
    myUrl <- myBgeeObject$topAnatUrl
    if(compareVersion(gsub("_", ".", myBgeeObject$release), OLD_WEBSERVICE_VERSION) > 0){
      myUrl <- paste0(myUrl, "?page=r_package&action=get_anat_entity_relations&display_type=tsv&species_list=", myBgeeObject$speciesId, "&attr_list=SOURCE_ID&attr_list=TARGET_ID&api_key=", myBgeeObject$apiKey, "&source=BgeeDB_R_package&source_version=", as.character(packageVersion("BgeeDB")))
    } else {
      myUrl <- paste0(myUrl, "?page=dao&action=org.bgee.model.dao.api.ontologycommon.RelationDAO.getAnatEntityRelations&display_type=tsv&species_list=", myBgeeObject$speciesId, "&attr_list=SOURCE_ID&attr_list=TARGET_ID&api_key=", myBgeeObject$apiKey, "&source=BgeeDB_R_package&source_version=", as.character(packageVersion("BgeeDB")))
    }
    ## Query webservice
    cat(paste0("   URL successfully built (", myUrl,")\n   Submitting URL to Bgee webservice (can be long)\n"))
    success <- bgee_download_file(url = myUrl, destfile = paste0(myBgeeObject$pathToData, "/", organRelationshipsFileName, ".tmp"))

    if (success == 0){
      ## Read 5 last lines of file: should be empty indicating success of data transmission
      ## We cannot use a system call to UNIX command since some user might be on Windows
      tmp <- tail(read.table(paste0(myBgeeObject$pathToData, "/", organRelationshipsFileName, ".tmp"), header=TRUE, sep="\t", comment.char="", blank.lines.skip=FALSE, as.is=TRUE), n=5)
      if ( length(tmp[,1]) == 5 && (sum(tmp[,1] == "") == 5 || sum(is.na(tmp[,1])) == 5) ){
        ## The file transfer was successful, we rename the temporary file
        file.rename(paste0(myBgeeObject$pathToData, "/", organRelationshipsFileName, ".tmp"), paste0(myBgeeObject$pathToData, "/", organRelationshipsFileName))
      } else {
        ## delete the temporary file
        file.remove(paste0(myBgeeObject$pathToData, "/", organRelationshipsFileName, ".tmp"))
        stop(paste0("File ", organRelationshipsFileName, " is truncated, there may be a temporary problem with the Bgee webservice, or there was an error in the parameters."))
      }
      cat(paste0("   Got results from Bgee webservice. Files are written in \"", myBgeeObject$pathToData, "\"\n"))
    } else {
      serverAnswer = try(getURL(myUrl))
      if (class(serverAnswer) == "try-error"){
        stop("ERROR: the query to the server was not successful. Is your internet connection working?\n")
      } else {
        stop(paste0("ERROR: the query to the server was not successful. The server returned the following answer:\n", serverAnswer))
      }
    }
  }

  ## Second query: organ names
  organNamesFileName <- paste0("topAnat_AnatEntitiesNames_", myBgeeObject$speciesId, ".tsv");
  ## Check if file is already in cache
  if (file.exists(file.path(myBgeeObject$pathToData, organNamesFileName))){
    cat(paste0("\nNOTE: an organ names file was found in the download directory ", myBgeeObject$pathToData,
               ". Data will not be redownloaded.\n"))

  } else {
    cat("\nBuilding URLs to retrieve organ names from Bgee.................\n")
    myUrl <- myBgeeObject$topAnatUrl
    if(compareVersion(gsub("_", ".", myBgeeObject$release), OLD_WEBSERVICE_VERSION) > 0){
      myUrl <- paste0(myUrl, "?page=r_package&action=get_anat_entities&display_type=tsv&species_list=", myBgeeObject$speciesId, "&attr_list=ID&attr_list=NAME&api_key=", myBgeeObject$apiKey, "&source=BgeeDB_R_package&source_version=", as.character(packageVersion("BgeeDB")))
    }else {
      myUrl <- paste0(myUrl, "?page=dao&action=org.bgee.model.dao.api.anatdev.AnatEntityDAO.getAnatEntities&display_type=tsv&species_list=", myBgeeObject$speciesId, "&attr_list=ID&attr_list=NAME&api_key=", myBgeeObject$apiKey, "&source=BgeeDB_R_package&source_version=", as.character(packageVersion("BgeeDB")))
    }

    ## Query webservice
    cat(paste0("   URL successfully built (", myUrl,")\n   Submitting URL to Bgee webservice (can be long)\n"))
    success <- bgee_download_file(url = myUrl, destfile = paste0(myBgeeObject$pathToData, "/", organNamesFileName, ".tmp"))

    if (success == 0){
      ## Read 5 last lines of file: should be empty indicating success of data transmission
      ## We cannot use a system call to UNIX command since some user might be on Windows
      tmp <- tail(read.table(paste0(myBgeeObject$pathToData, "/", organNamesFileName, ".tmp"), header=TRUE, sep="\t", comment.char="", blank.lines.skip=FALSE, as.is=TRUE, quote = ""), n=5)
      if ( length(tmp[,1]) == 5 && (sum(tmp[,1] == "") == 5 || sum(is.na(tmp[,1])) == 5) ){
        ## The file transfer was successful, we rename the temporary file
        file.rename(paste0(myBgeeObject$pathToData, "/", organNamesFileName, ".tmp"), paste0(myBgeeObject$pathToData, "/", organNamesFileName))
      } else {
        ## delete the temporary file
        file.remove(paste0(myBgeeObject$pathToData, "/", organNamesFileName, ".tmp"))
        stop(paste0("File ", organNamesFileName, " is truncated, there may be a temporary problem with the Bgee webservice, or there was an error in the parameters."))
      }
     cat(paste0("   Got results from Bgee webservice. Files are written in \"", myBgeeObject$pathToData, "\"\n"))
    } else {
      serverAnswer = try(getURL(myUrl))
      if (class(serverAnswer) == "try-error"){
        stop("ERROR: the query to the server was not successful. Is your internet connection working?\n")
      } else {
        stop(paste0("ERROR: the query to the server was not successful. The server returned the following answer:\n", serverAnswer))
      }
    }
  }

  ## Third query: gene to organs mapping
  
  # The Java API does not distinguish between full length and droplet based single cell. A topAnat analysis can be run
  # on all single cell data but not on data coming from a subset of single cell technologies.
  # Bgee objects have been designed to allow the download of expression files for any datatype and then distinguish between
  # full length and droplet based single cell.
  # In order to solve that mismatch we update the type used to run topAnat analysis. If either sc_full_length or sc_droplet_based
  # datatype is selected, we run a topAnat analysis including all single cell technologies (full length AND droplet based)

  # First write a warning if only one single cell technology is selected
  if ("sc_full_length" %in% myBgeeObject$dataType & ! "sc_droplet_based" %in% myBgeeObject$dataType | 
    "sc_droplet_based" %in% myBgeeObject$dataType & ! "sc_full_length" %in% myBgeeObject$dataType) {
    message("WARNING: TopAnat can not be run on one single cell technology. Both full length and droplet based single cell data will",
      " be queried for this topAnat analysis. If you do not want to query single cell data please remove \"sc_full_length\" or \"sc_droplet_based\"",
      " from the list of datatypes of your Bgee object.")
  }

  # Then update the list of datatypes used to run topAnat
  topAnat_dataType <- myBgeeObject$dataType
  if ("sc_full_length" %in% topAnat_dataType | "sc_droplet_based" %in% topAnat_dataType) {
    topAnat_dataType <- topAnat_dataType[! topAnat_dataType %in% c("sc_full_length", "sc_droplet_based")]
    topAnat_dataType <- append(topAnat_dataType, "sc_rna_seq")
  }

  gene2anatomyFileName <- paste0("topAnat_GeneToAnatEntities_", myBgeeObject$speciesId, "_", toupper(callType))
  ## If a stage is specified, add it to file name
  if ( !is.null(stage) ){
    gene2anatomyFileName <- paste0(gene2anatomyFileName, "_", gsub(":", "_", stage))
  }
  ## If all data types specified, no need to add anything to file name. Otherwise, specify data types in file name
  if ( sum(topAnat_dataType %in% c("rna_seq","affymetrix","est","in_situ", "sc_rna_seq")) < 5 ){
    gene2anatomyFileName <- paste0(gene2anatomyFileName, "_", toupper(paste(sort(topAnat_dataType), collapse="_")))
  }
  ## If high quality data needed, specify in file name. Otherwise not specified
  if(compareVersion(gsub("_", ".", myBgeeObject$release), OLD_WEBSERVICE_VERSION) > 0){
    gene2anatomyFileName <- paste0(gene2anatomyFileName, "_", toupper(confidence))
  } else {
    if ( confidence == "high_quality" ){
      gene2anatomyFileName <- paste0(gene2anatomyFileName, "_HIGH")
    }
  }
  gene2anatomyFileName <- paste0(gene2anatomyFileName, ".tsv")

  ## Check if file is already in cache
  if (file.exists(file.path(myBgeeObject$pathToData, gene2anatomyFileName))){
    cat(paste0("\nNOTE: a gene to organs mapping file was found in the download directory ", myBgeeObject$pathToData,
               ". Data will not be redownloaded.\n"))

  } else {
    cat("\nBuilding URLs to retrieve mapping of gene to organs from Bgee...\n")
    myUrl <- myBgeeObject$topAnatUrl
    if(compareVersion(gsub("_", ".", myBgeeObject$release), OLD_WEBSERVICE_VERSION) > 0){
      myUrl <- paste0(myBgeeObject$topAnatUrl, "?page=r_package&action=get_expression_calls&display_type=tsv&species_list=", myBgeeObject$speciesId, "&attr_list=GENE_ID&attr_list=ANAT_ENTITY_ID&api_key=", myBgeeObject$apiKey, "&source=BgeeDB_R_package&source_version=", as.character(packageVersion("BgeeDB")))
    }else {
      myUrl <- paste0(myUrl, "?page=dao&action=org.bgee.model.dao.api.expressiondata.ExpressionCallDAO.getExpressionCalls&display_type=tsv&species_list=", myBgeeObject$speciesId, "&attr_list=GENE_ID&attr_list=ANAT_ENTITY_ID&api_key=", myBgeeObject$apiKey, "&source=BgeeDB_R_package&source_version=", as.character(packageVersion("BgeeDB")))
    }

    ## Add data type to file name: only if not all data types asked
    if ( sum(topAnat_dataType %in% c("rna_seq","sc_rna_seq","affymetrix","est","in_situ")) < 5 ){
      for (type in toupper(sort(topAnat_dataType))){
        myUrl <- paste0(myUrl, "&data_type=", type)
      }
    }
    ## Add data quality
    if(compareVersion(gsub("_", ".", myBgeeObject$release), OLD_WEBSERVICE_VERSION) > 0){
      myUrl <- paste0(myUrl, "&data_qual=", toupper(confidence))
    } else {
      if(confidence == "high_quality"){
        myUrl <- paste0(myUrl, "&data_qual=HIGH")
      }
    }

    if ( !is.null(stage) ){
      myUrl <- paste0(myUrl, "&stage_id=", stage)
    }

    ## Query webservice
    cat(paste0("   URL successfully built (", myUrl,")\n   Submitting URL to Bgee webservice (can be long)\n"))
    ## this download correspond to a file that can either be 1) generated on the fly if it is the first time this combination of conditions
    ## is queried or 2) stored on our server otherwise.
    ## If the file has to be generated it can take a lot of time (sometimes more than one hour if single cell data exist). In order to solve
    ## download errors due to apache connection stopping while the file is still generating and then throw an error 502, we decided to check
    ## the error retrieved by that download and restart the download if a specific error was retrieved.
    catchedError <- 1
    while (catchedError) {
      startTime <- Sys.time()
      tryCatch(
        {
          bgee_download_file(url = myUrl, destfile = paste0(myBgeeObject$pathToData, "/", gene2anatomyFileName, ".tmp"))
          catchedError <- 0
        },
        error = function(x) {
          #If it took a lot of time to throw an error there is a high probability that the file is currently generated.
          # In that case we just wait one minute and try to download the file again
          # by default the timeout is 60 seconds. In order to be safe we check that the error took more than 50
          # 40 sec to be thrown
          stopTime <- Sys.time()
          if (as.numeric(difftime(time1 = stopTime, time2 = startTime, units = "sec")) <= 40) {
            stop(paste0("ERROR: the query to the server was not successful. The server returned the following answer:\n", x))
          }
        },
        warning = function(x) {}
      )
    }

    tmp <- tail(read.table(paste0(myBgeeObject$pathToData, "/", gene2anatomyFileName, ".tmp"), header=TRUE, sep="\t", comment.char="", blank.lines.skip=FALSE, as.is=TRUE), n=5)
    if ( length(tmp[,1]) == 5 && (sum(tmp[,1] == "") == 5 || sum(is.na(tmp[,1])) == 5) ){
      ## The file transfer was successful, we rename the temporary file
      file.rename(paste0(myBgeeObject$pathToData, "/", gene2anatomyFileName, ".tmp"), paste0(myBgeeObject$pathToData, "/", gene2anatomyFileName))
    } else {
      ## delete the temporary file
      file.remove(paste0(myBgeeObject$pathToData, "/", gene2anatomyFileName, ".tmp"))
      stop(paste0("File ", gene2anatomyFileName, " is truncated, there may be a temporary problem with the Bgee webservice, or there was an error in the parameters."))
    }
    cat(paste0("   Got results from Bgee webservice. Files are written in \"", myBgeeObject$pathToData, "\"\n"))
  }

  ## Process the data and build the final list to return
  cat("\nParsing the results.............................................\n")

  ## Relationships between organs
  if (file.exists(file.path(myBgeeObject$pathToData, organRelationshipsFileName))){
    if (file.info(file.path(myBgeeObject$pathToData, organRelationshipsFileName))$size != 0) {
      tab <- read.table(file.path(myBgeeObject$pathToData, organRelationshipsFileName), header=TRUE, sep="\t", blank.lines.skip=TRUE, as.is=TRUE)
      organRelationships <- tapply(as.character(tab$TARGET_ID), as.character(tab$SOURCE_ID), unique)
    } else {
      stop(paste0("File ", organRelationshipsFileName, " is empty, there may be a temporary problem with the Bgee webservice, or there was an error in the parameters."))
    }
  } else {
    stop(paste0("File ", organRelationshipsFileName, " not found. There may be a temporary problem with the Bgee webservice, or there was an error in the parameters."))
  }
  ## Organ names
  if (file.exists(file.path(myBgeeObject$pathToData, organNamesFileName))){
    if (file.info(file.path(myBgeeObject$pathToData, organNamesFileName))$size != 0) {
      organNames <- read.table(file.path(myBgeeObject$pathToData, organNamesFileName), header=TRUE, sep="\t", comment.char="", blank.lines.skip=TRUE, as.is=TRUE, quote = "")
    } else {
      stop(paste0("File ", organNamesFileName, " is empty, there may be a temporary problem with the Bgee webservice, or there was an error in the parameters."))
    }
  } else {
    stop(paste0("File ", organNamesFileName, " not found. There may be a temporary problem with the Bgee webservice, or there was an error in the parameters."))
  }
  ## Mapping of genes to tissues
  if (file.exists(file.path(myBgeeObject$pathToData, gene2anatomyFileName))){
    if (file.info(file.path(myBgeeObject$pathToData, gene2anatomyFileName))$size != 0) {
      tab <- read.table(file.path(myBgeeObject$pathToData, gene2anatomyFileName), header=TRUE, sep="\t", blank.lines.skip=TRUE, as.is=TRUE)
      if(length(tab$GENE_ID) != 0){
        gene2anatomy <- tapply(as.character(tab$ANAT_ENTITY_ID), as.character(tab$GENE_ID), unique)
      } else {
        stop("There was no mapping of genes to anatomical structures found. Probably the parameters are too stringent, or this data type is absent in this species. See listBgeeSpecies() for data types availability.")
      }
    } else {
      stop(paste0("File ", gene2anatomyFileName, " is empty, there may be a temporary problem with the Bgee webservice, or there was an error in the parameters. It is also possible that the parameters are too stringent and returned no data, please try to relax them."))
    }
  } else {
    stop(paste0("File ", gene2anatomyFileName, " not found. There may be a temporary problem with the Bgee webservice, or there was an error in the parameters. It is also possible that the parameters are too stringent and returned no data, please try to relax them."))
  }

  cat("\nAdding BGEE:0 as unique root of all terms of the ontology.......\n")
  ## There can be multiple roots among all the terms downloaded. We need to add one unique root for topGO to work: BGEE:0
  ## Add all organs from organNames that are not source (child / names of the list) in organsRelationship to the organsRelationship list (with value / target / parent = BGEE:0)
  ## Some organs are not present in the organRelationships file because they have no relations to other organs (not linked to a target).
  ## That's why we use all organs present in organNames to find missingParents
  missingParents <- setdiff(organNames[, 1], names(organRelationships))

  ## Add new values
  organRelationships <- c(organRelationships, as.list(rep("BGEE:0", times=length(missingParents))))
  ## Add new keys
  names(organRelationships)[(length(organRelationships) - length(missingParents) + 1):length(organRelationships)] = as.character(missingParents)
  ## Add BGEE:0	/ root to organNames
  organNames <- rbind(organNames, c("BGEE:0", "root"))

  ## Check if some organ names are missing, and add them if necessary
  missingNames <- setdiff(unique(unique(unlist(organRelationships, use.names = FALSE)), names(organRelationships)), organNames$ID)
  if (length(missingNames) > 0){
    cat(paste0("\nWARNING: some organs names appear to be missing. There might be some problem with the ontology data.\n"))
    organNames <- rbind(organNames, setNames(data.frame(missingNames, "?"), names(organNames)))
  }

  cat("\nDone.\n")
  return(list(gene2anatomy = gene2anatomy, organ.relationships = organRelationships, organ.names = organNames, bgee.object = myBgeeObject))
}

context("expect_output")


test_that("Loading of topAnatData files is working", {
  bgee <- Bgee$new(species="Bos_taurus", dataType="rna_seq")
  myTopAnatData <- loadTopAnatData(bgee, stage="UBERON:0000092")

  expect_that( myTopAnatData, is_a("list") )
  expect_that( length(myTopAnatData), equals(4) )
  expect_that( myTopAnatData$gene2anatomy, is_a("array") )
  expect_that( myTopAnatData$organ.relationships, is_a("list") )
  expect_that( myTopAnatData$organ.names, is_a("data.frame") )
  expect_that( myTopAnatData$bgee.object, is_a("Bgee") )
  expect_message( message("Done.") )

})

#' @title Retrieve Bgee processed expression values for bulk RNA-seq, full length single-cell RNA-seq,
#'  droplet based single-cell RNA-seq or Affymetrix data.
#'
#' @description This function loads the quantitative expression data and presence calls 
#' for samples available from Bgee. These data are available for all droplet based single-cell
#' RNA-seq, full length single-cell RNA-seq, bulk RNA-seq and affymetrix samples present in Bgee.
#' It is possible to filter the processed expression data using experiment IDs, chip IDs (affymetrix)
#' or library IDs (RNA-seq), anatomical entity IDs from the UBERON ontology, developmental stage IDs,
#' cell-type IDs, sex and/or strain. Celltype, anatomical entity and stage IDs are normalized and come from
#' ontologies, allowing to retrieve data annotated with the selected term or with the term and all its descendants.
#' This option allows for instance to retrieve all processed expression values coming from the brain or
#' any subpart of the brain.
#'
#' @param myBgeeObject A Reference Class Bgee object, notably specifying the targeted species 
#' and data type.
#'
#' @param experimentId Filter allowing to specify one or more ArrayExpress or GEO accession, e.g., 
#' GSE43721. Default is NULL: takes all available experiments for targeted species and data type.
#' 
#' @param sampleId Filter allowing to specify one or more sample ID. Depending on the selected 
#' datatype this sample IDs can correspond to Chip IDs (affymetrix) or RNA-Seq library IDs (rna_seq). 
#' Default is NULL: takes all available samples for targeted species and data type.
#'
#' @param anatEntityId Filter allowing to specify one or more anatomical entity IDs from the UBERON 
#' ontology (http://uberon.github.io/). Default is NULL: takes all available anatomical entities for 
#' targeted species and data type.
#'
#' @param stageId Filter allowing to specify one or more developmental stage IDs from Developmental 
#' Stage Ontology (https://github.com/obophenotype/developmental-stage-ontologies). Default is 
#' NULL: takes all available developmental stages for targeted species and data type.
#' 
#' @param cellTypeId Filter specific to single cell datatype (sc_full_length) allowing to specify 
#' one or more cell type IDs from the UBERON ontology (http://uberon.github.io/). Default is 
#' NULL: takes all available cell types for targeted species and data type. Available for Bgee 15.0 and after
#' 
#' @param sex Filter allowing to specify one or more sexes. Default is 
#' NULL: takes all available sexes for targeted species and data type. Available for Bgee 15.0 and after
#' 
#' @param strain Filter allowing to specify one or more strains. Default is 
#' NULL: takes all available strains for targeted species and data type. Available for Bgee 15.0 and after
#' 
#' @param withDescendantAnatEntities Allows to filter on the selected anatEntityId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#'
#' @param withDescendantStages Allows to filter on the selected stageId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#'
#' @param withDescendantCellTypes Allows to filter on the selected cellTypeId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#'
#' @return Return a dataframe containing all Bgee processed expression data from the selected species 
#' and datatype using specified filters with operator AND.
#'
#' @author Julien Wollbrett, Andrea Komljenovic and Julien Roux.
#'
#' @examples{
#'   bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
#'   dataMouseGSE43721 <- getSampleProcessedData(bgee, experimentId = "GSE43721")
#'   dataMouseVariousFilters <- getSampleProcessedData(bgee, experimentId = c("GSE43721", "GSE36026"), 
#'                              anatEntityId = c("UBERON:0002107", "UBERON:0000956", "UBERON:0002048"))
#' }
#'
#' @import RSQLite
#' @importFrom R.utils gunzip
#' @export
#' 
getSampleProcessedData <- function(myBgeeObject, experimentId = NULL, sampleId = NULL,
                                   anatEntityId = NULL, stageId = NULL, cellTypeId = NULL, sex = NULL, strain = NULL,
                                   withDescendantAnatEntities = FALSE, withDescendantStages = FALSE,
                                   withDescendantCellTypes = FALSE) {
  check_object(myBgeeObject)
  # write a warning message if user tried to retrieve ontology terms with descendants for
  # a bgee release where this functionality was not yet implemented
  compare_version_number <- gsub("_", ".", myBgeeObject$release)
  if ((withDescendantAnatEntities || withDescendantStages || withDescendantCellTypes) &
      compareVersion(a = compare_version_number , b = "15.0") < 0) {
    message("withDescendant functionality is available only for Bgee 15.0",
            " release and after. Will not retrieve descendant of selected parameters.")
  }
  if (withDescendantAnatEntities && compareVersion(a = compare_version_number , b = "15.0") >= 0) {
    if(is.null(anatEntityId)) {
      warning("No anatomical entity was provided. Not possible to filter on descendant anatomical entities.")
    }
    anatEntityId <- c(anatEntityId, getDescendantAnatEntities(bgee = myBgeeObject, ids = anatEntityId))
  }
  if (withDescendantStages && compareVersion(a = compare_version_number , b = "15.0") >= 0) {
    if(is.null(stageId)) {
      warning("No developmental stage was provided. Not possible to filter on descendant developmental stages.")
    }
    stageId <- c(stageId, getDescendantStages(bgee = myBgeeObject, ids = stageId))
  }
  if (withDescendantCellTypes && compareVersion(a = compare_version_number , b = "15.0") >= 0) {
    if(is.null(cellTypeId)) {
      warning("No cell type was provided. Not possible to filter on descendant cell types.")
    }
    cellTypeId <- c(cellTypeId, getDescendantCellTypes(bgee = myBgeeObject, ids = cellTypeId))
  }
  check_condition_parameters(myBgeeObject = myBgeeObject, anatEntityId = anatEntityId, 
                             stageId = stageId, cellTypeId = cellTypeId, sex = sex, strain = strain)
  import_data(myBgeeObject = myBgeeObject, experimentId = experimentId, sampleId = sampleId, 
              anatEntityId = anatEntityId, stageId = stageId, cellTypeId = cellTypeId, sex = sex, 
              strain = strain)
  return(query_data(myBgeeObject = myBgeeObject, experimentId = experimentId, sampleId = sampleId, 
                    anatEntityId = anatEntityId, stageId = stageId, cellTypeId = cellTypeId, sex = sex, 
                    strain = strain))
}

check_object = function(myBgeeObject, experimentId = NULL){
  ## check that the Bgee object is valid
  if (length(myBgeeObject$quantitativeData) == 0 ){
    stop("ERROR: there seems to be a problem with the input Bgee class object, some fields ", 
         "are empty. Please check that the object was correctly built.")
  } else {
    ## Check that download of quantitative data is possible for targeted species and data type
    ## Try to output error message that helps a bit the user
    if (myBgeeObject$quantitativeData != TRUE){
      if (length(myBgeeObject$dataType) > 1){
        stop("ERROR: downloading quantitative data is only possible if a single data ", 
             "type (\"rna_seq\", \"sc_full_length\", \"sc_droplet_based\" or \"affymetrix\") is specified ",
             "in the input Bgee class object.")
      } else if (length(myBgeeObject$dataType) == 1 && !(myBgeeObject$dataType %in% c('rna_seq','sc_full_length',
        'sc_droplet_based', 'affymetrix'))){
        stop("ERROR: downloading quantitative data is not possible for the species and data ", 
             "type specified in the input Bgee class object. Maybe the specified data type is not available ", 
             "for the targeted species in Bgee? See listBgeeSpecies() for details on data types availability for each species.")
      } else {
        stop("ERROR: downloading quantitative data is not possible for the species and data type ", 
             "specified in the input Bgee class object.")
      }
    } else if (length(myBgeeObject$experimentUrl) == 0 || length(myBgeeObject$allExperimentsUrl) == 0 
               || length(myBgeeObject$dataType) == 0 || length(myBgeeObject$pathToData) == 0){
      stop("ERROR: there seems to be a problem with the input Bgee class object, some fields are ", 
           "empty. Please check that the object was correctly built.")
    }
  }
}

check_condition_parameters = function(myBgeeObject, anatEntityId, stageId, 
                                      cellTypeId, sex, strain){
  ## check that the condition parameters queried are compatible with Bgee release
  ## selected
  if(compareVersion(a = gsub("_", ".", myBgeeObject$release), b = "15.0") < 0) {
    if (!is.null(cellTypeId) || !is.null(sex) || !is.null(strain)) {
      stop("ERROR: cellTypeId, sex, and strain can be filtered only for Bgee 15.0",
           " release and after.")
    }
  }
}

## extract the global path to the sqliteDB
sqlitePath <- function(myBgeeObject){
  sqlite_file <- file.path(myBgeeObject$pathToData, paste0(myBgeeObject$speciesName, 
                                                           myBgeeObject$sqlite_extension))
  return(sqlite_file)
}

import_data = function(myBgeeObject, experimentId = NULL, sampleId = NULL, 
                       anatEntityId = NULL, stageId = NULL, cellTypeId = NULL, sex = NULL, strain = NULL) {
  user_experiments <- detect_experiments(myBgeeObject, experimentId, sampleId, anatEntityId, 
                                         stageId, cellTypeId, sex, strain)
  # stop code if no experiments correspond to user criteria
  if(length(user_experiments) == 0) {
    stop("No data correspond to selected parameters. Please look at the annotation to ",
         "select available data.")
  }
  missing_experiments <- experiments_to_download(myBgeeObject = myBgeeObject, 
                                                 experimentId = user_experiments, sqlite_file = sqlitePath(myBgeeObject))
  integrate_experiments(myBgeeObject = myBgeeObject, experimentId = missing_experiments, 
                        sqlite_file = sqlitePath(myBgeeObject))
} 

# function using annotations to select which experiments the user needs to load all wanted data
detect_experiments = function(myBgeeObject, experimentId = NULL, sampleId = NULL, 
                              anatEntityId = NULL, stageId = NULL, cellTypeId = NULL, sex = NULL, strain = NULL) {
  annotation <- suppressMessages(getAnnotation(myBgeeObject))
  experiments <- annotation$sample.annotation
  # filter list of experiments to download
  if(!is.null(experimentId)) {
    experiments <- experiments[experiments$Experiment.ID %in% experimentId,]
  } 
  if(!is.null(sampleId)) {
    if(myBgeeObject$dataType == "rna_seq" || myBgeeObject$dataType == "sc_full_length" || myBgeeObject$dataType == "sc_droplet_based") {
      experiments <- experiments[experiments$Library.ID %in% sampleId,]
    } else if(myBgeeObject$dataType == "affymetrix") {
      experiments <- experiments[experiments$Chip.ID %in% sampleId,]
    } else {
      stop("unrecognized datatype : ", myBgeeObject$dataType)
    }
  }
  if(!is.null(anatEntityId)) {
    experiments <- experiments[experiments$Anatomical.entity.ID %in% anatEntityId,]
  }
  if(!is.null(stageId)) {
    experiments <- experiments[experiments$Stage.ID %in% stageId,]
  }
  if(!is.null(sex)) {
    # annotation files contain NA values. dplyr:na_if(sex, "NA") function allows to change
    # queried "NA" values to NA values
    experiments <- experiments[experiments$Sex %in% dplyr::na_if(sex, "NA"),]
  }
  if(!is.null(strain)) {
    experiments <- experiments[experiments$Strain %in% strain,]
  }
  if(!is.null(cellTypeId)) {
    if (myBgeeObject$dataType == "sc_full_length" || myBgeeObject$dataType == "sc_droplet_based") {
      #In Bgee 15.0 the name of the column was "Cell type ID" and was changed to "Celltype ID" starting from Bgee 15.2
      if (compareVersion(a = gsub("_", ".", myBgeeObject$release), b = "15.0") == 0) {
        experiments <- experiments[experiments$Cell.type.ID %in% cellTypeId,]
      } else {
        experiments <- experiments[experiments$Celltype.ID %in% cellTypeId,]
      }
    } else {
      stop("Can only filter on cell type ID when single cell datatype (sc_full_length or sc_droplet_based) is selected.")
    }
  }
  return(unique(experiments$Experiment.ID))
}

# function that check among experiment wanted by user which one are already stored in the local database.
# return experiments not already present in the database.
experiments_to_download = function(myBgeeObject, experimentId, sqlite_file) {
  if (file.exists(sqlite_file)) {
    conn <- dbConnect(RSQLite::SQLite(), sqlite_file)
    on.exit(dbDisconnect(conn))
    tables <- dbListTables(conn)
    if (isTRUE(myBgeeObject$dataType %in% tables)) {
      exp_queries <- paste0("Select distinct([Experiment.ID]) from ", myBgeeObject$dataType)
      existing_experiments <- dbGetQuery(conn, exp_queries)
      return(experimentId[(!experimentId %in% existing_experiments$Experiment.ID)])
    }
  }
  # if local database does not exist return all experiments
  return(experimentId)
}

# function that take care of both downloading and integrating data in the local database
# one local database is created per species (for the moment the package does not contain any
# multispecies functionality)
integrate_experiments = function(myBgeeObject, experimentId, sqlite_file) {
  if (!length(experimentId) == 0) {
    message("downloading data from Bgee FTP...")
    # If more than 15 experiments have to be downloaded then download all experiments of this 
    # species (in order not to download too many files from the Bgee FTP)
    if (length(experimentId) > 15) {
      message("You tried to download more than 15 experiments, because of that all the ", 
              "Bgee data for this species will be downloaded.")
      myData <- download_and_uncompress_species(myBgeeObject, experimentId)
    } else {
      myData <- download_and_uncompress_experiment(myBgeeObject, experimentId)
    }
    message("Save data in local sqlite database")
    conn <- dbConnect(RSQLite::SQLite(), sqlite_file)
    on.exit(dbDisconnect(conn))
    for(i in seq(myData)) {
      dbWriteTable(conn = conn, name = myBgeeObject$dataType, 
                   value = myData[i], header=TRUE, append=TRUE, sep="\t")
    }
    # those lines are mandatory as Rank and pValue can have NA values that
    # are transformed to 0 by dbQuery() function.
    if(compareVersion(a = gsub("_", ".", myBgeeObject$release), b = "15.0") >= 0) {
      updatePvalues <- dbExecute(conn, paste0("UPDATE ",myBgeeObject$dataType," set [pValue] = NULL 
        where [pValue] = \"NA\""))
      if(myBgeeObject$dataType == "rna_seq" || myBgeeObject$dataType == "sc_full_length"
         || myBgeeObject$dataType == "sc_droplet_based") {
        updateRanks <- dbExecute(conn, paste0("UPDATE ", myBgeeObject$dataType, " set [Rank] = NULL 
          where [Rank] = \"NA\""))
      }
    }
    unlink(myData)
  }
}

# function that download all data of one species for the specified datatype from the ftp
# return the name of all uncompressed files
download_and_uncompress_species = function(myBgeeObject, experimentId) {
  message("Downloading all expression data for species ", myBgeeObject$speciesName)
  
  #temporarily update timeout option
  op <- options(timeout = 1800)
  on.exit(options(op))
  
  allExpressionValues <- basename(myBgeeObject$allExperimentsUrl)
  success <- bgee_download_file(url = myBgeeObject$allExperimentsUrl, 
                                destfile = file.path(myBgeeObject$pathToData, allExpressionValues), mode = 'wb')
  if (success != 0){
    stop("ERROR: Download from FTP was not successful.")
  }
  # compressed files for Bgee 13.2
  if (grepl(".zip$", allExpressionValues, perl = TRUE)) {
    message("Saved expression data file in", myBgeeObject$pathToData, 
            "folder. Now unzip file...")
    allExperiments <- unzip(file.path(myBgeeObject$pathToData, allExpressionValues), 
                            exdir=myBgeeObject$pathToData)
    unlink(file.path(myBgeeObject$pathToData, allExpressionValues))
    tempFiles <- grep(paste0(".*", paste(experimentId, collapse=".*|.*"), ".*"), 
                      allExperiments, value = TRUE)
    myData <- unlist(lapply(tempFiles, unzip, exdir=myBgeeObject$pathToData))
    file.remove(allExperiments)
    # at this point remaining .zip files correspond to archive downloaded but not
    # asked by the user. They have to to be removed
    unlink(file.path(myBgeeObject$pathToData, "*.zip"))
    
    # compressed tarball for bgee14.0 and after
  } else if (grepl(".tar.gz$", allExpressionValues, perl = TRUE)) {
    message("Saved expression data file in ", myBgeeObject$pathToData, " folder. Now untar file...")
    # When using untar it is only possible to untar OR list files/dir present in a tarball. 
    # It is not possible to do both actions in one line of code
    allFilesAndDir <- untar(file.path(myBgeeObject$pathToData, allExpressionValues), 
                            exdir=myBgeeObject$pathToData)
    
    # list all files present in the tarball in order to be able to remove them all
    allFilesAndDir <- untar(file.path(myBgeeObject$pathToData, allExpressionValues), 
                            exdir=myBgeeObject$pathToData, list = TRUE)
    
    # keep only path to files to integrate to the local database. 
    toIntegrateFilesAndDir <- grep(paste0(".*",paste(experimentId,collapse=".*|.*"),".*"), 
                                   allFilesAndDir, value = TRUE)
    
    # retrieve absolute path to files
    toIntegrateFilesAndDir <- file.path(myBgeeObject$pathToData, toIntegrateFilesAndDir)
    
    myData <- NULL
    for(i in seq(toIntegrateFilesAndDir)) {
      # uncompress files and not directories (there was a bug when tar tried to uncompress 
      # GTeX experiment)
      # added this verification to be sure there will not be error with other experiments/species
      if (isTRUE(file_test("-f", toIntegrateFilesAndDir[i]))) {
        # uncompress expression files for Bgee 14
        if (grepl(".tar.gz$", toIntegrateFilesAndDir[i], perl = TRUE)) {
          currentData <-untar(toIntegrateFilesAndDir[i], exdir=myBgeeObject$pathToData)
          # list all expression files
          currentData <- file.path(myBgeeObject$pathToData, untar(toIntegrateFilesAndDir[i], 
                                                                  exdir=myBgeeObject$pathToData, list = TRUE))
          # uncompress expression files for Bgee 15 and after
        } else if (grepl(".tsv.gz$", toIntegrateFilesAndDir[i], perl = TRUE)) {
          fileNameUncompressed <- gsub(".gz", "", basename(toIntegrateFilesAndDir[i]))
          currentData <- unlist(gunzip(toIntegrateFilesAndDir[i], remove=FALSE, overwrite = TRUE,
                                       destname = file.path(myBgeeObject$pathToData, fileNameUncompressed)))
        }
        myData <- c(myData, currentData)
      } 
    }
    
    # delete intermediary archives
    unlink(file.path(myBgeeObject$pathToData, allExpressionValues))
    unlink(file.path(myBgeeObject$pathToData, allFilesAndDir))
    
    # Until Bgee 15.0 rna_seq species tarball contained a directory containing all
    # experiment files. This directory had to be deleted. We use this trick to delete
    # a potential directory because we are not 100% sure that a directory was always 
    # present for rna_seq data of Bgee 13.2 and 14.0
    parentDirUncompressedFiles <- dirname(toIntegrateFilesAndDir[1])
    if(parentDirUncompressedFiles != myBgeeObject$pathToData) {
      unlink(parentDirUncompressedFiles, recursive = TRUE)
    }
    
    message("Finished uncompress tar files")
  }
  return(myData)
}

# function that download a list of experiments from the ftp and return the name of downloaded
# uncompressed files
download_and_uncompress_experiment = function(myBgeeObject, experimentId) {
  message("download experiments")
  #temporarily update timeout option
  op <- options(timeout = 1800)
  # on.exit(options(op))
  tempFiles <- NULL
  for (i in seq(experimentId)) {
    finalExperimentUrl <- gsub("EXPIDPATTERN", experimentId[i], myBgeeObject$experimentUrl)
    tempFile <- file.path(myBgeeObject$pathToData, basename(finalExperimentUrl))
    success <- bgee_download_file(url = finalExperimentUrl, destfile = tempFile, mode = 'wb')
    if (success != 0){
      stop("ERROR: Download from FTP was not successful. Check the experiments present ", 
           "in Bgee with the getAnnotation() function.")
    }
    tempFiles <- c(tempFiles, tempFile)
  }
  message("uncompress processed expression data...")
  if (grepl(".tar.gz$", tempFiles[1], perl = TRUE)) {
    myData <- file.path(myBgeeObject$pathToData, lapply(tempFiles, untar, 
                                                        exdir=myBgeeObject$pathToData))
    myData <- file.path(myBgeeObject$pathToData, unlist(lapply(tempFiles, untar, 
                                                               exdir=myBgeeObject$pathToData, list = TRUE)))
  } else if (grepl(".zip$", tempFiles[1], perl = TRUE)) {
    myData <- unlist(lapply(tempFiles, unzip, exdir=myBgeeObject$pathToData))
  } else if (grepl(".tsv.gz$", tempFiles[1], perl = TRUE)) {
    myData <- unlist(lapply(tempFiles, gunzip, remove=FALSE, overwrite = TRUE))
  }
  unlink(tempFiles)
  return(myData)
}

# function that query the data in the local database
query_data = function(myBgeeObject, experimentId = NULL, sampleId = NULL, anatEntityId = NULL, 
                      stageId = NULL, cellTypeId = NULL, sex = NULL, strain = NULL, 
                      sqlite_file = sqlitePath(myBgeeObject)) {
  conn <- dbConnect(RSQLite::SQLite(), sqlite_file)
  on.exit(dbDisconnect(conn))
  # generate query
  query <- paste0("SELECT * from ", myBgeeObject$dataType)
  if(! (is.null(experimentId) && is.null(sampleId) && is.null(anatEntityId) && is.null(stageId)
        && is.null(sex) && is.null(strain) && is.null(cellTypeId)) ) {
    query <- paste0(query, " WHERE ")
    if (!is.null(experimentId)) {
      if(length(experimentId) == 1) {
        query <- paste0(query, "[Experiment.ID] = \"",as.character(experimentId), "\"")
      } else {
        query <- paste0(query, "[Experiment.ID] IN (\"",paste(as.character(experimentId), collapse="\", \""), "\")")
      }
    }
    if (!is.null(sampleId)) {
      if (!is.null(experimentId)) {
        query <- paste0(query, " AND ")
      }
      if(myBgeeObject$dataType %in% c("rna_seq", "sc_full_length", "sc_droplet_based")) {
        if (length(sampleId) == 1 ) {
          query <- paste0(query, "[Library.ID] = \"", as.character(sampleId), "\"")
        } else {
          query <- paste0(query, "[Library.ID] IN (\"",paste(as.character(sampleId), collapse="\", \""), "\")")
        }
      } else if (myBgeeObject$dataType == "affymetrix") {
        if (length(sampleId) == 1 ) {
          query <- paste0(query, "[Chip.ID] = \"", as.character(sampleId), "\"")
        } else {
          query <- paste0(query, "[Chip.ID] IN (\"",paste(as.character(sampleId), collapse="\", \""), "\")")
        }
      }
    }
    if (!is.null(anatEntityId)) {
      if (!(is.null(experimentId) && is.null(sampleId))) {
        query <- paste0(query, " AND ")
      }
      if(length(anatEntityId) == 1) {
        query <- paste0(query, "[Anatomical.entity.ID] = \"", as.character(anatEntityId), "\"")
      } else {
        query <- paste0(query, "[Anatomical.entity.ID] IN (\"",paste(as.character(anatEntityId), collapse="\", \""), "\")")
      }
    }
    if (!is.null(stageId)) {
      if (!(is.null(experimentId) && is.null(sampleId) && is.null(anatEntityId))) {
        query <- paste0(query, " AND ")
      }
      if(length(stageId) == 1) {
        query <- paste0(query, "[Stage.ID] = \"", as.character(stageId), "\"")
      } else {
        query <- paste0(query, "[Stage.ID] IN (\"",paste(as.character(stageId), 
                                                         collapse="\", \""), "\")")
      }
    }
    # All following filters are available for Bgee 15.0 and after
    if(compareVersion(a = gsub("_", ".", myBgeeObject$release), b = "15.0") >= 0) {
      if (!is.null(cellTypeId)) {
        if (!(is.null(experimentId) && is.null(sampleId) && is.null(anatEntityId) 
              && is.null(stageId))) {
          query <- paste0(query, " AND ")
        }

        # Name of the column has been updated after Bgee 15.0
        cellTypeColumnName <- "Celltype.ID"
        if(compareVersion(a = gsub("_", ".", myBgeeObject$release), b = "15.0") == 0) {
          cellTypeColumnName <- "Cell.type.ID"
        }

        if(length(cellTypeId) == 1) {
          query <- paste0(query, "[", cellTypeColumnName, "] = \"", as.character(cellTypeId), "\"")
        } else {
          query <- paste0(query, "[", cellTypeColumnName, "] IN (\"",paste(as.character(cellTypeId), 
                                                               collapse="\", \""), "\")")
        }
      }
      if (!is.null(sex)) {
        if (!(is.null(experimentId) && is.null(sampleId) && is.null(anatEntityId) 
              && is.null(stageId) && is.null(cellTypeId))) {
          query <- paste0(query, " AND ")
        }
        if(length(sex) == 1) {
          query <- paste0(query, "[Sex] = \"", as.character(sex), "\"")
        } else {
          query <- paste0(query, "[Sex] IN (\"",paste(as.character(sex), 
                                                      collapse="\", \""), "\")")
        }
      }
      if (!is.null(strain)) {
        if (!(is.null(experimentId) && is.null(sampleId) && is.null(anatEntityId) 
              && is.null(stageId) && is.null(cellTypeId) && is.null(sex))) {
          query <- paste0(query, " AND ")
        }
        # rsqlite does not allow to remove double quotes when inserting data in the local database.
        # strain data are surrounded by " in the tsv files. These quotes allows to have
        # strain with a name containing special characters.
        # Following the SQL syntax, it is mandatory to provide 2 double quotes in order
        # escape existing double quotes in the database.
        if(length(strain) == 1) {
          query <- paste0(query, "[Strain] = \"\"\"", as.character(strain), "\"\"\"")
        } else {
          query <- paste0(query, "[Strain] IN (\"\"\"",paste(as.character(strain), 
                                                             collapse="\"\"\", \"\"\""), "\"\"\")")
        }
      }
    }
  }
  message("Load queried data. The query is : ", query)
  result <- dbGetQuery(conn, query)
  return(result)
}

getDescendantAnatEntities <- function (bgee, ids) {
  return(getDescendant(bgee = bgee, ids = ids, conditionParam = "anatEntities"))
}

getDescendantCellTypes <- function (bgee, ids) {
  return(getDescendant(bgee = bgee, ids = ids, conditionParam = "cellTypes"))
}

getDescendantStages <- function (bgee, ids) {
  return(getDescendant(bgee = bgee, ids = ids, conditionParam = "stages"))
}

getDescendant <- function (bgee, ids, conditionParam) {
  myUrl <- paste0(bgee$topAnatUrl,
                  "?page=r_package&action=COND_PARAM&ENTITIES&species_id=SPECIES&",
                  "propagation=DESCENDANTS&attr_list=ID&display_type=tsv")
  myUrl <- gsub("SPECIES", bgee$speciesId, myUrl, perl = FALSE)
  if (conditionParam == "anatEntities" || conditionParam == "cellTypes") {
    myUrl <- gsub("COND_PARAM", "get_propagation_anat_entity", myUrl, perl = TRUE)
    myUrl <- gsub("ENTITIES", paste0("anat_entity_id=",
                                     paste(ids, collapse = "&anat_entity_id=")), myUrl, perl = TRUE)
  } else if (conditionParam == "stages") {
    myUrl <- gsub("COND_PARAM", "get_propagation_dev_stage", myUrl, perl = TRUE)
    myUrl <- gsub("ENTITIES", paste0("stage_id=",
                                     paste(ids, collapse = "&stage_id=")), myUrl, perl = TRUE)
  }
  destFile <- file.path(bgee$pathToData, "descendants.tsv")
  success <- bgee_download_file(url = myUrl, destfile = destFile)
  descendants <- read.table(destFile, header = TRUE, sep = "\t")
  # retrieve annotation in order to keep only descendants present in the annotation
  annotation <- suppressMessages(getAnnotation(bgee)$sample.annotation)
  if (conditionParam == "anatEntities") {
    present <- unique(annotation$Anatomical.entity.ID)
  } else if (conditionParam == "cellTypes") {
    if (gsub("_", ".", myBgeeObject$release) == "15.0") {
      present <- unique(annotation$Cell.type.ID)
    } else {
      present <- unique(annotation$Celltype.ID)
    }
  } else if (conditionParam == "stages") {
    present <- unique(annotation$Stage.ID)
  }
  # Do not use column name because it is not the same depending on the
  # queried condition parameter
  wanted_descendants <- descendants[descendants[,1] %in% present,]
  unlink(destFile)
  return(wanted_descendants)
}

#' @title List species in the Bgee database and the available data types for each of them
#'
#' @description Returns information on available species in Bgee
#'
#' @param release A character specifying a targeted release number. In the form "Release.subrelease" or "Release_subrelease", e.g., "13.2" or 13_2". If not specified, the latest release is used.
#'
#' @param allReleases A data frame with information on all releases. Avoid redownloading this information if .getBgeeRelease() already called.
#'
#' @param ordering A numeric indicating the number of the column which should be used to sort the data frame. Default NULL, returning unsorted data frame.
#'
#' @param removeFile Boolean indicating whether the downloaded file should be deleted. Default to TRUE.
#'
#' @return A data frame with species Id, genus name, species name, common name and data type availability for targeted Bgee release
#'
#' @examples{
#'  listBgeeSpecies()
#'  # species present in a specific Bgee release
#'  listBgeeSpecies(release = "13.2")
#'  # in order to order species according to theit taxonomical IDs
#'  listBgeeSpecies(ordering = 1)
#' }
#'
#' @author Julien Roux
#' @export

listBgeeSpecies <- function(release=NULL, ordering=NULL, allReleases=NULL, removeFile=TRUE){
  
  OLD_WEBSERVICE_VERSION = '13.2'
  NEW_VERSION_FROM_FTP_FILE = '15.2'

  if (length(allReleases)==0) {
    cat("\nQuerying Bgee to get release information...\n")
    allReleases <- .getBgeeRelease()
  }
  if (length(release)==0) {
    ## Take latest release
    release <- gsub("\\.", "_", allReleases$release[1])
  } else if (length(release)==1){
    # In case the release number is written with a dot
    release <- gsub("\\.", "_", release)
    # test if required release exists
    if (sum(as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release)))!=1){
      stop("ERROR: The specified release number is invalid, or is not available for BgeeDB.")
    }
  } else {
    stop("ERROR: The specified release number is invalid.")
  }
  
  speciesFilePath <- paste0(getwd(), "/species_Bgee_", release, ".tsv")
  cat(paste0("\nBuilding URL to query species in Bgee release ", release, "...\n"))
  
  myUrl <- allReleases$TopAnat.URL[as.numeric(allReleases$release) == as.numeric(gsub("_", ".", release))]
  
  #create the url depending on selected release of Bgee. It uses a webservice for releases before 15.2 and a file starting from 15.2
  if(compareVersion(gsub("_", ".",release), OLD_WEBSERVICE_VERSION) > 0){
    if (compareVersion(gsub("_", ".",release), NEW_VERSION_FROM_FTP_FILE) >= 0){
      myUrl <- file.path(paste0("https://www.bgee.org/ftp/bgee_v", release), "rPackageSpeciesInfo.tsv")
    } else {
      myUrl <- paste0(myUrl, "?page=r_package&action=get_all_species&display_type=tsv&source=BgeeDB_R_package&source_version=", packageVersion("BgeeDB"))
    }
  } else {
    myUrl <- paste0(myUrl, "?page=species&display_type=tsv&source=BgeeDB_R_package&source_version=", packageVersion("BgeeDB"))
  }

  ## Set the internet.info to 2 to have less verbose output (only reports critical warnings)
  options(internet.info=2)
  ## Set the timeout option to 600 seconds to let some time to the server to send data (default is 60s)
  options(timeout = 600)

  ## Query webservice
  cat(paste0("\ndownloading Bgee species info... (", myUrl,")\n"))
  bgee_download_file(url = myUrl, destfile = speciesFilePath)
  ## Read 5 last lines of file: should be empty indicating success of data transmission
  ## We cannot use a system call to UNIX command since some user might be on Windows
  tmp <- tail(read.table(speciesFilePath,
                         header=TRUE,
                         sep="\t",
                         comment.char="",
                         blank.lines.skip=FALSE,
                         as.is=TRUE),
              n=5)
  if ( length(tmp[,1]) == 5 && (sum(tmp[,1] == "") == 5 || sum(is.na(tmp[,1])) == 5) ){
    ## The file transfer was successful!
    cat(paste0("\nDownload of species information successful!\n"))
    allSpecies <- read.table(speciesFilePath,
                             header=TRUE,
                             sep="\t",
                             blank.lines.skip=TRUE,
                             as.is=TRUE)
    if (removeFile == TRUE){
      ## Remove temporary file
      file.remove(speciesFilePath)
    }
  } else {
    ## delete the temporary file
    file.remove(speciesFilePath)
    stop(paste0("ERROR: The queried file is truncated, ",
                "there may be a temporary problem with the Bgee webservice."))
  }

  if (length(ordering) == 0){
    return(allSpecies)
  } else if(length(ordering) == 1 & is.numeric(ordering) & ordering <= length(allSpecies[1,])){
    return(allSpecies[order(allSpecies[,ordering]),])
  } else {
    cat(paste0("\nWARNING: Invalid ordering parameter, returning the data frame unsorted.\n"))
    return(allSpecies)
  }
}

context("expect_output")

test_that("Distinct species", {
  species <- listBgeeSpecies(release = "14_2")
	expect_that( species, is_a("data.frame") )
  expect_that( ncol(species), equals(8) )
  expect_message(message("Query to Bgee webservice successful!"))
  expect_true( all(colnames(species) %in% c("ID", "GENUS",
                                            "SPECIES_NAME", "COMMON_NAME",
                                            "AFFYMETRIX", "EST",
                                            "IN_SITU", "RNA_SEQ")))
})

#' @title Delete local data for the species of the reference class Bgee object
#'
#' @description This function delete data present in the local database. It can delete data 
#' rather from one datatype or all datatypes of one species.
#'
#' @param myBgeeObject A Reference Class Bgee object, notably specifying the targeted species
#' 
#' @param allDataTypes A boolean defining rather data of all datatypes from the selected species
#'  should be deleted. If TRUE, the local sqlite database file is deleted. If FALSE, data
#'  corresponding to the datatype of the `myBgeeObject` object will be deleted from the local
#'  sqlite database. 
#' 
#' @author Julien Wollbrett
#'
#' @examples{
#'   bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
#'   data <- getData(bgee, experimentId = "SRP007359")
#'   deleteLocalData(bgee, allDataTypes = TRUE)
#' }
#'
#' @export
#' 
deleteLocalData <- function(myBgeeObject, allDataTypes = FALSE) {
  database_path <- sqlitePath(myBgeeObject = myBgeeObject)
  if(allDataTypes) {
    file.remove(database_path)
    message("Local database deleted successfully for ",
      myBgeeObject$speciesName, " [speciesId:", myBgeeObject$speciesId, "]")
  } else {
    conn <- dbConnect(RSQLite::SQLite(), database_path)
    on.exit(dbDisconnect(conn))
    dbRemoveTable(conn, myBgeeObject$dataType)
    message(myBgeeObject$dataType, " data have been removed from the local sqlite ",
      " database for the species ", myBgeeObject$speciesName, " [speciesId:", 
      myBgeeObject$speciesId, "]")
  }
}

#' @title Delete .rds data of one species coming from an old version of the BgeeDB R package.
#'
#' @description Since Bioconductor 3.11 BgeeDB store data in a local SQLite database allowing
#' to optimize memory usage.
#' This function allows to delete .rds files used to store local data from BgeeDB versions older
#' than Bioconductor 3.11, for the species selected in the reference class Bgee object.
#'
#' @param myBgeeObject A Reference Class Bgee object, notably specifying the targeted species
#' 
#' @author Julien Wollbrett
#'
#' @examples{
#'   bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
#'   deleteOldData(bgee)
#' }
#'
#' @export
#' 
deleteOldData <- function(myBgeeObject) {
  files <- list.files(path = myBgeeObject$pathToData, pattern = "*.rds", full.names = TRUE)
  file.remove(files)
}

#' @title Retrieve Bgee experiments annotation for targeted species and data type.
#'
#' @description This function loads the annotation of experiments and samples of quantitative expression datasets (rna_seq, affymetrix, sc_full_length, sc_droplet_based) that are available from Bgee.
#'
#' @param myBgeeObject A Reference Class Bgee object, notably specifying the targeted species and data type.

#' @return A list of two elements, including a data frame of the annotation of experiments for chosen species (field "experiment.annotation") and a data frame of the annotation of chips/libraries from these experiments (field "sample.annotation").
#'
#' @author Andrea Komljenovic and Julien Roux.
#'
#' @examples{
#'   bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
#'   myAnnotation <- getAnnotation(bgee)
#' }
#'
#' @export

getAnnotation = function(myBgeeObject){

  ## check that the Bgee object is valid
  if (length(myBgeeObject$quantitativeData) == 0 ){
    stop("ERROR: there seems to be a problem with the input Bgee class object, some fields are empty. Please check that the object was correctly built.")
  } else {
    ## Check that download of quantitative data is possible for targeted species and data type
    ## Try to output error message that helps a bit the user
    if (myBgeeObject$quantitativeData != TRUE){
      if (length(myBgeeObject$dataType) > 1){
        stop("ERROR: downloading annotation files is only possible if a single data type (\"rna_seq\", \"sc_full_length\", \"sc_droplet_based\" or \"affymetrix\") is specified in the input Bgee class object.")
      } else if (length(myBgeeObject$dataType) == 1 & (myBgeeObject$dataType == "rna_seq" | myBgeeObject$dataType == "sc_full_length" | myBgeeObject$dataType == "sc_droplet_based" | myBgeeObject$dataType == "affymetrix")){
        stop("ERROR: downloading annotation files is not possible for the species and data type specified in the input Bgee class object. Maybe the specified data type is not available for the targeted species in Bgee? See listBgeeSpecies() for details on data types availability for each species.")
      } else {
        stop("ERROR: downloading annotation files is not possible for the species and data type specified in the input Bgee class object.")
      }
    } else if (length(myBgeeObject$annotationUrl) == 0 | length(myBgeeObject$dataType) == 0 | length(myBgeeObject$pathToData) == 0){
      stop("ERROR: there seems to be a problem with the input Bgee class object, some fields are empty. Please check that the object was correctly built.")
    }
  }

  ## Get name of annotation file from the URL field of Bgee object
  annotationFile <- basename(myBgeeObject$annotationUrl)
  ## To get the names of experiment and sample files, we start from the annotationFile
  if (myBgeeObject$dataType == "affymetrix"){
    annotationExperiments <- gsub("_chips", "", annotationFile)
  } else if (myBgeeObject$dataType == "rna_seq" | myBgeeObject$dataType == "sc_full_length" | myBgeeObject$dataType == "sc_droplet_based"){
    annotationExperiments <- gsub("_libraries", "", annotationFile)
  }
  annotationSamples     <- gsub("_experiments", "", annotationFile)
  if(grepl(".zip$", annotationExperiments, perl = TRUE)){
    annotationExperiments <- gsub("zip", "tsv", annotationExperiments)
    annotationSamples     <- gsub("zip", "tsv", annotationSamples)
  }else if(grepl(".tar.gz$", annotationExperiments, perl = TRUE)){
    annotationExperiments <- gsub("tar.gz", "tsv", annotationExperiments)
    annotationSamples     <- gsub("tar.gz", "tsv", annotationSamples)
  }else{
    stop("\nThe annotation file can not be uncompressed because it is not a zip nor a tar.gz file\n")
  }
  ## Check if file is already in cache. If so, skip download step
  if (file.exists(file.path(myBgeeObject$pathToData, annotationExperiments)) && file.exists(file.path(myBgeeObject$pathToData, annotationSamples))){
    message("\nNOTE: annotation files for this species were found in the download directory ", myBgeeObject$pathToData,
               ". Data will not be redownloaded.\n")
  } else {
    success <- bgee_download_file(url = myBgeeObject$annotationUrl,
                             destfile = file.path(myBgeeObject$pathToData, annotationFile), mode='wb')
    if (success != 0){
      stop("ERROR: Download from FTP was not successful.")
    }
    if(grepl(".zip$", annotationFile, perl = TRUE)){
      myData <- unzip(file.path(myBgeeObject$pathToData, annotationFile), exdir=myBgeeObject$pathToData)
    }else if(grepl(".tar.gz$", annotationFile, perl = TRUE)){
      untar(file.path(myBgeeObject$pathToData, annotationFile), exdir=myBgeeObject$pathToData)
      myData <- untar(file.path(myBgeeObject$pathToData, annotationFile), exdir=myBgeeObject$pathToData, list = TRUE)
      myData <- file.path(myBgeeObject$pathToData, myData)
      # move annotation files from "experiments repository" to pathToData repository in order to be consistent with zip uncompression
      if (dirname(myData[1]) != myBgeeObject$pathToData) {
        file.rename(from = myData,  to = file.path(myBgeeObject$pathToData, basename(myData)))
        # delete the "experiment repository" folder
        unlink(dirname(myData), recursive = TRUE)
      }
    }else{
      stop("\nThe annotation file can not be uncompressed because it is not a zip nor a tar.gz file\n")
    }
    cat("\nSaved annotation files in", myBgeeObject$pathToData, "folder.\n")
    ## Clean directory and remove .zip file
    file.remove(file.path(myBgeeObject$pathToData, annotationFile))
    ## Test if extracted files are OK
    if (!(file.exists(file.path(myBgeeObject$pathToData, annotationExperiments)) & file.exists(file.path(myBgeeObject$pathToData, annotationSamples)))){
      stop("ERROR: extraction of annotation files from downloaded zip file went wrong.")
    }
  }

  ## Read the 2 annotation files
  myAnnotation <- list(sample.annotation=as.data.frame(fread(file.path(myBgeeObject$pathToData, annotationSamples))),
                       experiment.annotation=as.data.frame(fread(file.path(myBgeeObject$pathToData, annotationExperiments)))
  )
  ## remove spaces in headers
  names(myAnnotation$experiment.annotation) <- make.names(names(myAnnotation$experiment.annotation))
  names(myAnnotation$sample.annotation) <- make.names(names(myAnnotation$sample.annotation))
  ## Double dots are now present in some column names, due to column headers in FTP file such as "Min. read length"
  ## and "Max. read length". These are replaced by a single dot.
  names(myAnnotation$sample.annotation) <- gsub("\\.\\.", ".", names(myAnnotation$sample.annotation))

  return(myAnnotation)
}

context("expect_output")


test_that("Annotation files", {
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  annotation_bgee_mouse <- getAnnotation(bgee)
  expect_that( annotation_bgee_mouse, is_a("list") )
  expect_that( length(annotation_bgee_mouse), equals(2) )
  expect_true( unique(names(annotation_bgee_mouse) %in% c("experiment.annotation", "sample.annotation")))
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "affymetrix")
  annotation_bgee_mouse <- getAnnotation(bgee)
  expect_that( annotation_bgee_mouse, is_a("list") )
  
  #test loading old version of annotation (bgee version < 14.0)
  bgee_v13 <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq", release = "13.2")
  annotation_bgee_mouse_v13 <- getAnnotation(bgee_v13)
  expect_that( annotation_bgee_mouse_v13, is_a("list") )
  bgee_v13 <- Bgee$new(species = "Mus_musculus", dataType = "affymetrix", release = "13.2")
  annotation_bgee_mouse_v13 <- getAnnotation(bgee_v13)
  expect_that( annotation_bgee_mouse_v13, is_a("list") )
})

#' @title Formats results of the enrichment test on anatomical structures.
#'
#' @description This function loads the results from the topGO test and creates an output table with organ names,
#' fold enrichment and FDR. Data are sorted by p-value and only terms below the specified FDR cutoff are included.
#'
#' @param topAnatData A list produced by the function loadTopAnatData().
#'
#' @param topAnatObject An object produced by the function topAnat().
#'
#' @param results A result object, produced by the runtest() function of topGO.
#'
#' @param cutoff An FDR cutoff between 0 and 1. Only terms with FDR lower than this cutoff are included.
#' Default is 1, meaning that all terms are included.
#'
#' @param ordering A numeric indicating which column should be used to sort the data frame. If the column number is preceded by a \"-\" sign, results are displayed in decreasing ordering. Default is "7", returning data frame sorted by p-values in increasing order.
#'
#' @return A data frame with significantly enriched anatomical structures, sorted by p-value.
#'
#' @author Julien Roux
#'
#' @examples{
#' bgee <- Bgee$new(species="Bos_taurus", dataType="rna_seq")
#' myTopAnatData <- loadTopAnatData(bgee, stage="UBERON:0000092")
#' geneList <- as.factor(c(rep(0, times=85), rep(1, times=15)))
#' names(geneList) <- c("ENSBTAG00000000011","ENSBTAG00000000014","ENSBTAG00000000016",
#'                      "ENSBTAG00000000026","ENSBTAG00000000039","ENSBTAG00000000040",
#'                      "ENSBTAG00000000042","ENSBTAG00000000050","ENSBTAG00000000056",
#'                      "ENSBTAG00000000064","ENSBTAG00000000067","ENSBTAG00000000071",
#'                      "ENSBTAG00000000072","ENSBTAG00000000080","ENSBTAG00000000081",
#'                      "ENSBTAG00000000084","ENSBTAG00000000091","ENSBTAG00000000099",
#'                      "ENSBTAG00000000111","ENSBTAG00000000123","ENSBTAG00000000132",
#'                      "ENSBTAG00000000153","ENSBTAG00000000162","ENSBTAG00000000163",
#'                      "ENSBTAG00000000169","ENSBTAG00000000179","ENSBTAG00000000197",
#'                      "ENSBTAG00000000199","ENSBTAG00000000202","ENSBTAG00000000203",
#'                      "ENSBTAG00000000204","ENSBTAG00000000213","ENSBTAG00000000215",
#'                      "ENSBTAG00000000223","ENSBTAG00000000224","ENSBTAG00000000225",
#'                      "ENSBTAG00000000236","ENSBTAG00000000250","ENSBTAG00000000251",
#'                      "ENSBTAG00000000252","ENSBTAG00000000253","ENSBTAG00000000261",
#'                      "ENSBTAG00000000274","ENSBTAG00000000277","ENSBTAG00000000279",
#'                      "ENSBTAG00000000285","ENSBTAG00000000286","ENSBTAG00000000287",
#'                      "ENSBTAG00000000289","ENSBTAG00000000297","ENSBTAG00000000305",
#'                      "ENSBTAG00000000312","ENSBTAG00000000328","ENSBTAG00000000335",
#'                      "ENSBTAG00000000341","ENSBTAG00000000343","ENSBTAG00000000354",
#'                      "ENSBTAG00000000355","ENSBTAG00000000356","ENSBTAG00000000365",
#'                      "ENSBTAG00000000372","ENSBTAG00000000379","ENSBTAG00000000380",
#'                      "ENSBTAG00000000382","ENSBTAG00000000396","ENSBTAG00000000404",
#'                      "ENSBTAG00000000405","ENSBTAG00000000406","ENSBTAG00000000411",
#'                      "ENSBTAG00000000425","ENSBTAG00000000434","ENSBTAG00000000435",
#'                      "ENSBTAG00000000438","ENSBTAG00000000448","ENSBTAG00000000451",
#'                      "ENSBTAG00000000454","ENSBTAG00000000456","ENSBTAG00000000457",
#'                      "ENSBTAG00000000459","ENSBTAG00000000462","ENSBTAG00000000469",
#'                      "ENSBTAG00000000470","ENSBTAG00000000484","ENSBTAG00000000497",
#'                      "ENSBTAG00000000501","ENSBTAG00000009707","ENSBTAG00000026266",
#'                      "ENSBTAG00000021992","ENSBTAG00000005353","ENSBTAG00000005333",
#'                      "ENSBTAG00000006424","ENSBTAG00000026972","ENSBTAG00000010799",
#'                      "ENSBTAG00000010799","ENSBTAG00000014614","ENSBTAG00000014614",
#'                      "ENSBTAG00000045757","ENSBTAG00000046332","ENSBTAG00000046332",
#'                      "ENSBTAG00000008394")
#' myTopAnatObject <-  topAnat(myTopAnatData, geneList)
#' resFis <- runTest(myTopAnatObject, algorithm = 'elim', statistic = 'fisher')
#' ## Format results
#' tableOver <- makeTable(myTopAnatData, myTopAnatObject, resFis, 0.1)
#' }
#' @import stats
#' @export

makeTable <- function(topAnatData, topAnatObject, results, cutoff=1, ordering=7){
  ## Perform some checks on the input data
  if(any(is.na(score(results))) || length(score(results)) == 0){
    stop("ERROR: the results object is empty.")
  }
  if (!is.numeric(cutoff)){
    cutoff <- as.numeric(cutoff)
  }
  if(is.null(topAnatObject)){
    stop("ERROR: the topAnatObject is empty.")
  }
  if( length(topAnatData$organ.names[,1]) == 0 ) {
    stop("ERROR: the organ.names data frame of your topAnatData object is empty.")
  }

  dec <- FALSE
  is.wholenumber <- function(x, tol = .Machine$double.eps^0.5){
    abs(x - round(x)) < tol
  }
  if(length(ordering) == 0){
    ordering <- 7
  }
  if(length(ordering) == 1 & is.numeric(ordering) & ordering <= 8){
    if (ordering < 0){
      dec <- TRUE
      ordering <- -ordering
    }
    if (!is.wholenumber(ordering)){
      stop("ERROR: problem with ordering argument. It should be an integer (column number).")
    }
  }
  else {
    stop("ERROR: problem with ordering argument. It should be an integer (column number), preceded by a \"-\" sign if decreasing ordering is needed. It should not be bigger than the number of columns in results data frame.")
  }
  ## retrieve p-values for the enrichment
  scores <- score(results)
  fdr <- p.adjust(p=scores, method = "fdr")
  topTerms <- sort(scores[fdr <= cutoff])
  topTerms <- as.data.frame(topTerms)

  if( nrow(topTerms) != 0 ){
    cat(paste0("\nBuilding the results table for the ", nrow(topTerms), " significant terms at FDR threshold of ", cutoff, "...\n"))
    odds <- termStat(topAnatObject, row.names(topTerms))
    foldEnrichment <- odds[2]/odds[3]

    ## Rounding odds, P-values and FDR to 2 decimal places
    ## Now commented because might be problematic for downstream reuse of the data. Maybe reintroduce with optional argument?
    # foldEnrichment <- format(foldEnrichment, digits=3)
    # topTerms <- format(topTerms, digits=3)
    # fdr[row.names(topTerms)] <- format(fdr[row.names(topTerms)], digits=3)

    topTerms <- cbind(odds, foldEnrichment, topTerms, fdr[row.names(topTerms)])
    topTable <- merge(topAnatData$organ.names, topTerms, by.x=1, by.y=0)
    names(topTable) <- c("organId", "organName", "annotated", "significant", "expected", "foldEnrichment" , "pValue", "FDR")

    cat(paste0("Ordering results by ", names(topTable)[ordering], " column in ", ifelse(dec, "decreasing", "increasing")," order...\n"))
    topTable <- topTable[order(topTable[, ordering], decreasing=dec), ]

    ## remove arbitrary numbers that are used as row names
    row.names(topTable) <- topTable$organId

    cat("Done\n\n")
    return(topTable)
  } else {
    cat("\nWARNING: there was no significant term at FDR threshold of", cutoff, "\n\n")
    return(NA)
  }
}

#' @title Galaxy adapter to the `getAnnotation` function
#' 
#' @description
#' Adapter of the `getAnnotation` function allowing to run it on Galaxy.
#' 
#' @param species name of the species for which annotation have to be be retrieved (e.g Homo_sapiens)
#' @param dataType The datatype for which annotation have to be retrieved. To be chosen among:
#' \itemize{
#'   \item{"rna_seq: target only bulk RNA-seq data"}
#'   \item{"sc_full_length: target only full length single-cell RNA-seq data"}
#'   \item{"sc_droplet_based: target only droplet based single-cell RNA-seq data"}
#'   \item{"affymetrix: target only Affymtrix microarray data"}
#' }
#' @param experimentAnnotationPath path to the file containing experiment annotation
#' @param sampleAnnotationPath path to the file containing sample annotation
#' @export

getAnnotation_galaxy <- function(species = NULL, dataType = character(0),
                                 experimentAnnotationPath = NULL, sampleAnnotationPath = NULL) {
  if (is.null(experimentAnnotationPath) || is.null(sampleAnnotationPath)) {
    stop("path to output files should always be provided")
  }
  myBgeeObject <- createBgeeObject_galaxy(species = species, dataTypes = dataType)
  annotation <- getAnnotation(myBgeeObject = myBgeeObject)
  write.table(x = annotation$experiment.annotation, file = experimentAnnotationPath,
              sep = "\t", quote = FALSE, row.names = FALSE)
  write.table(x = annotation$sample.annotation, file = sampleAnnotationPath,
              sep = "\t", quote = FALSE, row.names = FALSE)
}

#' @title Galaxy adapter to the `getSampleProcessedData` function
#' 
#' @description
#' This function loads the quantitative expression data and presence calls 
#' for samples available from Bgee. These data are available for all droplet based single-cell
#' RNA-seq, full length single-cell RNA-seq, bulk RNA-seq and affymetrix samples present in Bgee.
#' It is possible to filter the processed expression data using experiment IDs, chip IDs (affymetrix)
#' or library IDs (RNA-seq), anatomical entity IDs from the UBERON ontology, developmental stage IDs,
#' cell-type IDs, sex and/or strain. Celltype, anatomical entity and stage IDs are normalized and come from
#' ontologies, allowing to retrieve data annotated with the selected term or with the term and all its descendants.
#' This option allows for instance to retrieve all processed expression values coming from the brain or
#' any subpart of the brain.
#' 
#' @param species name of the species for which annotation have to be be retrieved (e.g Homo_sapiens)
#' @param dataType The datatype for which annotation have to be retrieved. To be chosen among:
#' \itemize{
#'   \item{"rna_seq: target only bulk RNA-seq data"}
#'   \item{"sc_full_length: target only full length single-cell RNA-seq data"}
#'   \item{"sc_droplet_based: target only droplet based single-cell RNA-seq data"}
#'   \item{"affymetrix: target only Affymtrix microarray data"}
#' }
#' @param experimentId Filter allowing to specify one or more ArrayExpress or GEO accession, e.g., 
#' GSE43721. Default is NULL: takes all available experiments for targeted species and data type.
#' @param sampleId Filter allowing to specify one or more sample ID. Depending on the selected 
#' datatype this sample IDs can correspond to Chip IDs (affymetrix) or RNA-Seq library IDs (rna_seq). 
#' Default is NULL: takes all available samples for targeted species and data type.
#' @param anatEntityId Filter allowing to specify one or more anatomical entity IDs from the UBERON 
#' ontology (http://uberon.github.io/). Default is NULL: takes all available anatomical entities for 
#' targeted species and data type.
#' @param stageId Filter allowing to specify one or more developmental stage IDs from Developmental 
#' Stage Ontology (https://github.com/obophenotype/developmental-stage-ontologies). Default is 
#' NULL: takes all available developmental stages for targeted species and data type.
#' @param cellTypeId Filter specific to single cell datatype (sc_full_length) allowing to specify 
#' one or more cell type IDs from the UBERON ontology (http://uberon.github.io/). Default is 
#' NULL: takes all available cell types for targeted species and data type. Available for Bgee 15.0 and after
#' @param sex Filter allowing to specify one or more sexes. Default is 
#' NULL: takes all available sexes for targeted species and data type. Available for Bgee 15.0 and after
#' @param strain Filter allowing to specify one or more strains. Default is 
#' NULL: takes all available strains for targeted species and data type. Available for Bgee 15.0 and after
#' @param withDescendantAnatEntities Allows to filter on the selected anatEntityId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#' @param withDescendantStages Allows to filter on the selected stageId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#' @param withDescendantCellTypes Allows to filter on the selected cellTypeId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#' @param sampleProcessedOutputFile path to the file containing sample processed data.
#' @export

getSampleProcessedData_galaxy <- 
  function(species = NULL, dataType = character(0),
           experimentId = NULL, sampleId = NULL, anatEntityId = NULL, stageId = NULL,
           cellTypeId = NULL, sex = NULL, strain = NULL, withDescendantAnatEntities = FALSE,
           withDescendantStages = FALSE, withDescendantCellTypes = FALSE,
           sampleProcessedOutputFile = NULL) {
    if (is.null(sampleProcessedOutputFile)) {
      stop("path to the output file should always be provided")
    }
    myBgeeObject <- createBgeeObject_galaxy(species = species, dataTypes = dataType)
    sample_processed_data <-
      getSampleProcessedData(myBgeeObject = myBgeeObject,experimentId = experimentId,
                             sampleId = sampleId, anatEntityId = anatEntityId, stageId = stageId,
                             cellTypeId = cellTypeId, sex = sex, strain = strain,
                             withDescendantAnatEntities = withDescendantAnatEntities,
                             withDescendantStages = withDescendantStages,
                             withDescendantCellTypes = withDescendantCellTypes)
    write.table(x = sample_processed_data, file = sampleProcessedOutputFile, sep = "\t",
                quote = FALSE, row.names = FALSE)
  }

#' @title Run a GO-like enrichment of anatomical terms, mapped to genes by expression patterns
#'
#' @description This function GO-like enrichment of anatomical terms using the Uberon ontology.
#' 
#' @param species name of the species for which annotation have to be be retrieved (e.g Homo_sapiens)
#' @param dataTypes The datatypes for which calls will to be retrieved. To be chosen among:
#' \itemize{
#'   \item{"rna_seq: target only bulk RNA-seq data"}
#'   \item{"sc_full_length: target only full length single-cell RNA-seq data"}
#'   \item{"sc_droplet_based: target only droplet based single-cell RNA-seq data"}
#'   \item{"affymetrix: target only Affymtrix microarray data"}
#' }
#' @param foregroundGenes the list of genes for which TopAnat will find anatomical entities that
#' have over or under-represented expression using annotations for that gene set, compared to the
#' background genes. Can be NULL if foregroundGenesFile is provided.
#'
#' @param backgroundGenes the list of genes you want to consider as the universe in your analysis.
#' Optional. If NULL and backgroundGenesFile is also NULL, all genes in the species will be used as background.
#'
#' @param foregroundGenesFile path to a one-column TSV file without header containing foreground genes.
#' Can be NULL if foregroundGenes is provided.
#'
#' @param backgroundGenesFile path to a one-column TSV file without header containing background genes.
#' Optional. If NULL and backgroundGenes is also NULL, all genes in the species will be used as background.
#'
#' @param nodeSize Minimum number of genes mapped to a node for it to be tested. Default is 10.
#' 
#' @param algorithm Decorrelation algorithm used to take into account the topology of the anatomical
#' ontology (default = weight). By default `classic` is used. The full list of algorithm can be
#' retrieved with topGO::whichAlgorithms()
#' 
#' @param statistics Character string specifing which test to use. By default `fisher`is used. The full list
#' of tests can be retrieved with topGO::whichTests()
#' 
#' @param confidence A character indicating if only high quality present calls should be
#' retrieved. For Bgee releases prior to 14, options are "all" (default) or "high_quality".
#' For Bgee release 14 and above, options are "silver" (default) and "gold".
#' 
#' @param resultFile The file where GO-like enrichment of anatomical terms is stored
#'
#' @author Julien Wollbrett
#'
#' @examples{
#' foregroundGenes <- c("ENSBTAG00000000011","ENSBTAG00000000014","ENSBTAG00000000016",
#' "ENSBTAG00000000026","ENSBTAG00000000039","ENSBTAG00000000040",
#' "ENSBTAG00000000042","ENSBTAG00000000050","ENSBTAG00000000056",
#' "ENSBTAG00000000064","ENSBTAG00000000067","ENSBTAG00000000071",
#' "ENSBTAG00000000072","ENSBTAG00000000080","ENSBTAG00000000081")
#' backgroundGenes <- c("ENSBTAG00000000011","ENSBTAG00000000014","ENSBTAG00000000016",
#' "ENSBTAG00000000026","ENSBTAG00000000039","ENSBTAG00000000040",
#' "ENSBTAG00000000042","ENSBTAG00000000050","ENSBTAG00000000056",
#' "ENSBTAG00000000064","ENSBTAG00000000067","ENSBTAG00000000071",
#' "ENSBTAG00000000072","ENSBTAG00000000080","ENSBTAG00000000081",
#' "ENSBTAG00000000084","ENSBTAG00000000091","ENSBTAG00000000099",
#' "ENSBTAG00000000111","ENSBTAG00000000123","ENSBTAG00000000132",
#' "ENSBTAG00000000153","ENSBTAG00000000162","ENSBTAG00000000163",
#' "ENSBTAG00000000169","ENSBTAG00000000179","ENSBTAG00000000197",
#' "ENSBTAG00000000199","ENSBTAG00000000202","ENSBTAG00000000203",
#' "ENSBTAG00000000204","ENSBTAG00000000213","ENSBTAG00000000215",
#' "ENSBTAG00000000223","ENSBTAG00000000224","ENSBTAG00000000225",
#' "ENSBTAG00000000236","ENSBTAG00000000250","ENSBTAG00000000251",
#' "ENSBTAG00000000252","ENSBTAG00000000253","ENSBTAG00000000261",
#' "ENSBTAG00000000274","ENSBTAG00000000277","ENSBTAG00000000279",
#' "ENSBTAG00000000285","ENSBTAG00000000286","ENSBTAG00000000287",
#' "ENSBTAG00000000289","ENSBTAG00000000297","ENSBTAG00000000305",
#' "ENSBTAG00000000312","ENSBTAG00000000328","ENSBTAG00000000335",
#' "ENSBTAG00000000341","ENSBTAG00000000343","ENSBTAG00000000354",
#' "ENSBTAG00000000355","ENSBTAG00000000356","ENSBTAG00000000365",
#' "ENSBTAG00000000372","ENSBTAG00000000379","ENSBTAG00000000380",
#' "ENSBTAG00000000382","ENSBTAG00000000396","ENSBTAG00000000404",
#' "ENSBTAG00000000405","ENSBTAG00000000406","ENSBTAG00000000411",
#' "ENSBTAG00000000425","ENSBTAG00000000434","ENSBTAG00000000435",
#' "ENSBTAG00000000438","ENSBTAG00000000448","ENSBTAG00000000451",
#' "ENSBTAG00000000454","ENSBTAG00000000456","ENSBTAG00000000457",
#' "ENSBTAG00000000459","ENSBTAG00000000462","ENSBTAG00000000469",
#' "ENSBTAG00000000470","ENSBTAG00000000484","ENSBTAG00000000497",
#' "ENSBTAG00000000501","ENSBTAG00000009707","ENSBTAG00000026266",
#' "ENSBTAG00000021992","ENSBTAG00000005353","ENSBTAG00000005333",
#' "ENSBTAG00000006424","ENSBTAG00000026972","ENSBTAG00000010799",
#' "ENSBTAG00000014614","ENSBTAG00000045757","ENSBTAG00000046332",
#' "ENSBTAG00000008394")
#' topAnat_galaxy(species = "Bos_taurus", stageId = "UBERON:0000092",
#'               foregroundGenes = foregroundGenes, backgroundGenes = backgroundGenes,
#'               resultFile = "result.tsv")
#' }
#'
#' @import topGO graph
#' @export
#' 
topAnat_galaxy <- function(species = NULL, dataTypes = character(0), stageId = NULL,
                           foregroundGenes = NULL, backgroundGenes = NULL, 
                           foregroundGenesFile = NULL, backgroundGenesFile = NULL,
                           algorithm = "classic",
                           statistics = "fisher", resultFile = NULL, nodeSize = 10,
                           confidence = "silver") {
  if (is.null(resultFile)) {
    stop("resultFile parameter is required")
  }
  myBgeeObject <- createBgeeObject_galaxy(species = species, dataTypes = dataTypes)
  myTopAnatData <- loadTopAnatData(myBgeeObject = myBgeeObject, stage = stageId, 
                                   confidence = confidence)
  
  if (is.null(foregroundGenes) && is.null(foregroundGenesFile)) {
    stop("Either foregroundGenes or foregroundGenesFile must be provided")
  }
  
  if (!is.null(foregroundGenes) && !is.null(foregroundGenesFile)) {
    stop("Please provide either foregroundGenes or foregroundGenesFile, not both")
  }

  if (!is.null(foregroundGenesFile)) {
    if (!file.exists(foregroundGenesFile)) {
      stop("Foreground genes file not found: ", foregroundGenesFile)
    }
    # Read foreground genes from file
    foregroundGenes <- read.table(foregroundGenesFile, header = FALSE, sep = "\t", 
                                  stringsAsFactors = FALSE)[, 1]
  }
  # Ensure foreground genes are unique
  foregroundGenes <- unique(foregroundGenes)
  
  if (length(foregroundGenes) == 0) {
    stop("Foreground genes list is empty")
  }
    
  if (!is.null(backgroundGenes) && !is.null(backgroundGenesFile)) {
    stop("Please provide either backgroundGenes or backgroundGenesFile, not both")
  }
  if (!is.null(backgroundGenesFile)) {
    if (!file.exists(backgroundGenesFile)) {
      stop("Background genes file not found: ", backgroundGenesFile)
    }
    backgroundGenes <- read.table(backgroundGenesFile, header = FALSE, sep = "\t", 
                                  stringsAsFactors = FALSE)[, 1]
  }
  # If background genes are not provided, use all genes in myTopAnatData as background
  if (is.null(backgroundGenes)) {
    # Use all genes available in myTopAnatData as background
    backgroundGenes <- names(myTopAnatData$gene2anatomy)
  } else {
    # Ensure background genes are unique
    backgroundGenes <- unique(backgroundGenes)
  }

  if (!all(foregroundGenes %in% backgroundGenes)) {
    stop("All foreground genes should be part of the background")
  }
  # create the geneList vector used as input of topAnat
  geneList <- rep(0, length(backgroundGenes))
  names(geneList) <- backgroundGenes
  geneList[names(geneList) %in% foregroundGenes] <- 1
  geneList <- as.factor(geneList)
  myTopAnatObject <- NULL
  tryCatch(
    {
      myTopAnatObject <- topAnat(topAnatData = myTopAnatData, geneList = geneList, nodeSize = nodeSize)
    },
    error = function(e) {
      stop("Did not manage to run topAnat. You probably did not provide proper gene list. ",
           "It has to be gene IDs and not gene names. ", conditionMessage(e))
    }
  )
  results <- runTest(object = myTopAnatObject, algorithm = algorithm, statistic = statistics)
  tableOver <- makeTable(myTopAnatData, myTopAnatObject, results)
  write.table(x = tableOver, file = resultFile, quote = FALSE, sep = "\t", row.names = FALSE)
}

#' @title Retrieve Bgee calls from one species.
#'
#' @description Loads the integrated expression calls from one species. These calls are 
#' equivalent to the one provided in the gene page of the Bgee website. The calls have been
#' generated using all datatypes present in Bgee (EST, In Situ, Affymetrix, bulk RNA-Seq and 
#' single-cell RNA-Seq).
#'
#' @param species name of the species for which annotation have to be be retrieved (e.g Homo_sapiens)
#' @param conditionParameters Specify which condition parameter you are interested
#' in. It can be `anatEntity` or `allCondParams`. The `anatEntity` option retrieve calls generated 
#' using only anatomical entity and cell-type as a condition parameter. The `allCondParams` option retrieve calls generated taking
#' into consideration all condition parameters present in Bgee (anat, entity, cell-type, developmental stage,
#' sex and strain). By default the `anatEntity` option is selected.
#' @param advancedColumns Boolean allowing to specify if advancend columns are required. The default value
#' is `FALSE` meaning that only gene , condition parameters, call quality, FDR, expression rank and
#' expression score are retrieved. If `TRUE` then a lot more information used to generate the calls are
#' retrieved like the number of self and descendant observation for each datatype, or the rank and score
#' per datatype. 
#' @param geneIds List of genes for which expression calls have to be retrieved. It has to be ensembl
#' IDs if the Bgee genome source is Ensembl (e.g ENSG00000244734) or RefSeq IDs if the Bgee genome source
#' is RefSeq (e.g 734881) but not gene names (e.g HBB, Apoc1, etc.).
#' @param anatEntityIds List of anatomical entity IDs for which expression calls have to be retrieved.
#' It has to be IDs (e.g UBERON:0000955) and not names (e.g brain)
#' @param callsOutputFile name of the file where intergrated calls will be stored
#' @export
#' 
getIntergratedCalls_galaxy <- function(species = NULL, conditionParameters = "anatEntity",
                                       advancedColumns = FALSE, geneIds = NULL,
                                       anatEntityIds = NULL, callsOutputFile = NULL) {
  myBgeeObject <- createBgeeObject_galaxy(species = species)
  integrated_calls <- getIntegratedCalls(myBgeeObject = myBgeeObject,
                                         conditionParameters = conditionParameters,
                                         advancedColumns = advancedColumns, geneIds = geneIds,
                                         anatEntityIds = anatEntityIds)
  write.table(x = integrated_calls, file = callsOutputFile, quote = FALSE, sep = "\t",
              row.names = FALSE)
}

#' @title Retrieve Bgee processed expression values at cell level for single cell data.
#'
#' @description This function download the processed gene count sparse matrix of single cell experiments
#' in Bgee. These data are available for all droplet based single-cell RNA-seq and full length
#' single-cell RNA-seq. These files are stored in the H5AD format.
#'
#' @param species name of the species for which annotation have to be be retrieved (e.g Homo_sapiens)
#' @param experimentId Filter allowing to specify one ArrayExpress or GEO accession, e.g., 
#' GSE43721. Can not be null 
#' @param cellProcessedDataFile Name of the file where processed expression values at cell
#' level is stored
#' 
#' @author Julien Wollbrett
#' 
#' @export

downloadCellProcessedData_galaxy <- function(species = NULL, experimentId = NULL,
                                             cellProcessedDataFile = NULL) {
  myBgeeObject <- createBgeeObject_galaxy(species = species)
  destFile <- downloadCellProcessedFile(myBgeeObject = myBgeeObject,
                                        experimentId = experimentId)
  file.copy(from = destFile, to = cellProcessedDataFile)
  file.remove(destFile)
} 

#' @description
#' helper function to create a Bgee object for Galaxy wrappers. It allows to handle
#' errors in a cleaner way for Galaxy. Especially it lists the releases,
#' explains the pattern used for the species field and list available datatypes if
#' provided one does not exist
#' @noRd
#' @noMd
createBgeeObject_galaxy <- function(species = NULL, dataTypes = NULL) {
  # then check the species
  invisible(capture.output(bgeeSpecies <- listBgeeSpecies()))
  bgeeSpecies$formatedSpeciesName <- paste0(gsub(" ", "_", bgeeSpecies$GENUS),
                                            "_", gsub(" ", "_", bgeeSpecies$SPECIES_NAME))
  if (! species %in% bgeeSpecies$formatedSpeciesName) {
    stop("The species is not present in Bgee. the Species should follow the pattern",
    "`Genus_species` (e.g Homo_sapiens, Mus_musculus, Canis_lupus_familiaris)")
  }
  return(Bgee$new(species = species, dataType = dataTypes))
}

#' @title Produces an object allowing to perform GO-like enrichment of anatomical terms using the topGO package
#'
#' @description This function produces a topAnatObject, ready to use for gene set enrichment testing using functions from the topGO package. This object uses the Uberon ontology instead of the GO ontology.
#'
#' @details To perform the enrichment test for expression in anatomical structures for each term of Uberon ontology (browsable at \url{http://www.ontobee.org/ontology/UBERON}), the data are formatted to use the topGO package for testing. This package is interesting because it propagates the mapping of gene to terms to parent terms, and it possesses a pannel of enrichment tests and decorrelation methods. Expert users should be able to use information from the topAnatObject to test enrichment with other packages than topGO.
#'
#' @param topAnatData a list including a gene2anatomy list, an organ.relationships list and an organ.names data.frame, produced by the function loadTopAnatData().
#'
#' @param geneList Vector indicating foreground and background genes. Names of the vector indicate the background genes. Values are 1 (gene in foreground) or 0 (gene not in foreground).
#'
#' @param nodeSize Minimum number of genes mapped to a node for it to be tested. Default is 10.
#'
#' @param ... Additional parameters as passed to build topGOdata object in topGO package.
#'
#' @return topAnatObject, a topAnatData class object, ready for gene set enrichment testing with topGO.
#'
#' @author Julien Roux
#'
#' @examples{
#' bgee <- Bgee$new(species="Bos_taurus", dataType="rna_seq")
#' myTopAnatData <- loadTopAnatData(bgee, stage="UBERON:0000092")
#' geneList <- as.factor(c(rep(0, times=85), rep(1, times=15)))
#' names(geneList) <- c("ENSBTAG00000000011","ENSBTAG00000000014","ENSBTAG00000000016",
#'                      "ENSBTAG00000000026","ENSBTAG00000000039","ENSBTAG00000000040",
#'                      "ENSBTAG00000000042","ENSBTAG00000000050","ENSBTAG00000000056",
#'                      "ENSBTAG00000000064","ENSBTAG00000000067","ENSBTAG00000000071",
#'                      "ENSBTAG00000000072","ENSBTAG00000000080","ENSBTAG00000000081",
#'                      "ENSBTAG00000000084","ENSBTAG00000000091","ENSBTAG00000000099",
#'                      "ENSBTAG00000000111","ENSBTAG00000000123","ENSBTAG00000000132",
#'                      "ENSBTAG00000000153","ENSBTAG00000000162","ENSBTAG00000000163",
#'                      "ENSBTAG00000000169","ENSBTAG00000000179","ENSBTAG00000000197",
#'                      "ENSBTAG00000000199","ENSBTAG00000000202","ENSBTAG00000000203",
#'                      "ENSBTAG00000000204","ENSBTAG00000000213","ENSBTAG00000000215",
#'                      "ENSBTAG00000000223","ENSBTAG00000000224","ENSBTAG00000000225",
#'                      "ENSBTAG00000000236","ENSBTAG00000000250","ENSBTAG00000000251",
#'                      "ENSBTAG00000000252","ENSBTAG00000000253","ENSBTAG00000000261",
#'                      "ENSBTAG00000000274","ENSBTAG00000000277","ENSBTAG00000000279",
#'                      "ENSBTAG00000000285","ENSBTAG00000000286","ENSBTAG00000000287",
#'                      "ENSBTAG00000000289","ENSBTAG00000000297","ENSBTAG00000000305",
#'                      "ENSBTAG00000000312","ENSBTAG00000000328","ENSBTAG00000000335",
#'                      "ENSBTAG00000000341","ENSBTAG00000000343","ENSBTAG00000000354",
#'                      "ENSBTAG00000000355","ENSBTAG00000000356","ENSBTAG00000000365",
#'                      "ENSBTAG00000000372","ENSBTAG00000000379","ENSBTAG00000000380",
#'                      "ENSBTAG00000000382","ENSBTAG00000000396","ENSBTAG00000000404",
#'                      "ENSBTAG00000000405","ENSBTAG00000000406","ENSBTAG00000000411",
#'                      "ENSBTAG00000000425","ENSBTAG00000000434","ENSBTAG00000000435",
#'                      "ENSBTAG00000000438","ENSBTAG00000000448","ENSBTAG00000000451",
#'                      "ENSBTAG00000000454","ENSBTAG00000000456","ENSBTAG00000000457",
#'                      "ENSBTAG00000000459","ENSBTAG00000000462","ENSBTAG00000000469",
#'                      "ENSBTAG00000000470","ENSBTAG00000000484","ENSBTAG00000000497",
#'                      "ENSBTAG00000000501","ENSBTAG00000009707","ENSBTAG00000026266",
#'                      "ENSBTAG00000021992","ENSBTAG00000005353","ENSBTAG00000005333",
#'                      "ENSBTAG00000006424","ENSBTAG00000026972","ENSBTAG00000010799",
#'                      "ENSBTAG00000010799","ENSBTAG00000014614","ENSBTAG00000014614",
#'                      "ENSBTAG00000045757","ENSBTAG00000046332","ENSBTAG00000046332",
#'                      "ENSBTAG00000008394")
#' myTopAnatObject <-  topAnat(myTopAnatData, geneList)
#' }
#'
#' @import topGO graph
#' @export

topAnat <- function(topAnatData, geneList, nodeSize = 10, ... ){

  ## Test if topAnatData not empty
  cat("\nChecking topAnatData object.............\n")
  if( length(topAnatData$gene2anatomy) == 0 ) {
    stop("ERROR: the gene2anatomy list of your topAnatData object is empty.")
  }
  if( length(topAnatData$organ.relationships) == 0 ) {
    stop("ERROR: the organ.relationships list of your topAnatData object is empty.")
  }
  if( length(topAnatData$organ.names[,1]) == 0 ) {
    stop("ERROR: the organ.names data frame of your topAnatData object is empty.")
  }

  ## Test if gene list is fine
  cat("\nChecking gene list......................\n")
  if (!is.factor(geneList)){
    geneList <- as.factor(geneList)
  }
  if (length(geneList) == 0){
    stop("ERROR: the gene list provided is empty.")
  }
  if (length(levels(geneList)) != 2){
    stop("ERROR: the gene list provided is not in the right format (should be a named vector including only 0 and 1 values).")
  }
  if (length(geneList) < 100) {
    cat("\nWARNING: Given the low number of genes provided, it is very unlikely that the test will have enough power.\n")
  }
  ## If geneList includes genes not present in topAnatData$gene2anatomy, restrict to these genes
  if (sum(names(geneList) %in% names(topAnatData$gene2anatomy)) != length(geneList)){
    cat("\nWARNING: Some genes in your gene list have no expression data in Bgee, and will not be included in the analysis.", sum(names(geneList) %in% names(topAnatData$gene2anatomy)), "genes in background will be kept.\n")
  }

  if (nodeSize == 0){
    stop("ERROR: the node size parameter has to be at least 1.")
  }
  if (!is.numeric(nodeSize)){
    stop("ERROR: the node size parameter should be a numeric, integer value.")
  }
  nodeSize <- as.integer(nodeSize)

  ## Building the modified topAnatData object. This also reports to the user how many genes are in the background / foreground
  topAnatObject <- new("topAnatData",
                         description = "topAnatData class object, ready for anatomical ontology enrichment test",
                         ontology = "UBERON ontology describing animal anatomical structures",
                         allGenes = geneList,
                         nodeSize = nodeSize,
                         parentMapping = topAnatData$organ.relationships,
                         gene2Nodes = topAnatData$gene2anatomy
                         )

  ## Create a hash from the topAnatObject and the topAnatData objects
  myAnalysisInfo <- c(topAnatObject, topAnatData)
  ## Use library digest to create a SHA512 hash, that will be used as analysis Id
  analysisId <- digest(myAnalysisInfo, algo = "sha512")
  ## cat(paste0("\nAnalysis Id hash built: ", analysisId, "\n"))

  ## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  ## The analysis id is used for our internal statistics. It is a secure hash that does not allow the
  ## identification of the gene lists and datasets used by the user, but will inform us on the
  ## number of distinct analyses that are run with the package.
  ## !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  ## Send a query to our webservice for statistics purposes of topAnat use (even when cached files are used)
  if (topAnatData$bgee.object$sendStats == TRUE){
    myUrl <- paste0(topAnatData$bgee.object$topAnatUrl, "?page=stats&action=launch_top_anat_analysis&api_key=", topAnatData$bgee.object$apiKey, "&analysis_id=", analysisId, "&source=BgeeDB_R_package&source_version=", as.character(packageVersion("BgeeDB")))
    ## Send query, but no need to wait for the answer
    try(getURL(myUrl, followLocation = TRUE, .opts = list(timeout = 1)), silent=TRUE)
  }

  return(topAnatObject)
}

#################################################################
## Sets of functions written with help of Adrian Alexa (author of the topGO package, personnal communication) to make topGO work with another ontology than the Gene Ontology

.annFUN.gene2Nodes <- function(feasibleGenes = NULL, gene2Nodes) {
  ## Restrict the mappings to the feasibleGenes set
  if(!is.null(feasibleGenes))
    gene2Nodes <- gene2Nodes[intersect(names(gene2Nodes), feasibleGenes)]

  ## Throw-up the genes which have no annotation
  if(any(is.na(gene2Nodes)))
    gene2Nodes <- gene2Nodes[!is.na(gene2Nodes)]

  gene2Nodes <- gene2Nodes[sapply(gene2Nodes, length) > 0]

  ## Get all the Terms and keep a one-to-one mapping with the genes
  allNodes <- unlist(gene2Nodes, use.names = FALSE)
  geneID <- rep(names(gene2Nodes), sapply(gene2Nodes, length))

  return(split(geneID, allNodes))
}
#################################################################

.buildGraph.topology <- function(knownNodes, parentMapping) {
  ## first build the lookUp table for the terms
  nodeLookUp <- new.env(hash = TRUE, parent = emptyenv())

  ## warping functions for a easier acces to the lookUp table
  isNodeInDAG <- function(node) {
    return(exists(node, envir = nodeLookUp, mode = 'logical', inherits = FALSE))
  }
  setNodeInDAG <- function(node) {
    assign(node, TRUE, envir = nodeLookUp)
  }

  ## get the root of the ontology
  GENE.ONTO.ROOT <- setdiff(unique(unlist(parentMapping, use.names = FALSE)),
      names(parentMapping))
  if (length(GENE.ONTO.ROOT) > 1){
    cat('\n Multiple roots were found... Possibly some nodes are not listed in the organ.names data frame\n')
  }

  ## we read all the database once and the access the list
  adjLookUP <- as.list(parentMapping)

  ## we use an environment of environments to store edges: (this way is faster)
  ## in the end we will coerce it to a list of list and build a graphNEL obj.
  edgeEnv <- new.env(hash = TRUE, parent = emptyenv())

  ## add the arc (u --> v) to edgeEnv of type :
  envAddEdge <- function(u, v) {
    assign(v, 0, envir = get(u, envir = edgeEnv))
  }

  ## recursivly build the induced graph starting from one node
  buildInducedGraph <- function(node) {
    ## if we have visited the node, there is nothing to do
    if(isNodeInDAG(node))
      return(1)

    ## we put the node in the graph and we get his parents
    setNodeInDAG(node)    # we visit the node
    assign(node, new.env(hash = TRUE, parent = emptyenv()), envir = edgeEnv) # adj list

    if(node == GENE.ONTO.ROOT)
      return(2)

    adjNodes <- adjLookUP[[node]]

    ## debuging point! should not happen!
    if(length(adjNodes) == 0)
      cat('\n There are no adj nodes for node: ', node, '\n')

    for(i in 1:length(adjNodes)) {
      x <- as.character(adjNodes[i])
      envAddEdge(node, x)
      buildInducedGraph(x)
    }

    return(0)
  }

  ## we start from the most specific nodes
  lapply(knownNodes, buildInducedGraph)

  ## now we must transform our env into a Graph structure
  ## for now we use lapply, later we can do it with eapply
  .graphNodes <- ls(edgeEnv)
  .edgeList <- eapply(edgeEnv,
                      function(adjEnv) {
                        aux <- as.list(adjEnv)
                        return(list(edges = match(names(aux), .graphNodes),
                                    weights = as.numeric(aux)))
                      })

  ## now we can build the graphNEL object
  return(new('graphNEL',
             nodes = .graphNodes,
             edgeL = .edgeList,
             edgemode = 'directed'))
}
######################## topAnatData class ########################
## This class is an extension of the topGOdata class from
## the topGO package. The additional fields are:
## - parentMapping: a list describing parent-child
##   relationships in the ontology to be used in place of
##   the Gene Ontology
## - gene2Nodes: a list providing the mapping of genes to
##   term from the new ontology

## Declare topAnatData as an extension of topGOdata class
setClass("topAnatData",
         representation = representation(
           ## Added slot to give us the structure of the ontology
           parentMapping = "character",
           ## Added slot to give us the mapping of genes to terms of the ontology
           gene2Nodes = "character"),
         contains = "topGOdata"
)

setMethod("initialize", "topAnatData",
          function(.Object,
                   ## which Ontology to be used
                   ontology = "Uberon",
                   ## a named numeric or factor, the names are the genes ID
                   allGenes,
                   ## function to select the signif. genes
                   geneSelectionFun = NULL,
                   ## description of the class
                   description = character(0),
                   ## minimum node size
                   nodeSize = 1,
                   ## the child-parent relationship
                   parentMapping,
                   ## annotation data
                   gene2Nodes,
                   ## additional parameters
                   ...) {

            .Object@description <- description
            .Object@ontology <- ontology

            ## some checking
            if(is.null(names(allGenes)))
              stop("allGenes must be a named vector")

            if(!is.factor(allGenes) && !is.numeric(allGenes))
              stop("allGenes should be a factor or a numeric vector")

            .Object@allGenes <- names(allGenes)

            if(is.factor(allGenes)) {
              if(length(levels(allGenes)) != 2)
                stop("allGenes must be a factor with 2 levels")
              .Object@allScores <- factor(as.character(allGenes))
              .Object@geneSelectionFun <- function(x) {
                return(as.logical(as.integer(levels(x)))[x])
              }
            }
            else {
              .Object@allScores <- as.numeric(allGenes)

              ## function to select which genes are significant
              if(is.null(geneSelectionFun))
                warning("No function to select the significant genes provided!")
              .Object@geneSelectionFun <- geneSelectionFun
            }

            ## size of the nodes which will be pruned
            .Object@nodeSize = as.integer(max(nodeSize, 1))

            ## this function is returning a list of terms from the specified ontology
            ## with each entry being a vector of genes
            cat("\nBuilding most specific Ontology terms... ")
            mostSpecificTerms <- .annFUN.gene2Nodes(feasibleGenes = .Object@allGenes, gene2Nodes = gene2Nodes)
            cat(" ( ", length(mostSpecificTerms), " Ontology terms found. )\n")

            ## the graph is build starting from the most specific terms
            cat("\nBuilding DAG topology................... ")
            g <- .buildGraph.topology(names(mostSpecificTerms), parentMapping)
            cat(" ( ",  graph::numNodes(g), " Ontology terms and ", graph::numEdges(g), " relations. )\n")

            ## probably is good to store the leves but for the moment we don't
            .nodeLevel <- buildLevels(g, leafs2root = TRUE)

            ## annotate the nodes in the graph with genes
            cat("\nAnnotating nodes (Can be long).......... ")
            g <- topGO:::mapGenes2GOgraph(g, mostSpecificTerms, nodeLevel = .nodeLevel) ## leafs2root

            ## select the feasible genes
            gRoot <- getGraphRoot(g)
            feasibleGenes <- ls(graph::nodeData(g, n = gRoot, attr = "genes")[[gRoot]])
            cat(" ( ", length(feasibleGenes), " genes annotated to the Ontology terms. )\n")

            .Object@feasible <- .Object@allGenes %in% feasibleGenes


            ## prune the GO graph
            if(.Object@nodeSize > 1) {
              cc <- .countsInNode(g, graph::nodes(g))
              .Object@graph <- graph::subGraph(names(cc)[cc >= .Object@nodeSize], g)
            } else {
              .Object@graph <-  g
            }

            .Object
          })

#' @title Deprecated : Retrieve Bgee RNA-seq or Affymetrix data.
#'
#' @description Deprecated : This function loads the quantitative expression data and presence calls 
#' for samples available from Bgee (rna_seq, affymetrix, sc_full_length).
#'
#' @param myBgeeObject A Reference Class Bgee object, notably specifying the targeted species 
#' and data type.
#'
#' @param experimentId Filter allowing to specify one or more ArrayExpress or GEO accession, e.g., 
#' GSE43721. Default is NULL: takes all available experiments for targeted species and data type.
#' 
#' @param sampleId Filter allowing to specify one or more sample ID. Depending on the selected 
#' datatype this sample IDs can correspond to Chip IDs (affymetrix) or RNA-Seq library IDs (rna_seq). 
#' Default is NULL: takes all available samples for targeted species and data type.
#'
#' @param anatEntityId Filter allowing to specify one or more anatomical entity IDs from the UBERON 
#' ontology (http://uberon.github.io/). Default is NULL: takes all available anatomical entities for 
#' targeted species and data type.
#'
#' @param stageId Filter allowing to specify one or more developmental stage IDs from Developmental 
#' Stage Ontology (https://github.com/obophenotype/developmental-stage-ontologies). Default is 
#' NULL: takes all available developmental stages for targeted species and data type.
#' 
#' @param cellTypeId Filter specific to single cell datatype (sc_full_length) allowing to specify 
#' one or more cell type IDs from the UBERON ontology (http://uberon.github.io/). Default is 
#' NULL: takes all available cell types for targeted species and data type. Available for Bgee 15.0 and after
#' 
#' @param sex Filter allowing to specify one or more sexes. Default is 
#' NULL: takes all available sexes for targeted species and data type. Available for Bgee 15.0 and after
#' 
#' @param strain Filter allowing to specify one or more strains. Default is 
#' NULL: takes all available strains for targeted species and data type. Available for Bgee 15.0 and after
#' 
#' @param withDescendantAnatEntities Allows to filter on the selected anatEntityId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#'
#' @param withDescendantStages Allows to filter on the selected stageId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#'
#' @param withDescendantCellTypes Allows to filter on the selected cellTypeId and all its descendants.
#' This functionality is available for Bgee 15.0 release and after
#'
#' @return Return a dataframe containing all Bgee processed expression data from the selected species 
#' and datatype using specified filters with operator AND.
#'
#' @author Julien Wollbrett, Andrea Komljenovic and Julien Roux.
#'
#' @examples{
#'   bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
#'   dataMouseGSE43721 <- getData(bgee, experimentId = "GSE43721")
#'   dataMouseVariousFilters <- getData(bgee, experimentId = c("GSE43721", "GSE36026"), 
#'                              anatEntityId = c("UBERON:0002107", "UBERON:0000956", "UBERON:0002048"))
#' }
#'
#' @import RSQLite
#' @importFrom R.utils gunzip
#' @export
#' 
getData <- function(myBgeeObject, experimentId = NULL, sampleId = NULL, 
    anatEntityId = NULL, stageId = NULL, cellTypeId = NULL, sex = NULL, strain = NULL, 
    withDescendantAnatEntities = FALSE, withDescendantStages = FALSE, 
    withDescendantCellTypes = FALSE) {
  .Deprecated(new = "getSampleProcessedData")
  check_object(myBgeeObject)
  # write a warning message if user tried to retrieve ontology terms with descendants for
  # a bgee release where this functionality was not yet implemented
  compare_version_number <- gsub("_", ".", myBgeeObject$release)
  if ((withDescendantAnatEntities | withDescendantStages | withDescendantCellTypes) &
      compareVersion(a = compare_version_number , b = "15.0") < 0) {
    message("withDescendant functionality is available only for Bgee 15.0",
            " release and after. Will not retrieve descendant of selected parameters.")
  }
  if (withDescendantAnatEntities & compareVersion(a = compare_version_number , b = "15.0") >= 0) {
    if(is.null(anatEntityId)) {
      warning("No anatomical entity was provided. Not possible to filter on descendant anatomical entities.")
    }
    anatEntityId <- c(anatEntityId, getDescendantAnatEntities(bgee = myBgeeObject, ids = anatEntityId))
  }
  if (withDescendantStages & compareVersion(a = compare_version_number , b = "15.0") >= 0) {
    if(is.null(stageId)) {
      warning("No developmental stage was provided. Not possible to filter on descendant developmental stages.")
    }
    stageId <- c(stageId, getDescendantStages(bgee = myBgeeObject, ids = stageId))
  }
  if (withDescendantCellTypes & compareVersion(a = compare_version_number , b = "15.0") >= 0) {
    if(is.null(cellTypeId)) {
      warning("No cell type was provided. Not possible to filter on descendant cell types.")
    }
    cellTypeId <- c(cellTypeId, getDescendantCellTypes(bgee = myBgeeObject, ids = cellTypeId))
  }
  check_condition_parameters(myBgeeObject = myBgeeObject, anatEntityId = anatEntityId, 
    stageId = stageId, cellTypeId = cellTypeId, sex = sex, strain = strain)
  import_data(myBgeeObject = myBgeeObject, experimentId = experimentId, sampleId = sampleId, 
    anatEntityId = anatEntityId, stageId = stageId, cellTypeId = cellTypeId, sex = sex, 
    strain = strain)
  return(query_data(myBgeeObject = myBgeeObject, experimentId = experimentId, sampleId = sampleId, 
    anatEntityId = anatEntityId, stageId = stageId, cellTypeId = cellTypeId, sex = sex, 
    strain = strain))
}

context("expect_output")


test_that("Single experiment gene expression files", {
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  data_bgee_mouse_exp <- getData(bgee, experimentId  = "GSE43721")

  expect_that( data_bgee_mouse_exp, is_a("data.frame") )
})

context("expect_output")


test_that("Gene expression files", {
  bgee <- Bgee$new(species = "Drosophila_simulans", dataType = "rna_seq")
  data_bgee_mouse <- getData(bgee)


  expect_that( data_bgee_mouse, is_a("data.frame") )
  expect_message(message("Save data in local sqlite database"))
  expect_message(message("Load queried data. The query is : SELECT * from rna_seq"))

})

---
title: "BgeeDB, an R package for retrieval of curated expression datasets and for gene list enrichment tests"
author: "Andrea Komljenovic, Julien Roux, Marc Robinson-Rechavi, Frederic Bastian"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{BgeeDB, an R package for retrieval of curated expression datasets and for gene list enrichment tests}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```BgeeDB``` is a collection of functions to import data from the Bgee database (<https://bgee.org/>) directly into R, and to facilitate downstream analyses, such as gene set enrichment test based on expression of genes in anatomical structures. Bgee provides annotated and processed expression data and expression calls from curated wild-type healthy samples, from human and many other animal species.
 
The package retrieves the annotation of RNA-seq, single cell full length RNA-seq and Affymetrix experiments integrated into the Bgee database, and downloads into R the quantitative data and expression calls produced by the Bgee pipeline. The package also allows to run GO-like enrichment analyses based on anatomical terms, where genes are mapped to anatomical terms by expression patterns, based on the ```topGO``` package. This is the same as the TopAnat web-service available at (<https://www.bgee.org/analysis/top-anat/>), but with more flexibility in the choice of parameters and developmental stages.

In summary, the BgeeDB package allows to: 
* 1. List annotation of bulk RNA-seq, single cell RNA-seq and microarray data available the Bgee database
* 2. Download the processed gene expression data available in the Bgee database
* 3. Download the gene expression calls and use them to perform TopAnat analyses 

The pipeline used to generate Bgee expression data is documented and publicly available at (<https://github.com/BgeeDB/bgee_pipeline>)

If you find a bug or have any issues to use ```BgeeDB``` please write a bug report in our own GitHub issues manager available at (<https://github.com/BgeeDB/BgeeDB_R/issues>)

## Installation

In R:
``` {r}
#if (!requireNamespace("BiocManager", quietly=TRUE))
    #install.packages("BiocManager")
#BiocManager::install("BgeeDB")
```

In case BgeeDB is run on Windows please be sure your OS has curl installed. It is installed by default on Windows 10, version 1803 or later. If Git for Windows is installed on the OS then curl is already installed. If not installed please install it before running BgeeDB in order to avoid potential timeout errors when downloading files. 

## How to use BgeeDB package

### Load the package
``` {r, message = FALSE, warning = FALSE}
library(BgeeDB)
```

### Running example: downloading and formatting processed RNA-seq data

#### List available species in Bgee
The ```listBgeeSpecies()``` function allows to retrieve available species in the Bgee database, and which data types are available for each species. 

``` {r}
listBgeeSpecies()
```

It is possible to list all species from a specific release of Bgee with the ```release``` argument (see ```listBgeeRelease()``` function), and order the species according to a specific columns with the ```ordering``` argument. For example:

``` {r}
listBgeeSpecies(release = "13.2", order = 2)
```

#### Create a new Bgee object

In the following example we will choose to focus on zebrafish ("Danio\_rerio") RNA-seq. Species can be specified using their name or their NCBI taxonomic IDs. To specify that RNA-seq data want to be downloaded, the ```dataType``` argument is set to "rna\_seq". To download Affymetrix microarray data, set this argument to "affymetrix". To download single cell full length RNA-seq data, set this argument to "sc_full_length". To download droplet based single cell RNA-seq data, set this argument to "sc_droplet_based".   

``` {r}
bgee <- Bgee$new(species = "Danio_rerio", dataType = "rna_seq")
```

*Note 1*: It is possible to work with data from a specific release of Bgee by specifying the ```release``` argument, see ```listBgeeRelease()``` function for available releases. 

*Note 2*: The functions of the package will store the downloaded files in a versioned folder created by default in the working directory. These cache files allow faster re-access to the data. The directory where data are stored can be changed with the ```pathToData``` argument.

#### Retrieve the annotation of zebrafish RNA-seq datasets

The ```getAnnotation()``` function will output the list of bulk RNA-seq experiments and libraries available in Bgee for zebrafish. 

``` {r}
annotation_bgee_zebrafish <- getAnnotation(bgee)
# list the first experiments and libraries
lapply(annotation_bgee_zebrafish, head)
```

#### Download the processed zebrafish bulk RNA-seq data

The ```getSampleProcessedData()``` function will download processed bulk RNA-seq data from all zebrafish experiments in Bgee as a data frame.

``` {r}
# download all RNA-seq experiments from zebrafish
data_bgee_zebrafish <- getSampleProcessedData(bgee)
# number of experiments downloaded
length(data_bgee_zebrafish)
# check the downloaded data
lapply(data_bgee_zebrafish, head)
# isolate the first experiment
data_bgee_experiment1 <- data_bgee_zebrafish[[1]]
```

The result of the ```getSampleProcessedData()``` function is a data frame. Each row is a gene and the expression levels are displayed as raw read counts, RPKMs (up to Bgee 13.2), TPMs (from Bgee 14.0), or FPKMs (from Bgee 14.0 to Bgee 14.2). A detection flag indicates if the gene is significantly expressed above background level of expression. From Bgee 15.0 a pValue allows to have a precise metric indicating how much the gene is significantly expressed above background level of expression (the detection flag is still available and a gene is considered as present if pValue < 0.05).

*Note*: If microarray data are downloaded, rows corresponding to probesets and log2 of expression intensities are available instead of read counts/RPKMs/TPMs/FPKMs.

Alternatively, you can choose to download data from one RNA-seq experiment from Bgee with the `experimentId` parameter:

``` {r}
# download data for GSE68666
data_bgee_zebrafish_gse68666 <- getSampleProcessedData(bgee, experimentId = "GSE68666")
```

It is possible to download data by combining filters : 
* experimentId : one or more experimentId,
* sampleId : one or more sampleId (i.e libraryId for RNA-Seq and ChipId for Affymetrix),
* anatEntityId : one or more anatomical entity ID from the UBERON ontology (https://uberon.github.io/),
* withDescendantAnatEntities : filter on the selected anatEntityId and all its descendants (from Bgee 15.0),
* stageId : one or more developmental stage ID from the developmental stage ontologies (https://github.com/obophenotype/developmental-stage-ontologies),
* withDescendantStages : filter on the selected stageId and all its descendants (from Bgee 15.0)
* cellTypeId : one or more cell type, only for single cell datatype (from Bgee 15.0),
* withDescendantCellTypes : filter on the selected cellTypeId and all its descendants (from Bgee 15.0)
* sex : one or more sex (from Bgee 15.0),
* strain : one or more strain (from Bgee 15.0),
``` {r eval=FALSE}
# Examples of data downloading using different filtering combination
# retrieve zebrafish RNA-Seq data for heart (UBERON:0000955) or brain (UBERON:0000948)
data_bgee_zebrafish_filters <- getSampleProcessedData(bgee, anatEntityId = c("UBERON:0000955","UBERON:0000948"))
# retrieve zebrafish RNA-Seq data for embryo (UBERON:0000922) part of the experiment GSE68666
data_bgee_zebrafish_filters <- getSampleProcessedData(bgee, experimentId = "GSE68666", anatEntityId = "UBERON:0000922")
# retrieve zebrafish RNA-Seq data for head kidney (UBERON:0007132) or swim bladder (UBERON:0006860) from post-juvenile adult stage (UBERON:0000113)
data_bgee_zebrafish_filters <- getSampleProcessedData(bgee, stageId = "UBERON:0000113", anatEntityId = c("UBERON:0007132","UBERON:0006860"))
# retrieve zebrafish RNA-Seq data for brain (UBERON:0000948) and all substructures of brain from post-juvenile adult stage (UBERON:0000113)
data_bgee_zebrafish_filters <- getSampleProcessedData(bgee, stageId = "UBERON:0000113", anatEntityId = "UBERON:0000948", withDescendantAnatEntities = TRUE)
```

#### Format the sample level bulk RNA-seq data

It is sometimes easier to work with data organized as a matrix, where rows represent genes or probesets and columns represent different samples. The ```formatData()``` function reformats the data into an ExpressionSet object including:
* An expression data matrix, with genes or probesets as rows, and samples as columns (```assayData``` slot). The ```stats``` argument allows to choose if the matrix should be filled with read counts, RPKMs (up to Bgee 13.2), FPKMs (from Bgee 14.0 to Bgee 14.2), or TPMs (from Bgee 14.0) for RNA-seq data. For microarray data the matrix is filled with log2 expression intensities.
* A data frame listing the samples and their anatomical structure and developmental stage annotation (```phenoData``` slot)
* For microarray data, the mapping from probesets to Ensembl genes (```featureData``` slot)

The ```callType``` argument allows to retain only actively expressed genes or probesets, if set to "present" or "present high quality". Genes or probesets that are absent in a given sample are given ```NA``` values.

```{r}
# use only present calls and fill expression matrix with TPM values
gene.expression.zebrafish.tpm <- formatData(bgee, data_bgee_zebrafish_gse68666, callType = "present", stats = "tpm")
gene.expression.zebrafish.tpm 
```
#### Download cell level single-cell RNA-seq data

The ```getCellProcessedData()``` function will download processed single-cell RNA-seq data at the cell level. In the following example we will focus on downloading UMI counts at cell level for the Gallus gallus experiment ERP132576. The data are stored as h5ad files and can be opened using either the package \"zellkonverter\" (default) or the package \"anndata\" with the attribute ```package = "anndata```. Both \"zellkonverter\" and \"anndata\" packages require a configuration step when used the first time.

``` {r eval=FALSE}
#create a bgee object to download droplet based data from Gallus gallus
bgee <- Bgee$new(species = "Gallus_gallus", dataType = "sc_droplet_based")
# download cell data for one RNA-seq experiment
cell_data_bgee_gallus_gallus <- getCellProcessedData(bgee, experimentId = "ERP132576")
```

The result of the ```getCellProcessedData()``` depends on the package used to load the data. With \"zellkonverter\" it is an object of the class SingleCellExperiment. With \"anndata\" it is an object of the class AnnData. The expression levels are displayed as raw UMI counts.

### Running example: TopAnat gene expression enrichment analysis

For some documentation on the TopAnat analysis, please refer to our publications, or to the web-tool page (<https://www.bgee.org/analysis/top-anat/>).

#### Create a new Bgee object
Similarly to the quantitative data download example above, the first step of a topAnat analysis is to built an object from the Bgee class. For this example, we will focus on zebrafish:

```{r}
# Creating new Bgee class object
bgee <- Bgee$new(species = "Danio_rerio", release = "15.2")
```

*Note* : We are free to specify any data type of interest using the ```dataType``` argument among `rna_seq`, `sc_full_length`, `sc_droplet_based`, `affymetrix`, `est` or `in_situ`, or even a combination of data types. If nothing is specified, as in the above example, all data types available for the targeted species are used. This equivalent to specifying `dataType=c("rna_seq","sc_full_length","sc_droplet_based","affymetrix","est","in_situ")`.

#### Download the data allowing to perform TopAnat analysis

The ```loadTopAnatData()``` function loads a mapping from genes to anatomical structures based on calls of expression in anatomical structures. It also loads the structure of the anatomical ontology and the names of anatomical structures. 

```{r}
# Loading calls of expression
myTopAnatData <- loadTopAnatData(bgee)
# Look at the data
## str(myTopAnatData)
```

The stringency on the quality of expression calls can be changed with the ```confidence``` argument. Finally, if you are interested in expression data coming from a particular developmental stage or a group of stages, please specify the a Uberon stage Id in the ```stage``` argument. 

```{r, eval=FALSE}
## Loading silver and gold expression calls from affymetrix data made on embryonic samples only 
## This is just given as an example, but is not run in this vignette because only few data are returned
bgee <- Bgee$new(species = "Danio_rerio", dataType="affymetrix", release = "15.2")
myTopAnatData <- loadTopAnatData(bgee, stage="UBERON:0000068", confidence="silver")
```

*Note 1*: As mentioned above, the downloaded data files are stored in a versioned folder that can be set with the ```pathToData``` argument when creating the Bgee class object (default is the working directory). If you query again Bgee with the exact same parameters, these cached files will be read instead of querying the web-service again. **It is thus important, if you plan to reuse the same data for multiple parallel topAnat analyses, to plan to make use of these cached files instead of redownloading them for each analysis.** The cached files also give the possibility to repeat analyses offline.

*Note 2*: In releases up to Bgee 13.2 allowed ```confidence`` values were `low_quality` or or `high_quality`. Starting from Bgee 14.0 ```confidence``` values are `gold` or `silver`.

#### Prepare a topAnatData object allowing to perform TopAnat analysis with topGO

First we need to prepare a list of genes in the foreground and in the background. The input format is the same as the gene list required to build a ```topGOdata``` object in the ```topGO``` package: a vector with background genes as names, and 0 or 1 values depending if a gene is in the foreground or not. In this example we will look at genes with an annotated phenotype related to "pectoral fin" . We use the ```biomaRt``` package to retrieve this list of genes. We expect them to be enriched for expression in male tissues, notably the testes. The background list of genes is set to all genes annotated to at least one Gene Ontology term, allowing to account for biases in which types of genes are more likely to receive Gene Ontology annotation.

```{r, eval=FALSE}
# if (!requireNamespace("BiocManager", quietly=TRUE))
    # install.packages("BiocManager")
# BiocManager::install("biomaRt")
library(biomaRt)
ensembl <- useMart("ENSEMBL_MART_ENSEMBL", dataset="drerio_gene_ensembl", host="mar2016.archive.ensembl.org")

# get the mapping of Ensembl genes to phenotypes. It will corresponds to the background genes
universe <- getBM(filters=c("phenotype_source"), value=c("ZFIN"), attributes=c("ensembl_gene_id","phenotype_description"), mart=ensembl)

# select phenotypes related to pectoral fin
phenotypes <- grep("pectoral fin", unique(universe$phenotype_description), value=T)

# Foreground genes are those with an annotated phenotype related to "pectoral fin" 
myGenes <- unique(universe$ensembl_gene_id[universe$phenotype_description %in% phenotypes])

# Prepare the gene list vector 
geneList <- factor(as.integer(unique(universe$ensembl_gene_id) %in% myGenes))
names(geneList) <- unique(universe$ensembl_gene_id)
summary(geneList)

# Prepare the topGO object
myTopAnatObject <-  topAnat(myTopAnatData, geneList)
```

The above code using the `biomaRt` package is not executed in this vignette to prevent building issues of our package in case of biomaRt downtime. Instead we use a `geneList` object saved in the `data/` folder that we built using pre-downloaded data. 

```{r}
data(geneList)
myTopAnatObject <-  topAnat(myTopAnatData, geneList)
```

*Warning*: This can be long, especially if the gene list is large, since the Uberon anatomical ontology is large and expression calls will be propagated through the whole ontology (e.g., expression in the forebrain will also be counted as expression in parent structures such as the brain, nervous system, etc). Consider running a script in batch mode if you have multiple analyses to do.

#### Launch the enrichment test

For this step, see the vignette of the ```topGO``` package for more details, as you have to directly use the tests implemented in the ```topGO``` package, as shown in this example:

```{r}
results <- runTest(myTopAnatObject, algorithm = 'weight', statistic = 'fisher')
```

*Warning*: This can be long because of the size of the ontology. Consider running scripts in batch mode if you have multiple analyses to do.

#### Format the table of results after an enrichment test for anatomical terms

The ```makeTable``` function allows to filter and format the test results, and calculate FDR values. 

```{r}
# Display results sigificant at a 1% FDR threshold
tableOver <- makeTable(myTopAnatData, myTopAnatObject, results, cutoff = 0.01)
head(tableOver)
```

At the time of building this vignette (Sept. 2024), there was 9 significant anatomical structures. The first term is “pectoral fin”, and the second “pectoral appendage bud”. Other terms in the list, especially those with high enrichment folds, are clearly related to pectoral fins or substructures of fins. This analysis shows that genes with phenotypic effects on pectoral fins are specifically expressed in or next to these structures

By default results are sorted by p-value, but this can be changed with the ```ordering``` parameter by specifying which column should be used to order the results (preceded by a "-" sign to indicate that ordering should be made in decreasing order). For example, it is often convenient to sort  significant structures by decreasing enrichment fold, using `ordering = -6`. The full table of results can be obtained using `cutoff = 1`.

Of note, it is possible to retrieve for a particular tissue the significant genes that were mapped to it.

```{r}
# In order to retrieve significant genes mapped to the term " paired limb/fin bud"
term <- "UBERON:0004357"
termStat(myTopAnatObject, term) 

# 172 genes mapped to this term for Bgee 15.2
genesInTerm(myTopAnatObject, term)
# 37 significant genes mapped to this term for Bgee 15.2
annotated <- genesInTerm(myTopAnatObject, term)[["UBERON:0004357"]]
annotated[annotated %in% sigGenes(myTopAnatObject)]
```

*Warning*: it is debated whether FDR correction is appropriate on enrichment test results, since tests on different terms of the ontologies are not independent. A nice discussion can be found in the vignette of the ```topGO``` package.

#### Store expression data localy

Since version 2.14.0 (Bioconductor 3.11) BgeeDB store downloaded expression data in a local RSQLite database.
The advantages of this approach compared to the one used in the previous BgeeDB versions are:
* do not anymore need a server with lot of memory to access to subset of huge dataset (e.g GTeX dataset)
* more granular filtering using arguments in the getSampleProcessedData() function
* do not download twice the same data
* fast access to data once integrated in the local database

This approach comes with some drawbacks :
* the SQLite local database use more disk space than the previously conpressed .rds approach
* first access to a dataset takes more time (integration to SQLite local database is time consuming)

It is possible to remove .rds files generated in previous versions of BgeeDB and not used anymore since version
2.14.0. The function below delete all .rds files for the selected species and for all datatype.

```{r eval = FALSE}
bgee <- Bgee$new(species="Mus_musculus", release = "14.1")
# delete all old .rds files of species Mus musculus
deleteOldData(bgee)
```

As the new SQLite approach use more disk space it is now possible to delete all local data of one species from one release of Bgee.

```{r eval = FALSE}
bgee <- Bgee$new(species="Mus_musculus", release = "14.1")
# delete local SQLite database of species Mus musculus from Bgee 14.1
deleteLocalData(bgee)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getData.R
\name{getData}
\alias{getData}
\title{Deprecated : Retrieve Bgee RNA-seq or Affymetrix data.}
\usage{
getData(
  myBgeeObject,
  experimentId = NULL,
  sampleId = NULL,
  anatEntityId = NULL,
  stageId = NULL,
  cellTypeId = NULL,
  sex = NULL,
  strain = NULL,
  withDescendantAnatEntities = FALSE,
  withDescendantStages = FALSE,
  withDescendantCellTypes = FALSE
)
}
\arguments{
\item{myBgeeObject}{A Reference Class Bgee object, notably specifying the targeted species
and data type.}

\item{experimentId}{Filter allowing to specify one or more ArrayExpress or GEO accession, e.g.,
GSE43721. Default is NULL: takes all available experiments for targeted species and data type.}

\item{sampleId}{Filter allowing to specify one or more sample ID. Depending on the selected
datatype this sample IDs can correspond to Chip IDs (affymetrix) or RNA-Seq library IDs (rna_seq).
Default is NULL: takes all available samples for targeted species and data type.}

\item{anatEntityId}{Filter allowing to specify one or more anatomical entity IDs from the UBERON
ontology (http://uberon.github.io/). Default is NULL: takes all available anatomical entities for
targeted species and data type.}

\item{stageId}{Filter allowing to specify one or more developmental stage IDs from Developmental
Stage Ontology (https://github.com/obophenotype/developmental-stage-ontologies). Default is
NULL: takes all available developmental stages for targeted species and data type.}

\item{cellTypeId}{Filter specific to single cell datatype (sc_full_length) allowing to specify
one or more cell type IDs from the UBERON ontology (http://uberon.github.io/). Default is
NULL: takes all available cell types for targeted species and data type. Available for Bgee 15.0 and after}

\item{sex}{Filter allowing to specify one or more sexes. Default is
NULL: takes all available sexes for targeted species and data type. Available for Bgee 15.0 and after}

\item{strain}{Filter allowing to specify one or more strains. Default is
NULL: takes all available strains for targeted species and data type. Available for Bgee 15.0 and after}

\item{withDescendantAnatEntities}{Allows to filter on the selected anatEntityId and all its descendants.
This functionality is available for Bgee 15.0 release and after}

\item{withDescendantStages}{Allows to filter on the selected stageId and all its descendants.
This functionality is available for Bgee 15.0 release and after}

\item{withDescendantCellTypes}{Allows to filter on the selected cellTypeId and all its descendants.
This functionality is available for Bgee 15.0 release and after}
}
\value{
Return a dataframe containing all Bgee processed expression data from the selected species
and datatype using specified filters with operator AND.
}
\description{
Deprecated : This function loads the quantitative expression data and presence calls
for samples available from Bgee (rna_seq, affymetrix, sc_full_length).
}
\examples{
{
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  dataMouseGSE43721 <- getData(bgee, experimentId = "GSE43721")
  dataMouseVariousFilters <- getData(bgee, experimentId = c("GSE43721", "GSE36026"), 
                             anatEntityId = c("UBERON:0002107", "UBERON:0000956", "UBERON:0002048"))
}

}
\author{
Julien Wollbrett, Andrea Komljenovic and Julien Roux.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCellProcessedData.R
\name{getCellProcessedData}
\alias{getCellProcessedData}
\title{Retrieve Bgee processed expression values at cell level for single cell data in Bgee.}
\usage{
getCellProcessedData(myBgeeObject, experimentId, package = "zellkonverter")
}
\arguments{
\item{myBgeeObject}{A Reference Class Bgee object, notably specifying the targeted species
and data type.}

\item{experimentId}{Filter allowing to specify one ArrayExpress or GEO accession, e.g.,
GSE43721. Can not be null}

\item{package}{Name of the R package used to load the H5AD cell data. It can either be \"anndata\"
or \"zellkonverter\". If zellkonverter is used then its default reader (R) is used.}
}
\description{
This function loads the processed gene count matrix of single cell experiments in Bgee.
These data are available for all droplet based single-cell RNA-seq and full length single-cell RNA-seq.
These files are stored in the H5AD format.
}
\examples{
{
\dontrun{
  bgee <- Bgee$new(species = "Gallus_gallus", dataType = "sc_droplet_based")
  cellProcessedData <- getCellProcessedData(bgee, experimentId = "ERP132576")
}
}

}
\author{
Julien Wollbrett
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listBgeeSpecies.R
\name{listBgeeSpecies}
\alias{listBgeeSpecies}
\title{List species in the Bgee database and the available data types for each of them}
\usage{
listBgeeSpecies(
  release = NULL,
  ordering = NULL,
  allReleases = NULL,
  removeFile = TRUE
)
}
\arguments{
\item{release}{A character specifying a targeted release number. In the form "Release.subrelease" or "Release_subrelease", e.g., "13.2" or 13_2". If not specified, the latest release is used.}

\item{ordering}{A numeric indicating the number of the column which should be used to sort the data frame. Default NULL, returning unsorted data frame.}

\item{allReleases}{A data frame with information on all releases. Avoid redownloading this information if .getBgeeRelease() already called.}

\item{removeFile}{Boolean indicating whether the downloaded file should be deleted. Default to TRUE.}
}
\value{
A data frame with species Id, genus name, species name, common name and data type availability for targeted Bgee release
}
\description{
Returns information on available species in Bgee
}
\examples{
{
 listBgeeSpecies()
 # species present in a specific Bgee release
 listBgeeSpecies(release = "13.2")
 # in order to order species according to theit taxonomical IDs
 listBgeeSpecies(ordering = 1)
}

}
\author{
Julien Roux
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/makeTable.R
\name{makeTable}
\alias{makeTable}
\title{Formats results of the enrichment test on anatomical structures.}
\usage{
makeTable(topAnatData, topAnatObject, results, cutoff = 1, ordering = 7)
}
\arguments{
\item{topAnatData}{A list produced by the function loadTopAnatData().}

\item{topAnatObject}{An object produced by the function topAnat().}

\item{results}{A result object, produced by the runtest() function of topGO.}

\item{cutoff}{An FDR cutoff between 0 and 1. Only terms with FDR lower than this cutoff are included.
Default is 1, meaning that all terms are included.}

\item{ordering}{A numeric indicating which column should be used to sort the data frame. If the column number is preceded by a \"-\" sign, results are displayed in decreasing ordering. Default is "7", returning data frame sorted by p-values in increasing order.}
}
\value{
A data frame with significantly enriched anatomical structures, sorted by p-value.
}
\description{
This function loads the results from the topGO test and creates an output table with organ names,
fold enrichment and FDR. Data are sorted by p-value and only terms below the specified FDR cutoff are included.
}
\examples{
{
bgee <- Bgee$new(species="Bos_taurus", dataType="rna_seq")
myTopAnatData <- loadTopAnatData(bgee, stage="UBERON:0000092")
geneList <- as.factor(c(rep(0, times=85), rep(1, times=15)))
names(geneList) <- c("ENSBTAG00000000011","ENSBTAG00000000014","ENSBTAG00000000016",
                     "ENSBTAG00000000026","ENSBTAG00000000039","ENSBTAG00000000040",
                     "ENSBTAG00000000042","ENSBTAG00000000050","ENSBTAG00000000056",
                     "ENSBTAG00000000064","ENSBTAG00000000067","ENSBTAG00000000071",
                     "ENSBTAG00000000072","ENSBTAG00000000080","ENSBTAG00000000081",
                     "ENSBTAG00000000084","ENSBTAG00000000091","ENSBTAG00000000099",
                     "ENSBTAG00000000111","ENSBTAG00000000123","ENSBTAG00000000132",
                     "ENSBTAG00000000153","ENSBTAG00000000162","ENSBTAG00000000163",
                     "ENSBTAG00000000169","ENSBTAG00000000179","ENSBTAG00000000197",
                     "ENSBTAG00000000199","ENSBTAG00000000202","ENSBTAG00000000203",
                     "ENSBTAG00000000204","ENSBTAG00000000213","ENSBTAG00000000215",
                     "ENSBTAG00000000223","ENSBTAG00000000224","ENSBTAG00000000225",
                     "ENSBTAG00000000236","ENSBTAG00000000250","ENSBTAG00000000251",
                     "ENSBTAG00000000252","ENSBTAG00000000253","ENSBTAG00000000261",
                     "ENSBTAG00000000274","ENSBTAG00000000277","ENSBTAG00000000279",
                     "ENSBTAG00000000285","ENSBTAG00000000286","ENSBTAG00000000287",
                     "ENSBTAG00000000289","ENSBTAG00000000297","ENSBTAG00000000305",
                     "ENSBTAG00000000312","ENSBTAG00000000328","ENSBTAG00000000335",
                     "ENSBTAG00000000341","ENSBTAG00000000343","ENSBTAG00000000354",
                     "ENSBTAG00000000355","ENSBTAG00000000356","ENSBTAG00000000365",
                     "ENSBTAG00000000372","ENSBTAG00000000379","ENSBTAG00000000380",
                     "ENSBTAG00000000382","ENSBTAG00000000396","ENSBTAG00000000404",
                     "ENSBTAG00000000405","ENSBTAG00000000406","ENSBTAG00000000411",
                     "ENSBTAG00000000425","ENSBTAG00000000434","ENSBTAG00000000435",
                     "ENSBTAG00000000438","ENSBTAG00000000448","ENSBTAG00000000451",
                     "ENSBTAG00000000454","ENSBTAG00000000456","ENSBTAG00000000457",
                     "ENSBTAG00000000459","ENSBTAG00000000462","ENSBTAG00000000469",
                     "ENSBTAG00000000470","ENSBTAG00000000484","ENSBTAG00000000497",
                     "ENSBTAG00000000501","ENSBTAG00000009707","ENSBTAG00000026266",
                     "ENSBTAG00000021992","ENSBTAG00000005353","ENSBTAG00000005333",
                     "ENSBTAG00000006424","ENSBTAG00000026972","ENSBTAG00000010799",
                     "ENSBTAG00000010799","ENSBTAG00000014614","ENSBTAG00000014614",
                     "ENSBTAG00000045757","ENSBTAG00000046332","ENSBTAG00000046332",
                     "ENSBTAG00000008394")
myTopAnatObject <-  topAnat(myTopAnatData, geneList)
resFis <- runTest(myTopAnatObject, algorithm = 'elim', statistic = 'fisher')
## Format results
tableOver <- makeTable(myTopAnatData, myTopAnatObject, resFis, 0.1)
}
}
\author{
Julien Roux
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deleteLocalData.R
\name{deleteOldData}
\alias{deleteOldData}
\title{Delete .rds data of one species coming from an old version of the BgeeDB R package.}
\usage{
deleteOldData(myBgeeObject)
}
\arguments{
\item{myBgeeObject}{A Reference Class Bgee object, notably specifying the targeted species}
}
\description{
Since Bioconductor 3.11 BgeeDB store data in a local SQLite database allowing
to optimize memory usage.
This function allows to delete .rds files used to store local data from BgeeDB versions older
than Bioconductor 3.11, for the species selected in the reference class Bgee object.
}
\examples{
{
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  deleteOldData(bgee)
}

}
\author{
Julien Wollbrett
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getIntegratedCalls.R
\name{getIntegratedCalls}
\alias{getIntegratedCalls}
\title{Retrieve Bgee calls from one species.}
\usage{
getIntegratedCalls(
  myBgeeObject,
  conditionParameters = "anatEntity",
  advancedColumns = FALSE,
  geneIds = NULL,
  anatEntityIds = NULL
)
}
\arguments{
\item{myBgeeObject}{A Reference Class Bgee object, notably specifying the targeted species
and data type.}

\item{conditionParameters}{Specify which condition parameter you are interested
in. It can be \code{anatEntity} or \code{allCondParams}. The \code{anatEntity} option retrieve calls generated
using only anatomical entity and cell-type as a condition parameter. The \code{allCondParams} option retrieve calls generated taking
into consideration all condition parameters present in Bgee (anat, entity, cell-type, developmental stage,
sex and strain). By default the \code{anatEntity} option is selected.}

\item{advancedColumns}{Boolean allowing to specify if advancend columns are required. The default value
is \code{FALSE} meaning that only gene , condition parameters, call quality, FDR, expression rank and
expression score are retrieved. If \code{TRUE} then a lot more information used to generate the calls are
retrieved like the number of self and descendant observation for each datatype, or the rank and score
per datatype.}

\item{geneIds}{List of genes for which expression calls have to be retrieved. It has to be ensembl
IDs if the Bgee genome source is Ensembl (e.g ENSG00000244734) or RefSeq IDs if the Bgee genome source
is RefSeq (e.g 734881) but not gene names (e.g HBB, Apoc1, etc.).}

\item{anatEntityIds}{List of anatomical entity IDs for which expression calls have to be retrieved.
It has to be IDs (e.g UBERON:0000955) and not names (e.g brain)}
}
\value{
Return a data.table containing all calls from the specified species
}
\description{
This function loads the integrated expression calls from one species. These calls are
equivalent to the one provided in the gene page of the Bgee website. The calls have been
generated using all datatypes present in Bgee (EST, In Situ, Affymetrix, bulk RNA-Seq and
single-cell RNA-Seq).
}
\examples{
{
  bgee <- Bgee$new(species = "Danio_rerio")
  callsHer1 <- getIntegratedCalls(bgee, geneIds="ENSDARG00000014722")
}

}
\author{
Julien Wollbrett
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/topAnat.R
\name{topAnat}
\alias{topAnat}
\title{Produces an object allowing to perform GO-like enrichment of anatomical terms using the topGO package}
\usage{
topAnat(topAnatData, geneList, nodeSize = 10, ...)
}
\arguments{
\item{topAnatData}{a list including a gene2anatomy list, an organ.relationships list and an organ.names data.frame, produced by the function loadTopAnatData().}

\item{geneList}{Vector indicating foreground and background genes. Names of the vector indicate the background genes. Values are 1 (gene in foreground) or 0 (gene not in foreground).}

\item{nodeSize}{Minimum number of genes mapped to a node for it to be tested. Default is 10.}

\item{...}{Additional parameters as passed to build topGOdata object in topGO package.}
}
\value{
topAnatObject, a topAnatData class object, ready for gene set enrichment testing with topGO.
}
\description{
This function produces a topAnatObject, ready to use for gene set enrichment testing using functions from the topGO package. This object uses the Uberon ontology instead of the GO ontology.
}
\details{
To perform the enrichment test for expression in anatomical structures for each term of Uberon ontology (browsable at \url{http://www.ontobee.org/ontology/UBERON}), the data are formatted to use the topGO package for testing. This package is interesting because it propagates the mapping of gene to terms to parent terms, and it possesses a pannel of enrichment tests and decorrelation methods. Expert users should be able to use information from the topAnatObject to test enrichment with other packages than topGO.
}
\examples{
{
bgee <- Bgee$new(species="Bos_taurus", dataType="rna_seq")
myTopAnatData <- loadTopAnatData(bgee, stage="UBERON:0000092")
geneList <- as.factor(c(rep(0, times=85), rep(1, times=15)))
names(geneList) <- c("ENSBTAG00000000011","ENSBTAG00000000014","ENSBTAG00000000016",
                     "ENSBTAG00000000026","ENSBTAG00000000039","ENSBTAG00000000040",
                     "ENSBTAG00000000042","ENSBTAG00000000050","ENSBTAG00000000056",
                     "ENSBTAG00000000064","ENSBTAG00000000067","ENSBTAG00000000071",
                     "ENSBTAG00000000072","ENSBTAG00000000080","ENSBTAG00000000081",
                     "ENSBTAG00000000084","ENSBTAG00000000091","ENSBTAG00000000099",
                     "ENSBTAG00000000111","ENSBTAG00000000123","ENSBTAG00000000132",
                     "ENSBTAG00000000153","ENSBTAG00000000162","ENSBTAG00000000163",
                     "ENSBTAG00000000169","ENSBTAG00000000179","ENSBTAG00000000197",
                     "ENSBTAG00000000199","ENSBTAG00000000202","ENSBTAG00000000203",
                     "ENSBTAG00000000204","ENSBTAG00000000213","ENSBTAG00000000215",
                     "ENSBTAG00000000223","ENSBTAG00000000224","ENSBTAG00000000225",
                     "ENSBTAG00000000236","ENSBTAG00000000250","ENSBTAG00000000251",
                     "ENSBTAG00000000252","ENSBTAG00000000253","ENSBTAG00000000261",
                     "ENSBTAG00000000274","ENSBTAG00000000277","ENSBTAG00000000279",
                     "ENSBTAG00000000285","ENSBTAG00000000286","ENSBTAG00000000287",
                     "ENSBTAG00000000289","ENSBTAG00000000297","ENSBTAG00000000305",
                     "ENSBTAG00000000312","ENSBTAG00000000328","ENSBTAG00000000335",
                     "ENSBTAG00000000341","ENSBTAG00000000343","ENSBTAG00000000354",
                     "ENSBTAG00000000355","ENSBTAG00000000356","ENSBTAG00000000365",
                     "ENSBTAG00000000372","ENSBTAG00000000379","ENSBTAG00000000380",
                     "ENSBTAG00000000382","ENSBTAG00000000396","ENSBTAG00000000404",
                     "ENSBTAG00000000405","ENSBTAG00000000406","ENSBTAG00000000411",
                     "ENSBTAG00000000425","ENSBTAG00000000434","ENSBTAG00000000435",
                     "ENSBTAG00000000438","ENSBTAG00000000448","ENSBTAG00000000451",
                     "ENSBTAG00000000454","ENSBTAG00000000456","ENSBTAG00000000457",
                     "ENSBTAG00000000459","ENSBTAG00000000462","ENSBTAG00000000469",
                     "ENSBTAG00000000470","ENSBTAG00000000484","ENSBTAG00000000497",
                     "ENSBTAG00000000501","ENSBTAG00000009707","ENSBTAG00000026266",
                     "ENSBTAG00000021992","ENSBTAG00000005353","ENSBTAG00000005333",
                     "ENSBTAG00000006424","ENSBTAG00000026972","ENSBTAG00000010799",
                     "ENSBTAG00000010799","ENSBTAG00000014614","ENSBTAG00000014614",
                     "ENSBTAG00000045757","ENSBTAG00000046332","ENSBTAG00000046332",
                     "ENSBTAG00000008394")
myTopAnatObject <-  topAnat(myTopAnatData, geneList)
}

}
\author{
Julien Roux
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{geneList}
\alias{geneList}
\title{Example of gene list object used to run a topAnat enrichment test, created on June 2018.
The format of the gene list is the same as the gene list required to build a \code{topGOdata} object in the \code{topGO} package: a vector with background genes as names, and 0 or 1 values depending if a gene is in the foreground or not.
In this example the foreground genes are zebrafish genes with an annotated phenotype related to "pectoral fin", and the background is composed of all zebrafish Ensembl genes with an annotated phenotype from ZFIN.
The gene list was built using the biomaRt package, and the code used can be found in the vignette of the package.}
\format{
A named vector of factor values with 3005 elements. The factor levels are "0" for the 2858 genes in the background and "1" for the 147 genes in the foreground. Vector names are the Ensembl IDs of the zebrafish genes.
}
\usage{
data(geneList)
}
\description{
Example of gene list object used to run a topAnat enrichment test, created on June 2018.
The format of the gene list is the same as the gene list required to build a \code{topGOdata} object in the \code{topGO} package: a vector with background genes as names, and 0 or 1 values depending if a gene is in the foreground or not.
In this example the foreground genes are zebrafish genes with an annotated phenotype related to "pectoral fin", and the background is composed of all zebrafish Ensembl genes with an annotated phenotype from ZFIN.
The gene list was built using the biomaRt package, and the code used can be found in the vignette of the package.
}
\examples{
bgee <- Bgee$new(species = "Danio_rerio")
myTopAnatData <- loadTopAnatData(bgee)
data(geneList)
myTopAnatObject <-  topAnat(myTopAnatData, geneList)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/formatData.R
\name{formatData}
\alias{formatData}
\title{Format RNA-seq or Affymetrix data downloaded from Bgee.}
\usage{
formatData(myBgeeObject, data, stats = NULL, callType = "all")
}
\arguments{
\item{myBgeeObject}{A Reference Class Bgee object, notably specifying the targeted species and data type.}

\item{data}{A list of data frames including data from multiple experiments, or a data frame including data from a single experiment.}

\item{stats}{A character indicating what expression values should be used in the formatted data expressionSet object matrix.
\itemize{
\item{"rpkm" for bulk RNA-seq (Bgee release 13.2 and before)}
\item{"fpkm" for bulk RNA-seq (Bgee release 14.0 to 14.2)}
\item{"counts" for RNA-seq}
\item{"tpm" for bulk RNA-seq or full length single cell RNA-seq (Bgee release 14 and above)}
\item{"cpm" for droplet based single cell RNA-seq (Bgee release 15.2 and above)}
\item{"intensities" for Affymetrix microarrays}
}}

\item{callType}{A character indicating whether intensities should be displayed only for present (i.e., expressed) genes, present high quality genes, or all genes (default).
\itemize{
\item{"present"}
\item{"present high quality"}
\item{"all"}
}}
}
\value{
If data was a list of data frames from multiple experiments, returns a list of ExpressionSet objects. If data was a data frame from a single experiment, returns an ExpressionSet object.
}
\description{
This function formats the data downloaded with the getSampleRawData() function into an object of the Bioconductor "expressionSet" Class.
}
\examples{
{
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  dataMouseGSE43721 <- getData(bgee, experimentId = "GSE43721")
  dataMouseGSE43721.tpm <- formatData(bgee,
                                       dataMouseGSE43721,
                                       callType = "present",
                                       stats = "tpm")
}

}
\author{
Andrea Komljenovic and Julien Roux.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Bgee.R
\docType{class}
\name{Bgee-class}
\alias{Bgee-class}
\alias{Bgee}
\title{Bgee Reference Class}
\description{
This is used to specify information at the beginning of a BgeeDB working session, for example, the targeted species and data type. An object of this class is then passed as argument to other functions of the package to provide these informations. See examples in vignette.
}
\details{
Bgee (\url{https://www.bgee.org}) integrates different expression data types (RNA-seq, Affymetrix microarray, ESTs, and in-situ hybridizations) from multiple animal species. Expression patterns are based exclusively on curated "normal", healthy, expression data (e.g., no gene knock-out, no treatment, no disease), to provide a reference atlas of normal gene expression.
}
\section{Fields}{

\describe{
\item{\code{dataType}}{A vector of characters indicating data type(s) to be used. To be chosen among:
\itemize{
\item{"rna_seq: target only bulk RNA-seq data"}
\item{"sc_full_length: target only full length single-cell RNA-seq data"}
\item{"sc_droplet_based: target only droplet based single-cell RNA-seq data"}
\item{"affymetrix: target only Affymtrix microarray data"}
\item{"est: target only ESTs data"}
\item{"in_situ: target only In Situ data"}
}
By default all data type are included: \code{c("rna_seq","sc_full_length","sc_droplet_based","affymetrix","est","in_situ")}. For download of quantitative expression data, a single data type should be chosen among "rna_seq", "sc_full_length", sc_droplet_based" or "affymetrix".}

\item{\code{pathToData}}{Path to the directory where the data files are stored. By default the working directory is used. If many analyses are launched in parallel, please consider re-using the cached data files instead of redownlaoding them for each analysis.}

\item{\code{release}}{Bgee release number to download data from, in the form "Release.subrelease" or "Release_subrelease", e.g., "15.2" or 15_2". Work for release >=13.2. By default, the latest relase of Bgee is used.}

\item{\code{sendStats}}{A field specifying whether monitoring of users is performed for our internal usage statistics. This is useful to improve the settings of our servers and to get reliable usage statistics (e.g., when asking for funding for Bgee). No identification of the users is attempted, nor possible. Default to TRUE. This option can be set to FALSE, notably if all data files are in cache and that users want to be able to work offline.}

\item{\code{quantitativeData}}{A field specifying if a single type of quantitative expression data ("rna_seq", "sc_full_length", "sc_droplet_based" or "affymetrix") was specified and if it is available for targeted species, helping the package to know if it should proceed with the execution of getAnnotation() and getData() functions.}
}}


\examples{
{
 bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
 bgee <- Bgee$new(species = "Mus_musculus")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSampleProcessedData.R
\name{getSampleProcessedData}
\alias{getSampleProcessedData}
\title{Retrieve Bgee processed expression values for bulk RNA-seq, full length single-cell RNA-seq,
droplet based single-cell RNA-seq or Affymetrix data.}
\usage{
getSampleProcessedData(
  myBgeeObject,
  experimentId = NULL,
  sampleId = NULL,
  anatEntityId = NULL,
  stageId = NULL,
  cellTypeId = NULL,
  sex = NULL,
  strain = NULL,
  withDescendantAnatEntities = FALSE,
  withDescendantStages = FALSE,
  withDescendantCellTypes = FALSE
)
}
\arguments{
\item{myBgeeObject}{A Reference Class Bgee object, notably specifying the targeted species
and data type.}

\item{experimentId}{Filter allowing to specify one or more ArrayExpress or GEO accession, e.g.,
GSE43721. Default is NULL: takes all available experiments for targeted species and data type.}

\item{sampleId}{Filter allowing to specify one or more sample ID. Depending on the selected
datatype this sample IDs can correspond to Chip IDs (affymetrix) or RNA-Seq library IDs (rna_seq).
Default is NULL: takes all available samples for targeted species and data type.}

\item{anatEntityId}{Filter allowing to specify one or more anatomical entity IDs from the UBERON
ontology (http://uberon.github.io/). Default is NULL: takes all available anatomical entities for
targeted species and data type.}

\item{stageId}{Filter allowing to specify one or more developmental stage IDs from Developmental
Stage Ontology (https://github.com/obophenotype/developmental-stage-ontologies). Default is
NULL: takes all available developmental stages for targeted species and data type.}

\item{cellTypeId}{Filter specific to single cell datatype (sc_full_length) allowing to specify
one or more cell type IDs from the UBERON ontology (http://uberon.github.io/). Default is
NULL: takes all available cell types for targeted species and data type. Available for Bgee 15.0 and after}

\item{sex}{Filter allowing to specify one or more sexes. Default is
NULL: takes all available sexes for targeted species and data type. Available for Bgee 15.0 and after}

\item{strain}{Filter allowing to specify one or more strains. Default is
NULL: takes all available strains for targeted species and data type. Available for Bgee 15.0 and after}

\item{withDescendantAnatEntities}{Allows to filter on the selected anatEntityId and all its descendants.
This functionality is available for Bgee 15.0 release and after}

\item{withDescendantStages}{Allows to filter on the selected stageId and all its descendants.
This functionality is available for Bgee 15.0 release and after}

\item{withDescendantCellTypes}{Allows to filter on the selected cellTypeId and all its descendants.
This functionality is available for Bgee 15.0 release and after}
}
\value{
Return a dataframe containing all Bgee processed expression data from the selected species
and datatype using specified filters with operator AND.
}
\description{
This function loads the quantitative expression data and presence calls
for samples available from Bgee. These data are available for all droplet based single-cell
RNA-seq, full length single-cell RNA-seq, bulk RNA-seq and affymetrix samples present in Bgee.
It is possible to filter the processed expression data using experiment IDs, chip IDs (affymetrix)
or library IDs (RNA-seq), anatomical entity IDs from the UBERON ontology, developmental stage IDs,
cell-type IDs, sex and/or strain. Celltype, anatomical entity and stage IDs are normalized and come from
ontologies, allowing to retrieve data annotated with the selected term or with the term and all its descendants.
This option allows for instance to retrieve all processed expression values coming from the brain or
any subpart of the brain.
}
\examples{
{
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  dataMouseGSE43721 <- getSampleProcessedData(bgee, experimentId = "GSE43721")
  dataMouseVariousFilters <- getSampleProcessedData(bgee, experimentId = c("GSE43721", "GSE36026"), 
                             anatEntityId = c("UBERON:0002107", "UBERON:0000956", "UBERON:0002048"))
}

}
\author{
Julien Wollbrett, Andrea Komljenovic and Julien Roux.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listBgeeRelease.R
\name{listBgeeRelease}
\alias{listBgeeRelease}
\title{List Bgee releases available to use with BgeeDB package}
\usage{
listBgeeRelease(release = NULL)
}
\arguments{
\item{release}{A character specifying a targeted release number. In the form "Release.subrelease" or "Release_subrelease", e.g., "13.2" or 13_2". If not specified, all available releases are shown.}
}
\value{
A data frame with information on Bgee releases.
}
\description{
Returns information on available Bgee releases, the access URL for FTP and webservice, and the date of release
}
\examples{
{
 listBgeeRelease()
}

}
\author{
Julien Roux, Julien Wollbrett
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getAnnotation.R
\name{getAnnotation}
\alias{getAnnotation}
\title{Retrieve Bgee experiments annotation for targeted species and data type.}
\usage{
getAnnotation(myBgeeObject)
}
\arguments{
\item{myBgeeObject}{A Reference Class Bgee object, notably specifying the targeted species and data type.}
}
\value{
A list of two elements, including a data frame of the annotation of experiments for chosen species (field "experiment.annotation") and a data frame of the annotation of chips/libraries from these experiments (field "sample.annotation").
}
\description{
This function loads the annotation of experiments and samples of quantitative expression datasets (rna_seq, affymetrix, sc_full_length, sc_droplet_based) that are available from Bgee.
}
\examples{
{
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  myAnnotation <- getAnnotation(bgee)
}

}
\author{
Andrea Komljenovic and Julien Roux.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/loadTopAnatData.R
\name{loadTopAnatData}
\alias{loadTopAnatData}
\title{Retrieve data from Bgee to perform GO-like enrichment of anatomical terms, mapped to genes by expression patterns.}
\usage{
loadTopAnatData(
  myBgeeObject,
  callType = "presence",
  confidence = NULL,
  stage = NULL,
  timeout = 1800
)
}
\arguments{
\item{myBgeeObject}{An output object from Bgee$new().}

\item{callType}{A character of indicating the type of expression calls to be used for enrichment. Only calls for significant detection of expression are implemented so far ("presence"). Differential expression calls, based on differential expression analysis, might be implemented in the future.}

\item{confidence}{A character indicating if only high quality present calls should be
retrieved. For Bgee releases prior to 14, options are "all" (default) or "high_quality".
For Bgee release 14 and above, options are "silver" (default) and "gold".}

\item{stage}{A character indicating the targeted developmental stages for the analysis. Developmental stages can be chosen from the developmental stage ontology used in Bgee (available at \url{https://github.com/obophenotype/developmental-stage-ontologies}). If a stage is specified, the expression pattern mapped to this stage and all children developmental stages (substages) will be retrieved. Default is NULL, meaning that expression patterns of genes are retrieved regardless of the developmental stage displaying expression; this is equivalent to specifying stage="UBERON:0000104" (life cycle, the root of the stage ontology).
For information, the most useful stages (going no deeper than level 3 of the ontology) include:
\itemize{
\item{UBERON:0000068 (embryo stage)}
\itemize{
\item{UBERON:0000106 (zygote stage)}
\item{UBERON:0000107 (cleavage stage)}
\item{UBERON:0000108 (blastula stage)}
\item{UBERON:0000109 (gastrula stage)}
\item{UBERON:0000110 (neurula stage)}
\item{UBERON:0000111 (organogenesis stage)}
\item{UBERON:0007220 (late embryonic stage)}
\item{UBERON:0004707 (pharyngula stage)}
}
\item{UBERON:0000092 (post-embryonic stage)}
\itemize{
\item{UBERON:0000069 (larval stage)}
\item{UBERON:0000070 (pupal stage)}
\item{UBERON:0000066 (fully formed stage)}
}
}}

\item{timeout}{local timeout used when the function is run. It allows to modify the
timeout used by default in R to download files. If not provided the timeout will be
fixed at 1800 secondes. When the functions exits (either naturally or as the result
of an error) the download timeout go back to the original one used in the R session.}
}
\value{
A list of 4 elements:
\itemize{
\item{A \code{gene2anatomy} list, mapping genes to anatomical structures based on expression calls.}
\item{A \code{organ.names} data frame, with the name corresponding to UBERON IDs.}
\item{A \code{organ.relationships} list, giving the relationships between anatomical structures in the UBERON ontology (based on parent-child "is_a" and "part_of" relationships).}
\item{The Bgee class object thta was used to retrieve the data.}
}
}
\description{
This function loads a mapping from genes to anatomical structures based on calls of expression in anatomical structures. It also loads the structure of the anatomical ontology.
}
\details{
The expression calls come from Bgee (\url{http://bgee.org}), that integrates different expression data types (RNA-seq, Affymetrix microarray, ESTs, or in-situ hybridizations) from multiple animal species. Expression patterns are based exclusively on curated "normal", healthy, expression data (e.g., no gene knock-out, no treatment, no disease), to provide a reference atlas of normal gene expression. Anatomical structures are identified using IDs from the Uberon ontology (browsable at \url{http://www.ontobee.org/ontology/UBERON}). The mapping from genes to anatomical structures includes only the evidence of expression in these specific structures, and not the expression in their substructures (i.e., expression data are not propagated). The retrieval of propagated expression data might be implemented in the future, but meanwhile, it can be obtained using specialized packages such as topGO, see the \code{topAnat.R} function.
}
\examples{
{
  bgee <- Bgee$new(species = "Danio_rerio", dataType = "rna_seq")
  myTopAnatData <- loadTopAnatData(bgee)
}

}
\author{
Julien Roux, Julien Wollbrett
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deleteLocalData.R
\name{deleteLocalData}
\alias{deleteLocalData}
\title{Delete local data for the species of the reference class Bgee object}
\usage{
deleteLocalData(myBgeeObject, allDataTypes = FALSE)
}
\arguments{
\item{myBgeeObject}{A Reference Class Bgee object, notably specifying the targeted species}

\item{allDataTypes}{A boolean defining rather data of all datatypes from the selected species
should be deleted. If TRUE, the local sqlite database file is deleted. If FALSE, data
corresponding to the datatype of the \code{myBgeeObject} object will be deleted from the local
sqlite database.}
}
\description{
This function delete data present in the local database. It can delete data
rather from one datatype or all datatypes of one species.
}
\examples{
{
  bgee <- Bgee$new(species = "Mus_musculus", dataType = "rna_seq")
  data <- getData(bgee, experimentId = "SRP007359")
  deleteLocalData(bgee, allDataTypes = TRUE)
}

}
\author{
Julien Wollbrett
}
